@@ -7,14 +7,22 @@
 #include "AccessibleCaret.h"
 
 #include "AccessibleCaretLogger.h"
+#include "mozilla/Assertions.h"
+#include "mozilla/BuiltInStyleSheets.h"
+#include "mozilla/ErrorResult.h"
 #include "mozilla/FloatingPoint.h"
-#include "mozilla/Preferences.h"
+#include "mozilla/PresShell.h"
+#include "mozilla/StaticPrefs_layout.h"
+#include "mozilla/dom/Document.h"
+#include "mozilla/dom/ShadowRoot.h"
 #include "mozilla/ToString.h"
 #include "nsCanvasFrame.h"
 #include "nsCaret.h"
 #include "nsCSSFrameConstructor.h"
 #include "nsDOMTokenList.h"
+#include "nsGenericHTMLElement.h"
 #include "nsIFrame.h"
+#include "nsLayoutUtils.h"
 #include "nsPlaceholderFrame.h"
 
 namespace mozilla {
@@ -30,14 +38,8 @@ using namespace dom;
 
 NS_IMPL_ISUPPORTS(AccessibleCaret::DummyTouchListener, nsIDOMEventListener)
 
-float AccessibleCaret::sWidth = 0.0f;
-float AccessibleCaret::sHeight = 0.0f;
-float AccessibleCaret::sMarginLeft = 0.0f;
-float AccessibleCaret::sBarWidth = 0.0f;
-
-NS_NAMED_LITERAL_STRING(AccessibleCaret::sTextOverlayElementId, "text-overlay");
-NS_NAMED_LITERAL_STRING(AccessibleCaret::sCaretImageElementId, "image");
-NS_NAMED_LITERAL_STRING(AccessibleCaret::sSelectionBarElementId, "bar");
+static constexpr auto kTextOverlayElementId = u"text-overlay"_ns;
+static constexpr auto kCaretImageElementId = u"image"_ns;
 
 #define AC_PROCESS_ENUM_TO_STREAM(e) \
   case (e):                          \
@@ -62,7 +64,8 @@ std::ostream& operator<<(
   using PositionChangedResult = AccessibleCaret::PositionChangedResult;
   switch (aResult) {
     AC_PROCESS_ENUM_TO_STREAM(PositionChangedResult::NotChanged);
-    AC_PROCESS_ENUM_TO_STREAM(PositionChangedResult::Changed);
+    AC_PROCESS_ENUM_TO_STREAM(PositionChangedResult::Position);
+    AC_PROCESS_ENUM_TO_STREAM(PositionChangedResult::Zoom);
     AC_PROCESS_ENUM_TO_STREAM(PositionChangedResult::Invisible);
   }
   return aStream;
@@ -72,28 +75,13 @@ std::ostream& operator<<(
 // -----------------------------------------------------------------------------
 // Implementation of AccessibleCaret methods
 
-AccessibleCaret::AccessibleCaret(nsIPresShell* aPresShell)
+AccessibleCaret::AccessibleCaret(PresShell* aPresShell)
     : mPresShell(aPresShell) {
   // Check all resources required.
   if (mPresShell) {
-    MOZ_ASSERT(RootFrame());
     MOZ_ASSERT(mPresShell->GetDocument());
-    MOZ_ASSERT(mPresShell->GetCanvasFrame());
-    MOZ_ASSERT(mPresShell->GetCanvasFrame()->GetCustomContentContainer());
-
     InjectCaretElement(mPresShell->GetDocument());
   }
-
-  static bool prefsAdded = false;
-  if (!prefsAdded) {
-    Preferences::AddFloatVarCache(&sWidth, "layout.accessiblecaret.width");
-    Preferences::AddFloatVarCache(&sHeight, "layout.accessiblecaret.height");
-    Preferences::AddFloatVarCache(&sMarginLeft,
-                                  "layout.accessiblecaret.margin-left");
-    Preferences::AddFloatVarCache(&sBarWidth,
-                                  "layout.accessiblecaret.bar.width");
-    prefsAdded = true;
-  }
 }
 
 AccessibleCaret::~AccessibleCaret() {
@@ -102,16 +90,24 @@ AccessibleCaret::~AccessibleCaret() {
   }
 }
 
+dom::Element* AccessibleCaret::TextOverlayElement() const {
+  return mCaretElementHolder->Root()->GetElementById(kTextOverlayElementId);
+}
+
+dom::Element* AccessibleCaret::CaretImageElement() const {
+  return mCaretElementHolder->Root()->GetElementById(kCaretImageElementId);
+}
+
 void AccessibleCaret::SetAppearance(Appearance aAppearance) {
   if (mAppearance == aAppearance) {
     return;
   }
 
-  ErrorResult rv;
-  CaretElement()->ClassList()->Remove(AppearanceString(mAppearance), rv);
+  IgnoredErrorResult rv;
+  CaretElement().ClassList()->Remove(AppearanceString(mAppearance), rv);
   MOZ_ASSERT(!rv.Failed(), "Remove old appearance failed!");
 
-  CaretElement()->ClassList()->Add(AppearanceString(aAppearance), rv);
+  CaretElement().ClassList()->Add(AppearanceString(aAppearance), rv);
   MOZ_ASSERT(!rv.Failed(), "Add new appearance failed!");
 
   AC_LOG("%s: %s -> %s", __FUNCTION__, ToString(mAppearance).c_str(),
@@ -121,42 +117,28 @@ void AccessibleCaret::SetAppearance(Appearance aAppearance) {
 
   // Need to reset rect since the cached rect will be compared in SetPosition.
   if (mAppearance == Appearance::None) {
-    mImaginaryCaretRect = nsRect();
-    mZoomLevel = 0.0f;
+    ClearCachedData();
   }
 }
 
-void AccessibleCaret::SetSelectionBarEnabled(bool aEnabled) {
-  if (mSelectionBarEnabled == aEnabled) {
-    return;
-  }
-
-  AC_LOG("Set selection bar %s", aEnabled ? "Enabled" : "Disabled");
-
-  ErrorResult rv;
-  CaretElement()->ClassList()->Toggle(NS_LITERAL_STRING("no-bar"),
-                                      Optional<bool>(!aEnabled), rv);
-  MOZ_ASSERT(!rv.Failed());
-
-  mSelectionBarEnabled = aEnabled;
-}
-
-/* static */ nsAutoString AccessibleCaret::AppearanceString(
-    Appearance aAppearance) {
+/* static */
+nsAutoString AccessibleCaret::AppearanceString(Appearance aAppearance) {
   nsAutoString string;
   switch (aAppearance) {
     case Appearance::None:
+      string = u"none"_ns;
+      break;
     case Appearance::NormalNotShown:
-      string = NS_LITERAL_STRING("none");
+      string = u"hidden"_ns;
       break;
     case Appearance::Normal:
-      string = NS_LITERAL_STRING("normal");
+      string = u"normal"_ns;
       break;
     case Appearance::Right:
-      string = NS_LITERAL_STRING("right");
+      string = u"right"_ns;
       break;
     case Appearance::Left:
-      string = NS_LITERAL_STRING("left");
+      string = u"left"_ns;
       break;
   }
   return string;
@@ -170,9 +152,9 @@ bool AccessibleCaret::Intersects(const AccessibleCaret& aCaret) const {
   }
 
   nsRect rect =
-      nsLayoutUtils::GetRectRelativeToFrame(CaretElement(), RootFrame());
-  nsRect rhsRect =
-      nsLayoutUtils::GetRectRelativeToFrame(aCaret.CaretElement(), RootFrame());
+      nsLayoutUtils::GetRectRelativeToFrame(&CaretElement(), RootFrame());
+  nsRect rhsRect = nsLayoutUtils::GetRectRelativeToFrame(&aCaret.CaretElement(),
+                                                         RootFrame());
   return rect.Intersects(rhsRect);
 }
 
@@ -199,70 +181,64 @@ void AccessibleCaret::EnsureApzAware() {
   // If the caret element was cloned, the listener might have been lost. So
   // if that's the case we register a dummy listener if there isn't one on
   // the element already.
-  if (!CaretElement()->IsApzAware()) {
-    CaretElement()->AddEventListener(NS_LITERAL_STRING("touchstart"),
-                                     mDummyTouchListener, false);
+  if (!CaretElement().IsApzAware()) {
+    CaretElement().AddEventListener(u"touchstart"_ns, mDummyTouchListener,
+                                    false);
   }
 }
 
-void AccessibleCaret::InjectCaretElement(nsIDocument* aDocument) {
-  ErrorResult rv;
-  nsCOMPtr<Element> element = CreateCaretElement(aDocument);
-  mCaretElementHolder = aDocument->InsertAnonymousContent(*element, rv);
+bool AccessibleCaret::IsInPositionFixedSubtree() const {
+  return nsLayoutUtils::IsInPositionFixedSubtree(
+      mImaginaryCaretReferenceFrame.GetFrame());
+}
 
-  MOZ_ASSERT(!rv.Failed(), "Insert anonymous content should not fail!");
-  MOZ_ASSERT(mCaretElementHolder.get(), "We must have anonymous content!");
+void AccessibleCaret::InjectCaretElement(Document* aDocument) {
+  mCaretElementHolder = aDocument->InsertAnonymousContent(IgnoreErrors());
+  MOZ_RELEASE_ASSERT(mCaretElementHolder, "We must have anonymous content!");
 
-  // InsertAnonymousContent will clone the element to make an AnonymousContent.
-  // Since event listeners are not being cloned when cloning a node, we need to
-  // add the listener here.
+  CreateCaretElement();
   EnsureApzAware();
 }
 
-already_AddRefed<Element> AccessibleCaret::CreateCaretElement(
-    nsIDocument* aDocument) const {
+void AccessibleCaret::CreateCaretElement() const {
   // Content structure of AccessibleCaret
   // <div class="moz-accessiblecaret">  <- CaretElement()
-  //   <div id="text-overlay"           <- TextOverlayElement()
-  //   <div id="image">                 <- CaretImageElement()
-  //   <div id="bar">                   <- SelectionBarElement()
+  //   <#shadow-root>
+  //     <div id="text-overlay">          <- TextOverlayElement()
+  //     <div id="image">                 <- CaretImageElement()
 
-  ErrorResult rv;
-  nsCOMPtr<Element> parent = aDocument->CreateHTMLElement(nsGkAtoms::div);
-  parent->ClassList()->Add(NS_LITERAL_STRING("moz-accessiblecaret"), rv);
-  parent->ClassList()->Add(NS_LITERAL_STRING("none"), rv);
-  parent->ClassList()->Add(NS_LITERAL_STRING("no-bar"), rv);
+  constexpr bool kNotify = false;
 
-  auto CreateAndAppendChildElement =
-      [aDocument, &parent](const nsLiteralString& aElementId) {
-        nsCOMPtr<Element> child = aDocument->CreateHTMLElement(nsGkAtoms::div);
-        child->SetAttr(kNameSpaceID_None, nsGkAtoms::id, aElementId, true);
-        parent->AppendChildTo(child, false);
-      };
+  Element& host = CaretElement();
+  host.SetAttr(kNameSpaceID_None, nsGkAtoms::_class,
+               u"moz-accessiblecaret none"_ns, kNotify);
 
-  CreateAndAppendChildElement(sTextOverlayElementId);
-  CreateAndAppendChildElement(sCaretImageElementId);
-  CreateAndAppendChildElement(sSelectionBarElementId);
+  ShadowRoot* root = mCaretElementHolder->Root();
+  Document* doc = host.OwnerDoc();
+  root->AppendBuiltInStyleSheet(BuiltInStyleSheet::AccessibleCaret);
 
-  return parent.forget();
+  auto CreateAndAppendChildElement = [&](const nsLiteralString& aElementId) {
+    RefPtr<Element> child = doc->CreateHTMLElement(nsGkAtoms::div);
+    child->SetAttr(kNameSpaceID_None, nsGkAtoms::id, aElementId, kNotify);
+    mCaretElementHolder->Root()->AppendChildTo(child, kNotify, IgnoreErrors());
+  };
+
+  CreateAndAppendChildElement(kTextOverlayElementId);
+  CreateAndAppendChildElement(kCaretImageElementId);
 }
 
-void AccessibleCaret::RemoveCaretElement(nsIDocument* aDocument) {
-  CaretElement()->RemoveEventListener(NS_LITERAL_STRING("touchstart"),
-                                      mDummyTouchListener, false);
+void AccessibleCaret::RemoveCaretElement(Document* aDocument) {
+  CaretElement().RemoveEventListener(u"touchstart"_ns, mDummyTouchListener,
+                                     false);
 
-  if (nsIFrame* frame = CaretElement()->GetPrimaryFrame()) {
-    if (frame->HasAnyStateBits(NS_FRAME_OUT_OF_FLOW)) {
-      frame = frame->GetPlaceholderFrame();
-    }
-    nsAutoScriptBlocker scriptBlocker;
-    frame->GetParent()->RemoveFrame(nsIFrame::kPrincipalList, frame);
-  }
+  aDocument->RemoveAnonymousContent(*mCaretElementHolder);
+}
 
-  ErrorResult rv;
-  aDocument->RemoveAnonymousContent(*mCaretElementHolder, rv);
-  // It's OK rv is failed since nsCanvasFrame might not exists now.
-  rv.SuppressException();
+void AccessibleCaret::ClearCachedData() {
+  mImaginaryCaretRect = nsRect();
+  mImaginaryCaretRectInContainerFrame = nsRect();
+  mImaginaryCaretReferenceFrame = nullptr;
+  mZoomLevel = 0.0f;
 }
 
 AccessibleCaret::PositionChangedResult AccessibleCaret::SetPosition(
@@ -279,71 +255,83 @@ AccessibleCaret::PositionChangedResult AccessibleCaret::SetPosition(
 
   if (imaginaryCaretRectInFrame.IsEmpty()) {
     // Don't bother to set the caret position since it's invisible.
-    mImaginaryCaretRect = nsRect();
-    mZoomLevel = 0.0f;
+    ClearCachedData();
     return PositionChangedResult::Invisible;
   }
 
-  nsRect imaginaryCaretRect = imaginaryCaretRectInFrame;
-  nsLayoutUtils::TransformRect(aFrame, RootFrame(), imaginaryCaretRect);
-  float zoomLevel = GetZoomLevel();
-
-  if (imaginaryCaretRect.IsEqualEdges(mImaginaryCaretRect) &&
-      FuzzyEqualsMultiplicative(zoomLevel, mZoomLevel)) {
-    return PositionChangedResult::NotChanged;
-  }
-
-  mImaginaryCaretRect = imaginaryCaretRect;
-  mZoomLevel = zoomLevel;
-
-  // SetCaretElementStyle() requires the input rect relative to container frame.
+  // SetCaretElementStyle() requires the input rect relative to the custom
+  // content container frame.
   nsRect imaginaryCaretRectInContainerFrame = imaginaryCaretRectInFrame;
   nsLayoutUtils::TransformRect(aFrame, CustomContentContainerFrame(),
                                imaginaryCaretRectInContainerFrame);
+  const float zoomLevel = GetZoomLevel();
+  const bool isSamePosition = imaginaryCaretRectInContainerFrame.IsEqualEdges(
+      mImaginaryCaretRectInContainerFrame);
+  const bool isSameZoomLevel = FuzzyEqualsMultiplicative(zoomLevel, mZoomLevel);
+
+  // Always update cached mImaginaryCaretRect (relative to the root frame)
+  // because it can change when the caret is scrolled.
+  mImaginaryCaretRect = imaginaryCaretRectInFrame;
+  nsLayoutUtils::TransformRect(aFrame, RootFrame(), mImaginaryCaretRect);
+
+  if (isSamePosition && isSameZoomLevel) {
+    return PositionChangedResult::NotChanged;
+  }
+
+  mImaginaryCaretRectInContainerFrame = imaginaryCaretRectInContainerFrame;
+  mImaginaryCaretReferenceFrame = aFrame;
+  mZoomLevel = zoomLevel;
+
   SetCaretElementStyle(imaginaryCaretRectInContainerFrame, mZoomLevel);
 
-  return PositionChangedResult::Changed;
+  return isSamePosition ? PositionChangedResult::Zoom
+                        : PositionChangedResult::Position;
+}
+
+nsIFrame* AccessibleCaret::RootFrame() const {
+  return mPresShell->GetRootFrame();
 }
 
 nsIFrame* AccessibleCaret::CustomContentContainerFrame() const {
-  nsCanvasFrame* canvasFrame = mPresShell->GetCanvasFrame();
-  Element* container = canvasFrame->GetCustomContentContainer();
-  nsIFrame* containerFrame = container->GetPrimaryFrame();
-  return containerFrame;
+  Element* container = mPresShell->GetDocument()->GetCustomContentContainer();
+  return container->GetPrimaryFrame();
 }
 
 void AccessibleCaret::SetCaretElementStyle(const nsRect& aRect,
                                            float aZoomLevel) {
   nsPoint position = CaretElementPosition(aRect);
   nsAutoString styleStr;
-  styleStr.AppendPrintf(
-      "left: %dpx; top: %dpx; "
-      "width: ",
-      nsPresContext::AppUnitsToIntCSSPixels(position.x),
-      nsPresContext::AppUnitsToIntCSSPixels(position.y));
   // We can't use AppendPrintf here, because it does locale-specific
   // formatting of floating-point values.
-  styleStr.AppendFloat(sWidth / aZoomLevel);
-  styleStr.AppendLiteral("px; height: ");
-  styleStr.AppendFloat(sHeight / aZoomLevel);
+  styleStr.AppendLiteral("left: ");
+  styleStr.AppendFloat(nsPresContext::AppUnitsToFloatCSSPixels(position.x));
+  styleStr.AppendLiteral("px; top: ");
+  styleStr.AppendFloat(nsPresContext::AppUnitsToFloatCSSPixels(position.y));
+  styleStr.AppendLiteral("px; width: ");
+  styleStr.AppendFloat(StaticPrefs::layout_accessiblecaret_width() /
+                       aZoomLevel);
   styleStr.AppendLiteral("px; margin-left: ");
-  styleStr.AppendFloat(sMarginLeft / aZoomLevel);
-  styleStr.AppendLiteral("px");
+  styleStr.AppendFloat(StaticPrefs::layout_accessiblecaret_margin_left() /
+                       aZoomLevel);
+  styleStr.AppendLiteral("px; transition-duration: ");
+  styleStr.AppendFloat(
+      StaticPrefs::layout_accessiblecaret_transition_duration());
+  styleStr.AppendLiteral("ms");
 
-  CaretElement()->SetAttr(kNameSpaceID_None, nsGkAtoms::style, styleStr, true);
+  CaretElement().SetAttr(kNameSpaceID_None, nsGkAtoms::style, styleStr, true);
   AC_LOG("%s: %s", __FUNCTION__, NS_ConvertUTF16toUTF8(styleStr).get());
 
   // Set style string for children.
   SetTextOverlayElementStyle(aRect, aZoomLevel);
   SetCaretImageElementStyle(aRect, aZoomLevel);
-  SetSelectionBarElementStyle(aRect, aZoomLevel);
 }
 
 void AccessibleCaret::SetTextOverlayElementStyle(const nsRect& aRect,
                                                  float aZoomLevel) {
   nsAutoString styleStr;
-  styleStr.AppendPrintf("height: %dpx;",
-                        nsPresContext::AppUnitsToIntCSSPixels(aRect.height));
+  styleStr.AppendLiteral("height: ");
+  styleStr.AppendFloat(nsPresContext::AppUnitsToFloatCSSPixels(aRect.height));
+  styleStr.AppendLiteral("px;");
   TextOverlayElement()->SetAttr(kNameSpaceID_None, nsGkAtoms::style, styleStr,
                                 true);
   AC_LOG("%s: %s", __FUNCTION__, NS_ConvertUTF16toUTF8(styleStr).get());
@@ -352,28 +340,15 @@ void AccessibleCaret::SetTextOverlayElementStyle(const nsRect& aRect,
 void AccessibleCaret::SetCaretImageElementStyle(const nsRect& aRect,
                                                 float aZoomLevel) {
   nsAutoString styleStr;
-  styleStr.AppendPrintf("margin-top: %dpx;",
-                        nsPresContext::AppUnitsToIntCSSPixels(aRect.height));
+  styleStr.AppendLiteral("height: ");
+  styleStr.AppendFloat(StaticPrefs::layout_accessiblecaret_height() /
+                       aZoomLevel);
+  styleStr.AppendLiteral("px;");
   CaretImageElement()->SetAttr(kNameSpaceID_None, nsGkAtoms::style, styleStr,
                                true);
   AC_LOG("%s: %s", __FUNCTION__, NS_ConvertUTF16toUTF8(styleStr).get());
 }
 
-void AccessibleCaret::SetSelectionBarElementStyle(const nsRect& aRect,
-                                                  float aZoomLevel) {
-  nsAutoString styleStr;
-  styleStr.AppendPrintf("height: %dpx; width: ",
-                        nsPresContext::AppUnitsToIntCSSPixels(aRect.height));
-  // We can't use AppendPrintf here, because it does locale-specific
-  // formatting of floating-point values.
-  styleStr.AppendFloat(sBarWidth / aZoomLevel);
-  styleStr.AppendLiteral("px");
-
-  SelectionBarElement()->SetAttr(kNameSpaceID_None, nsGkAtoms::style, styleStr,
-                                 true);
-  AC_LOG("%s: %s", __FUNCTION__, NS_ConvertUTF16toUTF8(styleStr).get());
-}
-
 float AccessibleCaret::GetZoomLevel() {
   // Full zoom on desktop.
   float fullZoom = mPresShell->GetPresContext()->GetFullZoom();
