@@ -30,6 +30,7 @@ MiddleCroppingBlockFrame::MiddleCroppingBlockFrame(ComputedStyle* aStyle,
 MiddleCroppingBlockFrame::~MiddleCroppingBlockFrame() = default;
 
 void MiddleCroppingBlockFrame::UpdateDisplayedValue(const nsAString& aValue,
+                                                    bool aIsCropped,
                                                     bool aNotify) {
   auto* text = mTextNode.get();
   uint32_t oldLength = aNotify ? 0 : text->TextLength();
@@ -45,35 +46,40 @@ void MiddleCroppingBlockFrame::UpdateDisplayedValue(const nsAString& aValue,
       LinesBegin()->MarkDirty();
     }
   }
+  mCropped = aIsCropped;
 }
 
 void MiddleCroppingBlockFrame::UpdateDisplayedValueToUncroppedValue(
     bool aNotify) {
   nsAutoString value;
   GetUncroppedValue(value);
-  UpdateDisplayedValue(value, aNotify);
+  UpdateDisplayedValue(value, /* aIsCropped = */ false, aNotify);
 }
 
-nscoord MiddleCroppingBlockFrame::GetMinISize(gfxContext* aRenderingContext) {
-  nscoord result;
-  DISPLAY_MIN_INLINE_SIZE(this, result);
-
-  // Our min inline size is our pref inline size
-  result = GetPrefISize(aRenderingContext);
-  return result;
-}
-
-nscoord MiddleCroppingBlockFrame::GetPrefISize(gfxContext* aRenderingContext) {
-  nscoord result;
-  DISPLAY_PREF_INLINE_SIZE(this, result);
-
-  // Make sure we measure with the uncropped value.
-  if (mCachedPrefISize == NS_INTRINSIC_ISIZE_UNKNOWN) {
-    UpdateDisplayedValueToUncroppedValue(false);
+nscoord MiddleCroppingBlockFrame::IntrinsicISize(
+    const IntrinsicSizeInput& aInput, IntrinsicISizeType aType) {
+  auto* first = FirstContinuation();
+  if (this != first) {
+    return first->IntrinsicISize(aInput, aType);
   }
-
-  result = nsBlockFrame::GetPrefISize(aRenderingContext);
-  return result;
+  return mCachedIntrinsics.GetOrSet(*this, aType, aInput, [&] {
+    nsAutoString prevValue;
+    bool restoreOldValue = false;
+    if (mCropped) {
+      // Make sure we measure with the uncropped value, if we're currently
+      // cropped.
+      mTextNode->GetNodeValue(prevValue);
+      UpdateDisplayedValueToUncroppedValue(false);
+      restoreOldValue = true;
+    }
+    // Our min inline size is the same as our pref inline size, so we always
+    // delegate to nsBlockFrame's pref inline size.
+    const nscoord result = nsBlockFrame::PrefISize(aInput);
+    if (restoreOldValue) {
+      UpdateDisplayedValue(prevValue, /* aIsCropped = */ true, false);
+    }
+    return result;
+  });
 }
 
 bool MiddleCroppingBlockFrame::CropTextToWidth(gfxContext& aRenderingContext,
@@ -152,13 +158,13 @@ void MiddleCroppingBlockFrame::Reflow(nsPresContext* aPresContext,
   // Restore the uncropped value.
   nsAutoString value;
   GetUncroppedValue(value);
-  bool done = false;
+  bool cropped = false;
   while (true) {
-    UpdateDisplayedValue(value, false);  // update the text node
+    UpdateDisplayedValue(value, cropped, false);  // update the text node
     AddStateBits(NS_BLOCK_NEEDS_BIDI_RESOLUTION);
     LinesBegin()->MarkDirty();
     nsBlockFrame::Reflow(aPresContext, aDesiredSize, aReflowInput, aStatus);
-    if (done) {
+    if (cropped) {
       break;
     }
     nscoord currentICoord = aReflowInput.mLineLayout
@@ -173,9 +179,10 @@ void MiddleCroppingBlockFrame::Reflow(nsPresContext* aPresContext,
         aStatus.Reset();
         MarkSubtreeDirty();
         AddStateBits(NS_BLOCK_NEEDS_BIDI_RESOLUTION);
-        mCachedMinISize = NS_INTRINSIC_ISIZE_UNKNOWN;
-        mCachedPrefISize = NS_INTRINSIC_ISIZE_UNKNOWN;
-        done = true;
+        // FIXME(emilio): Why do we need to clear cached intrinsics, if they are
+        // always based off our uncropped value?
+        mCachedIntrinsics.Clear();
+        cropped = true;
         continue;
       }
     }
@@ -198,10 +205,9 @@ void MiddleCroppingBlockFrame::AppendAnonymousContentTo(
   aContent.AppendElement(mTextNode);
 }
 
-void MiddleCroppingBlockFrame::DestroyFrom(nsIFrame* aDestructRoot,
-                                           PostDestroyData& aPostDestroyData) {
-  aPostDestroyData.AddAnonymousContent(mTextNode.forget());
-  nsBlockFrame::DestroyFrom(aDestructRoot, aPostDestroyData);
+void MiddleCroppingBlockFrame::Destroy(DestroyContext& aContext) {
+  aContext.AddAnonymousContent(mTextNode.forget());
+  nsBlockFrame::Destroy(aContext);
 }
 
 }  // namespace mozilla
