@@ -74,6 +74,7 @@ class ScrollContainerFrame : public nsContainerFrame,
                              public nsIReflowCallback,
                              public nsIStatefulFrame {
  public:
+  using CSSPoint = mozilla::CSSPoint;
   using Element = dom::Element;
   using ScrollAnchorContainer = layout::ScrollAnchorContainer;
   using SnapTargetSet = nsTHashSet<RefPtr<nsIContent>>;
@@ -433,7 +434,7 @@ class ScrollContainerFrame : public nsContainerFrame,
    * will be exactly aScrollPosition at the end of the scroll animation unless
    * the SMOOTH_MSD animation is interrupted.
    */
-  void ScrollToCSSPixels(const CSSIntPoint& aScrollPosition,
+  void ScrollToCSSPixels(const CSSPoint& aScrollPosition,
                          ScrollMode aMode = ScrollMode::Instant);
 
   /**
@@ -453,6 +454,13 @@ class ScrollContainerFrame : public nsContainerFrame,
    */
   CSSIntPoint GetRoundedScrollPositionCSSPixels();
 
+  /**
+   * Returns the scroll position in CSS pixels.
+   */
+  CSSPoint GetScrollPositionCSSPixels() const {
+    return CSSPoint::FromAppUnits(GetScrollPosition());
+  }
+
   /**
    * Some platforms (OSX) may generate additional scrolling events even
    * after the user has stopped scrolling, simulating a momentum scrolling
@@ -479,7 +487,7 @@ class ScrollContainerFrame : public nsContainerFrame,
                 ScrollMomentum aMomentum = NOT_MOMENTUM,
                 ScrollSnapFlags aSnapFlags = ScrollSnapFlags::Disabled);
 
-  void ScrollByCSSPixels(const CSSIntPoint& aDelta,
+  void ScrollByCSSPixels(const CSSPoint& aDelta,
                          ScrollMode aMode = ScrollMode::Instant) {
     return ScrollByCSSPixelsInternal(aDelta, aMode);
   }
@@ -855,7 +863,7 @@ class ScrollContainerFrame : public nsContainerFrame,
                                             StyleScrollbarWidth);
 
   void ScrollByCSSPixelsInternal(
-      const CSSIntPoint& aDelta, ScrollMode aMode = ScrollMode::Instant,
+      const CSSPoint& aDelta, ScrollMode aMode = ScrollMode::Instant,
       // This ScrollByCSSPixels is mainly used for Element.scrollBy and
       // Window.scrollBy. An exception is the transmogrification of
       // ScrollToCSSPixels.
@@ -1020,6 +1028,8 @@ class ScrollContainerFrame : public nsContainerFrame,
 
   void AppendScrollUpdate(const ScrollPositionUpdate& aUpdate);
 
+  bool HasBeenScrolled() const { return mHasBeenScrolled; }
+
  protected:
   ScrollContainerFrame(ComputedStyle* aStyle, nsPresContext* aPresContext,
                        bool aIsRoot)
@@ -1083,7 +1093,7 @@ class ScrollContainerFrame : public nsContainerFrame,
   PhysicalAxes GetOverflowAxes() const;
 
   MOZ_CAN_RUN_SCRIPT nsresult FireScrollPortEvent();
-  void PostScrollEndEvent(bool aDelayed = false);
+  void PostScrollEndEvent();
   MOZ_CAN_RUN_SCRIPT void FireScrollEndEvent();
   void PostOverflowEvent();
 
@@ -1111,7 +1121,7 @@ class ScrollContainerFrame : public nsContainerFrame,
    */
   void CurPosAttributeChangedInternal(nsIContent*, bool aDoScroll = true);
 
-  void PostScrollEvent(bool aDelayed = false);
+  void PostScrollEvent();
   MOZ_CAN_RUN_SCRIPT void FireScrollEvent();
   void PostScrolledAreaEvent();
   MOZ_CAN_RUN_SCRIPT void FireScrolledAreaEvent();
@@ -1311,10 +1321,6 @@ class ScrollContainerFrame : public nsContainerFrame,
     mMayScheduleScrollAnimations = false;
   }
 
-  CSSPoint GetScrollPositionCSSPixels() const {
-    return CSSPoint::FromAppUnits(GetScrollPosition());
-  }
-
   static void RemoveDisplayPortCallback(nsITimer* aTimer, void* aClosure);
 
   // owning references to the nsIAnonymousContentCreator-built content
