@@ -12,81 +12,49 @@
 #include "StickyScrollContainer.h"
 
 #include "mozilla/OverflowChangedTracker.h"
+#include "mozilla/ScrollContainerFrame.h"
 #include "nsIFrame.h"
 #include "nsIFrameInlines.h"
-#include "nsIScrollableFrame.h"
 #include "nsLayoutUtils.h"
+#include "PresShell.h"
 
 namespace mozilla {
 
 NS_DECLARE_FRAME_PROPERTY_DELETABLE(StickyScrollContainerProperty,
                                     StickyScrollContainer)
 
-StickyScrollContainer::StickyScrollContainer(nsIScrollableFrame* aScrollFrame)
-    : mScrollFrame(aScrollFrame), mScrollPosition() {
-  mScrollFrame->AddScrollPositionListener(this);
+StickyScrollContainer::StickyScrollContainer(
+    ScrollContainerFrame* aScrollContainerFrame)
+    : mScrollContainerFrame(aScrollContainerFrame) {
+  mScrollContainerFrame->AddScrollPositionListener(this);
 }
 
 StickyScrollContainer::~StickyScrollContainer() {
-  mScrollFrame->RemoveScrollPositionListener(this);
+  mScrollContainerFrame->RemoveScrollPositionListener(this);
 }
 
 // static
 StickyScrollContainer* StickyScrollContainer::GetStickyScrollContainerForFrame(
     nsIFrame* aFrame) {
-  nsIScrollableFrame* scrollFrame = nsLayoutUtils::GetNearestScrollableFrame(
-      aFrame->GetParent(), nsLayoutUtils::SCROLLABLE_SAME_DOC |
-                               nsLayoutUtils::SCROLLABLE_STOP_AT_PAGE |
-                               nsLayoutUtils::SCROLLABLE_INCLUDE_HIDDEN);
-  if (!scrollFrame) {
+  ScrollContainerFrame* scrollContainerFrame =
+      nsLayoutUtils::GetNearestScrollContainerFrame(
+          aFrame->GetParent(), nsLayoutUtils::SCROLLABLE_SAME_DOC |
+                                   nsLayoutUtils::SCROLLABLE_STOP_AT_PAGE |
+                                   nsLayoutUtils::SCROLLABLE_INCLUDE_HIDDEN);
+  if (!scrollContainerFrame) {
     // We might not find any, for instance in the case of
     // <html style="position: fixed">
     return nullptr;
   }
-  nsIFrame* frame = do_QueryFrame(scrollFrame);
   StickyScrollContainer* s =
-      frame->GetProperty(StickyScrollContainerProperty());
+      scrollContainerFrame->GetProperty(StickyScrollContainerProperty());
   if (!s) {
-    s = new StickyScrollContainer(scrollFrame);
-    frame->SetProperty(StickyScrollContainerProperty(), s);
+    s = new StickyScrollContainer(scrollContainerFrame);
+    scrollContainerFrame->SetProperty(StickyScrollContainerProperty(), s);
   }
   return s;
 }
 
-// static
-void StickyScrollContainer::NotifyReparentedFrameAcrossScrollFrameBoundary(
-    nsIFrame* aFrame, nsIFrame* aOldParent) {
-  nsIScrollableFrame* oldScrollFrame = nsLayoutUtils::GetNearestScrollableFrame(
-      aOldParent, nsLayoutUtils::SCROLLABLE_SAME_DOC |
-                      nsLayoutUtils::SCROLLABLE_INCLUDE_HIDDEN);
-  if (!oldScrollFrame) {
-    // XXX maybe aFrame has sticky descendants that can be sticky now, but
-    // we aren't going to handle that.
-    return;
-  }
-
-  StickyScrollContainer* oldSSC =
-      static_cast<nsIFrame*>(do_QueryFrame(oldScrollFrame))
-          ->GetProperty(StickyScrollContainerProperty());
-  if (!oldSSC) {
-    // aOldParent had no sticky descendants, so aFrame doesn't have any sticky
-    // descendants, and we're done here.
-    return;
-  }
-
-  auto i = oldSSC->mFrames.Length();
-  while (i-- > 0) {
-    nsIFrame* f = oldSSC->mFrames[i];
-    StickyScrollContainer* newSSC = GetStickyScrollContainerForFrame(f);
-    if (newSSC != oldSSC) {
-      oldSSC->RemoveFrame(f);
-      if (newSSC) {
-        newSSC->AddFrame(f);
-      }
-    }
-  }
-}
-
 // static
 StickyScrollContainer*
 StickyScrollContainer::GetStickyScrollContainerForScrollFrame(
@@ -94,43 +62,46 @@ StickyScrollContainer::GetStickyScrollContainerForScrollFrame(
   return aFrame->GetProperty(StickyScrollContainerProperty());
 }
 
-static nscoord ComputeStickySideOffset(
-    Side aSide, const StyleRect<LengthPercentageOrAuto>& aOffset,
-    nscoord aPercentBasis) {
-  auto& side = aOffset.Get(aSide);
-  if (side.IsAuto()) {
+static nscoord ComputeStickySideOffset(Side aSide,
+                                       const nsStylePosition& aPosition,
+                                       nscoord aPercentBasis) {
+  // Guaranteed to resolve any use of anchor function as invalid.
+  const auto& side = aPosition.GetAnchorResolvedInset(
+      aSide, AnchorPosOffsetResolutionParams::UseCBFrameSize(
+                 {nullptr, StylePositionProperty::Sticky}));
+  if (side->IsAuto()) {
     return NS_AUTOOFFSET;
   }
-  return nsLayoutUtils::ComputeCBDependentValue(aPercentBasis, side);
+  return nsLayoutUtils::ComputeCBDependentValue(aPercentBasis,
+                                                side->AsLengthPercentage());
 }
 
 // static
 void StickyScrollContainer::ComputeStickyOffsets(nsIFrame* aFrame) {
-  nsIScrollableFrame* scrollableFrame =
-      nsLayoutUtils::GetNearestScrollableFrame(
+  ScrollContainerFrame* scrollContainerFrame =
+      nsLayoutUtils::GetNearestScrollContainerFrame(
           aFrame->GetParent(), nsLayoutUtils::SCROLLABLE_SAME_DOC |
                                    nsLayoutUtils::SCROLLABLE_INCLUDE_HIDDEN);
 
-  if (!scrollableFrame) {
+  if (!scrollContainerFrame) {
     // Bail.
     return;
   }
 
-  nsSize scrollContainerSize = scrollableFrame->GetScrolledFrame()
-                                   ->GetContentRectRelativeToSelf()
-                                   .Size();
+  nsSize scrollContainerSize =
+      scrollContainerFrame->GetScrolledFrameSizeAccountingForDynamicToolbar();
 
   nsMargin computedOffsets;
   const nsStylePosition* position = aFrame->StylePosition();
 
-  computedOffsets.left = ComputeStickySideOffset(eSideLeft, position->mOffset,
-                                                 scrollContainerSize.width);
-  computedOffsets.right = ComputeStickySideOffset(eSideRight, position->mOffset,
-                                                  scrollContainerSize.width);
-  computedOffsets.top = ComputeStickySideOffset(eSideTop, position->mOffset,
-                                                scrollContainerSize.height);
-  computedOffsets.bottom = ComputeStickySideOffset(
-      eSideBottom, position->mOffset, scrollContainerSize.height);
+  computedOffsets.left =
+      ComputeStickySideOffset(eSideLeft, *position, scrollContainerSize.width);
+  computedOffsets.right =
+      ComputeStickySideOffset(eSideRight, *position, scrollContainerSize.width);
+  computedOffsets.top =
+      ComputeStickySideOffset(eSideTop, *position, scrollContainerSize.height);
+  computedOffsets.bottom = ComputeStickySideOffset(eSideBottom, *position,
+                                                   scrollContainerSize.height);
 
   // Store the offset
   nsMargin* offsets = aFrame->GetProperty(nsIFrame::ComputedOffsetProperty());
@@ -142,9 +113,10 @@ void StickyScrollContainer::ComputeStickyOffsets(nsIFrame* aFrame) {
   }
 }
 
-static nscoord gUnboundedNegative = nscoord_MIN / 2;
-static nscoord gUnboundedExtent = nscoord_MAX;
-static nscoord gUnboundedPositive = gUnboundedNegative + gUnboundedExtent;
+static constexpr nscoord gUnboundedNegative = nscoord_MIN / 2;
+static constexpr nscoord gUnboundedExtent = nscoord_MAX;
+static constexpr nscoord gUnboundedPositive =
+    gUnboundedNegative + gUnboundedExtent;
 
 void StickyScrollContainer::ComputeStickyLimits(nsIFrame* aFrame,
                                                 nsRect* aStick,
@@ -165,7 +137,7 @@ void StickyScrollContainer::ComputeStickyLimits(nsIFrame* aFrame,
     return;
   }
 
-  nsIFrame* scrolledFrame = mScrollFrame->GetScrolledFrame();
+  nsIFrame* scrolledFrame = mScrollContainerFrame->GetScrolledFrame();
   nsIFrame* cbFrame = aFrame->GetContainingBlock();
   NS_ASSERTION(cbFrame == scrolledFrame ||
                    nsLayoutUtils::IsProperAncestorFrame(scrolledFrame, cbFrame),
@@ -185,20 +157,27 @@ void StickyScrollContainer::ComputeStickyLimits(nsIFrame* aFrame,
 
   // Containing block limits for the position of aFrame relative to its parent.
   // The margin box of the sticky element stays within the content box of the
-  // contaning-block element.
+  // containing-block element.
   if (cbFrame == scrolledFrame) {
     // cbFrame is the scrolledFrame, and it won't have continuations. Unlike the
-    // else clause, we consider scrollable overflow rect because and the union
-    // of its in-flow rects doesn't include the scrollable overflow area.
+    // else clause, we consider scrollable overflow rect because the union of
+    // its in-flow rects doesn't include the scrollable overflow area. We need
+    // to subtract the padding however, which _is_ included in the scrollable
+    // area, since we want the content box.
+    MOZ_ASSERT(cbFrame->GetUsedBorder() == nsMargin(),
+               "How did the ::-moz-scrolled-frame end up with border?");
     *aContain = cbFrame->ScrollableOverflowRectRelativeToSelf();
+    aContain->Deflate(cbFrame->GetUsedPadding());
     nsLayoutUtils::TransformRect(cbFrame, aFrame->GetParent(), *aContain);
   } else {
     *aContain = nsLayoutUtils::GetAllInFlowRectsUnion(
-        cbFrame, aFrame->GetParent(), nsLayoutUtils::RECTS_USE_CONTENT_BOX);
+        cbFrame, aFrame->GetParent(),
+        nsLayoutUtils::GetAllInFlowRectsFlag::UseContentBox);
   }
 
   nsRect marginRect = nsLayoutUtils::GetAllInFlowRectsUnion(
-      aFrame, aFrame->GetParent(), nsLayoutUtils::RECTS_USE_MARGIN_BOX);
+      aFrame, aFrame->GetParent(),
+      nsLayoutUtils::GetAllInFlowRectsFlag::UseMarginBoxWithAutoResolvedAsZero);
 
   // Deflate aContain by the difference between the union of aFrame's
   // continuations' margin boxes and the union of their border boxes, so that
@@ -219,7 +198,8 @@ void StickyScrollContainer::ComputeStickyLimits(nsIFrame* aFrame,
                        computedOffsets->top - sfOffset.y);
   }
 
-  nsSize sfSize = scrolledFrame->GetContentRectRelativeToSelf().Size();
+  nsSize sfSize =
+      mScrollContainerFrame->GetScrolledFrameSizeAccountingForDynamicToolbar();
 
   // Bottom
   if (computedOffsets->bottom != NS_AUTOOFFSET &&
@@ -367,7 +347,7 @@ void StickyScrollContainer::UpdatePositions(nsPoint aScrollPosition,
                                             nsIFrame* aSubtreeRoot) {
 #ifdef DEBUG
   {
-    nsIFrame* scrollFrameAsFrame = do_QueryFrame(mScrollFrame);
+    nsIFrame* scrollFrameAsFrame = do_QueryFrame(mScrollContainerFrame);
     NS_ASSERTION(!aSubtreeRoot || aSubtreeRoot == scrollFrameAsFrame,
                  "If reflowing, should be reflowing the scroll frame");
   }
@@ -376,22 +356,23 @@ void StickyScrollContainer::UpdatePositions(nsPoint aScrollPosition,
 
   OverflowChangedTracker oct;
   oct.SetSubtreeRoot(aSubtreeRoot);
-  for (nsTArray<nsIFrame*>::size_type i = 0; i < mFrames.Length(); i++) {
-    nsIFrame* f = mFrames[i];
+  // We need to position ancestors before children, so iter from shallowest. We
+  // don't use mFrames.IterFromShallowest() because we want to handle removal of
+  // non-first continuations while we iterate.
+  for (size_t i = mFrames.Length(); i--;) {
+    nsIFrame* f = mFrames.ElementAt(i);
     if (!nsLayoutUtils::IsFirstContinuationOrIBSplitSibling(f)) {
-      // This frame was added in nsIFrame::Init before we knew it wasn't
-      // the first ib-split-sibling.
+      // This frame was added in nsIFrame::DidSetComputedStyle before we knew it
+      // wasn't the first ib-split-sibling.
       mFrames.RemoveElementAt(i);
-      --i;
       continue;
     }
-
     if (aSubtreeRoot) {
       // Reflowing the scroll frame, so recompute offsets.
       ComputeStickyOffsets(f);
     }
     // mFrames will only contain first continuations, because we filter in
-    // nsIFrame::Init.
+    // nsIFrame::DidSetComputedStyle.
     PositionContinuations(f);
 
     f = f->GetParent();
@@ -411,4 +392,10 @@ void StickyScrollContainer::ScrollPositionDidChange(nscoord aX, nscoord aY) {
   UpdatePositions(nsPoint(aX, aY), nullptr);
 }
 
+void StickyScrollContainer::MarkFramesForReflow() {
+  PresShell* ps = mScrollContainerFrame->PresShell();
+  for (nsIFrame* frame : mFrames.IterFromShallowest()) {
+    ps->FrameNeedsReflow(frame, IntrinsicDirty::None, NS_FRAME_IS_DIRTY);
+  }
+}
 }  // namespace mozilla
