@@ -188,10 +188,12 @@ nsDisplayWrapList* ViewportFrame::BuildDisplayListForTopLayer(
     // the normal path.
     if (frame->StyleDisplay()->mTopLayer == StyleTopLayer::None) {
       MOZ_ASSERT(!aBuilder->IsForPainting() ||
+                 !elem->State().HasState(dom::ElementState::FULLSCREEN) ||
                  !ShouldInTopLayerForFullscreen(elem));
       continue;
     }
-    MOZ_ASSERT(ShouldInTopLayerForFullscreen(elem));
+    MOZ_ASSERT_IF(elem->State().HasState(dom::ElementState::FULLSCREEN),
+                  ShouldInTopLayerForFullscreen(elem));
     // Inner SVG, MathML elements, as well as children of some XUL
     // elements are not allowed to be out-of-flow. They should not
     // be handled as top layer element here.
@@ -218,24 +220,22 @@ nsDisplayWrapList* ViewportFrame::BuildDisplayListForTopLayer(
   }
 
   if (dom::ViewTransition* vt = doc->GetActiveViewTransition()) {
-    if (dom::Element* root = vt->GetRoot()) {
+    if (dom::Element* root = vt->GetSnapshotContainingBlock()) {
       if (nsIFrame* frame = root->GetPrimaryFrame()) {
         MOZ_ASSERT(frame->StyleDisplay()->mTopLayer != StyleTopLayer::None,
-                   "ua.css should ensure this");
+                   "the snapshot containing block should ensure this");
         MOZ_ASSERT(frame->HasAnyStateBits(NS_FRAME_OUT_OF_FLOW));
         BuildDisplayListForTopLayerFrame(aBuilder, frame, &topLayerList);
       }
     }
   }
 
-  if (nsCanvasFrame* canvasFrame = PresShell()->GetCanvasFrame()) {
-    if (dom::Element* container = canvasFrame->GetCustomContentContainer()) {
-      if (nsIFrame* frame = container->GetPrimaryFrame()) {
-        MOZ_ASSERT(frame->StyleDisplay()->mTopLayer != StyleTopLayer::None,
-                   "ua.css should ensure this");
-        MOZ_ASSERT(frame->HasAnyStateBits(NS_FRAME_OUT_OF_FLOW));
-        BuildDisplayListForTopLayerFrame(aBuilder, frame, &topLayerList);
-      }
+  if (dom::Element* container = doc->GetCustomContentContainer()) {
+    if (nsIFrame* frame = container->GetPrimaryFrame()) {
+      MOZ_ASSERT(frame->StyleDisplay()->mTopLayer != StyleTopLayer::None,
+                 "ua.css should ensure this");
+      MOZ_ASSERT(frame->HasAnyStateBits(NS_FRAME_OUT_OF_FLOW));
+      BuildDisplayListForTopLayerFrame(aBuilder, frame, &topLayerList);
     }
   }
   if (topLayerList.IsEmpty()) {
