@@ -9,14 +9,19 @@
 #ifndef nsGridContainerFrame_h___
 #define nsGridContainerFrame_h___
 
+#include "mozilla/CSSOrderAwareFrameIterator.h"
+#include "mozilla/IntrinsicISizesCache.h"
 #include "mozilla/Maybe.h"
-#include "mozilla/TypeTraits.h"
+#include "mozilla/HashTable.h"
+#include "nsAtomHashKeys.h"
 #include "nsContainerFrame.h"
-#include "nsHashKeys.h"
-#include "nsTHashtable.h"
+#include "nsILineIterator.h"
 
 namespace mozilla {
 class PresShell;
+namespace dom {
+class Grid;
+}
 }  // namespace mozilla
 
 /**
@@ -28,14 +33,6 @@ nsContainerFrame* NS_NewGridContainerFrame(mozilla::PresShell* aPresShell,
 
 namespace mozilla {
 
-// Forward-declare typedefs for grid item iterator helper-class:
-template <typename Iterator>
-class CSSOrderAwareFrameIteratorT;
-typedef CSSOrderAwareFrameIteratorT<nsFrameList::iterator>
-    CSSOrderAwareFrameIterator;
-typedef CSSOrderAwareFrameIteratorT<nsFrameList::reverse_iterator>
-    ReverseCSSOrderAwareFrameIterator;
-
 /**
  * The number of implicit / explicit tracks and their sizes.
  */
@@ -45,16 +42,21 @@ struct ComputedGridTrackInfo {
       uint32_t aStartFragmentTrack, uint32_t aEndFragmentTrack,
       nsTArray<nscoord>&& aPositions, nsTArray<nscoord>&& aSizes,
       nsTArray<uint32_t>&& aStates, nsTArray<bool>&& aRemovedRepeatTracks,
-      uint32_t aRepeatFirstTrack)
+      uint32_t aRepeatFirstTrack,
+      nsTArray<nsTArray<StyleCustomIdent>>&& aResolvedLineNames,
+      bool aIsSubgrid, bool aIsMasonry)
       : mNumLeadingImplicitTracks(aNumLeadingImplicitTracks),
         mNumExplicitTracks(aNumExplicitTracks),
         mStartFragmentTrack(aStartFragmentTrack),
         mEndFragmentTrack(aEndFragmentTrack),
-        mPositions(aPositions),
-        mSizes(aSizes),
-        mStates(aStates),
-        mRemovedRepeatTracks(aRemovedRepeatTracks),
-        mRepeatFirstTrack(aRepeatFirstTrack) {}
+        mPositions(std::move(aPositions)),
+        mSizes(std::move(aSizes)),
+        mStates(std::move(aStates)),
+        mRemovedRepeatTracks(std::move(aRemovedRepeatTracks)),
+        mResolvedLineNames(std::move(aResolvedLineNames)),
+        mRepeatFirstTrack(aRepeatFirstTrack),
+        mIsSubgrid(aIsSubgrid),
+        mIsMasonry(aIsMasonry) {}
   uint32_t mNumLeadingImplicitTracks;
   uint32_t mNumExplicitTracks;
   uint32_t mStartFragmentTrack;
@@ -62,27 +64,38 @@ struct ComputedGridTrackInfo {
   nsTArray<nscoord> mPositions;
   nsTArray<nscoord> mSizes;
   nsTArray<uint32_t> mStates;
+  // Indicates if a track has been collapsed. This will be populated for each
+  // track in the repeat(auto-fit) and repeat(auto-fill), even if there are no
+  // collapsed tracks.
   nsTArray<bool> mRemovedRepeatTracks;
+  // Contains lists of all line name lists, including the name lists inside
+  // repeats. When a repeat(auto) track exists, the internal track names will
+  // appear once each in this array.
+  nsTArray<nsTArray<StyleCustomIdent>> mResolvedLineNames;
   uint32_t mRepeatFirstTrack;
+  bool mIsSubgrid;
+  bool mIsMasonry;
 };
 
 struct ComputedGridLineInfo {
-  explicit ComputedGridLineInfo(nsTArray<nsTArray<nsString>>&& aNames,
-                                const nsTArray<nsString>& aNamesBefore,
-                                const nsTArray<nsString>& aNamesAfter,
-                                nsTArray<nsString>&& aNamesFollowingRepeat)
-      : mNames(aNames),
-        mNamesBefore(aNamesBefore),
-        mNamesAfter(aNamesAfter),
-        mNamesFollowingRepeat(aNamesFollowingRepeat) {}
-  nsTArray<nsTArray<nsString>> mNames;
-  nsTArray<nsString> mNamesBefore;
-  nsTArray<nsString> mNamesAfter;
-  nsTArray<nsString> mNamesFollowingRepeat;
+  explicit ComputedGridLineInfo(
+      nsTArray<nsTArray<RefPtr<nsAtom>>>&& aNames,
+      const nsTArray<RefPtr<nsAtom>>& aNamesBefore,
+      const nsTArray<RefPtr<nsAtom>>& aNamesAfter,
+      nsTArray<RefPtr<nsAtom>>&& aNamesFollowingRepeat)
+      : mNames(std::move(aNames)),
+        mNamesBefore(aNamesBefore.Clone()),
+        mNamesAfter(aNamesAfter.Clone()),
+        mNamesFollowingRepeat(std::move(aNamesFollowingRepeat)) {}
+  nsTArray<nsTArray<RefPtr<nsAtom>>> mNames;
+  nsTArray<RefPtr<nsAtom>> mNamesBefore;
+  nsTArray<RefPtr<nsAtom>> mNamesAfter;
+  nsTArray<RefPtr<nsAtom>> mNamesFollowingRepeat;
 };
 }  // namespace mozilla
 
-class nsGridContainerFrame final : public nsContainerFrame {
+class nsGridContainerFrame final : public nsContainerFrame,
+                                   public nsILineIterator {
  public:
   NS_DECL_FRAMEARENA_HELPERS(nsGridContainerFrame)
   NS_DECL_QUERYFRAME
@@ -90,14 +103,13 @@ class nsGridContainerFrame final : public nsContainerFrame {
   using ComputedGridLineInfo = mozilla::ComputedGridLineInfo;
   using LogicalAxis = mozilla::LogicalAxis;
   using BaselineSharingGroup = mozilla::BaselineSharingGroup;
+  using NamedArea = mozilla::StyleNamedArea;
 
   template <typename T>
-  using PerBaseline = mozilla::EnumeratedArray<BaselineSharingGroup,
-                                               BaselineSharingGroup(2), T>;
+  using PerBaseline = mozilla::EnumeratedArray<BaselineSharingGroup, T, 2>;
 
   template <typename T>
-  using PerLogicalAxis =
-      mozilla::EnumeratedArray<LogicalAxis, LogicalAxis(2), T>;
+  using PerLogicalAxis = mozilla::EnumeratedArray<LogicalAxis, T, 2>;
 
   // nsIFrame overrides
   void Reflow(nsPresContext* aPresContext, ReflowOutput& aDesiredSize,
@@ -105,61 +117,58 @@ class nsGridContainerFrame final : public nsContainerFrame {
               nsReflowStatus& aStatus) override;
   void Init(nsIContent* aContent, nsContainerFrame* aParent,
             nsIFrame* aPrevInFlow) override;
-  nscoord GetMinISize(gfxContext* aRenderingContext) override;
-  nscoord GetPrefISize(gfxContext* aRenderingContext) override;
+  void DidSetComputedStyle(ComputedStyle* aOldStyle) override;
+
+  nscoord IntrinsicISize(const mozilla::IntrinsicSizeInput& aInput,
+                         mozilla::IntrinsicISizeType aType) override;
+
   void MarkIntrinsicISizesDirty() override;
-  bool IsFrameOfType(uint32_t aFlags) const override {
-    return nsContainerFrame::IsFrameOfType(
-        aFlags & ~nsIFrame::eCanContainOverflowContainers);
-  }
 
   void BuildDisplayList(nsDisplayListBuilder* aBuilder,
                         const nsDisplayListSet& aLists) override;
 
-  nscoord GetLogicalBaseline(mozilla::WritingMode aWM) const override {
-    if (HasAnyStateBits(NS_STATE_GRID_SYNTHESIZE_BASELINE)) {
-      // Return a baseline synthesized from our margin-box.
-      return nsContainerFrame::GetLogicalBaseline(aWM);
+  Maybe<nscoord> GetNaturalBaselineBOffset(
+      mozilla::WritingMode aWM, BaselineSharingGroup aBaselineGroup,
+      BaselineExportContext) const override {
+    if (StyleDisplay()->IsContainLayout() ||
+        HasAnyStateBits(NS_STATE_GRID_SYNTHESIZE_BASELINE)) {
+      return Nothing{};
     }
-    nscoord b;
-    GetBBaseline(BaselineSharingGroup::First, &b);
-    return b;
-  }
-
-  bool GetVerticalAlignBaseline(mozilla::WritingMode aWM,
-                                nscoord* aBaseline) const override {
-    return GetNaturalBaselineBOffset(aWM, BaselineSharingGroup::First,
-                                     aBaseline);
-  }
-
-  bool GetNaturalBaselineBOffset(mozilla::WritingMode aWM,
-                                 BaselineSharingGroup aBaselineGroup,
-                                 nscoord* aBaseline) const override {
-    if (HasAnyStateBits(NS_STATE_GRID_SYNTHESIZE_BASELINE)) {
-      return false;
-    }
-    return GetBBaseline(aBaselineGroup, aBaseline);
+    return mozilla::Some(GetBBaseline(aBaselineGroup));
   }
 
 #ifdef DEBUG_FRAME_DUMP
   nsresult GetFrameName(nsAString& aResult) const override;
+  void ExtraContainerFrameInfo(nsACString& aTo,
+                               bool aListOnlyDeterministic) const override;
 #endif
 
   // nsContainerFrame overrides
   bool DrainSelfOverflowList() override;
-  void AppendFrames(ChildListID aListID, nsFrameList& aFrameList) override;
+  void AppendFrames(ChildListID aListID, nsFrameList&& aFrameList) override;
   void InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
-                    nsFrameList& aFrameList) override;
-  void RemoveFrame(ChildListID aListID, nsIFrame* aOldFrame) override;
-  uint16_t CSSAlignmentForAbsPosChild(
-      const ReflowInput& aChildRI,
-      mozilla::LogicalAxis aLogicalAxis) const override;
+                    const nsLineList::iterator* aPrevFrameLine,
+                    nsFrameList&& aFrameList) override;
+  void RemoveFrame(DestroyContext&, ChildListID, nsIFrame*) override;
 
 #ifdef DEBUG
   void SetInitialChildList(ChildListID aListID,
-                           nsFrameList& aChildList) override;
+                           nsFrameList&& aChildList) override;
 #endif
 
+  bool CanProvideLineIterator() const final { return true; }
+  nsILineIterator* GetLineIterator() final { return this; }
+  int32_t GetNumLines() const final;
+  bool IsLineIteratorFlowRTL() final;
+  mozilla::Result<LineInfo, nsresult> GetLine(int32_t aLineNumber) final;
+  int32_t FindLineContaining(nsIFrame* aFrame, int32_t aStartLine = 0) final;
+  NS_IMETHOD FindFrameAt(int32_t aLineNumber, nsPoint aPos,
+                         nsIFrame** aFrameFound, bool* aPosIsBeforeFirstFrame,
+                         bool* aPosIsAfterLastFrame) final;
+  NS_IMETHOD CheckLineOrder(int32_t aLine, bool* aIsReordered,
+                            nsIFrame** aFirstVisual,
+                            nsIFrame** aLastVisual) final;
+
   /**
    * Return the containing block for aChild which MUST be an abs.pos. child
    * of a grid container and that container must have been reflowed.
@@ -201,30 +210,49 @@ class nsGridContainerFrame final : public nsContainerFrame {
     return info;
   }
 
-  typedef nsBaseHashtable<nsStringHashKey, mozilla::css::GridNamedArea,
-                          mozilla::css::GridNamedArea>
-      ImplicitNamedAreas;
+  /**
+   * This property is set by the creation of a dom::Grid object, and cleared
+   * during GC unlink. Since the Grid object manages the lifecycle, the property
+   * itself is set without a destructor. The property is also cleared whenever
+   * new grid computed info is generated during reflow, ensuring that we aren't
+   * holding a stale dom::Grid object.
+   */
+  NS_DECLARE_FRAME_PROPERTY_WITHOUT_DTOR(GridFragmentInfo, mozilla::dom::Grid)
+  mozilla::dom::Grid* GetGridFragmentInfo() {
+    return GetProperty(GridFragmentInfo());
+  }
+
+  using ImplicitNamedAreas =
+      mozilla::HashMap<mozilla::AtomHashKey, NamedArea, mozilla::AtomHashKey>;
   NS_DECLARE_FRAME_PROPERTY_DELETABLE(ImplicitNamedAreasProperty,
                                       ImplicitNamedAreas)
   ImplicitNamedAreas* GetImplicitNamedAreas() const {
     return GetProperty(ImplicitNamedAreasProperty());
   }
 
-  typedef nsTArray<mozilla::css::GridNamedArea> ExplicitNamedAreas;
+  using ExplicitNamedAreas = mozilla::StyleOwnedSlice<NamedArea>;
   NS_DECLARE_FRAME_PROPERTY_DELETABLE(ExplicitNamedAreasProperty,
                                       ExplicitNamedAreas)
   ExplicitNamedAreas* GetExplicitNamedAreas() const {
     return GetProperty(ExplicitNamedAreasProperty());
   }
 
+  using nsContainerFrame::IsMasonry;
+
+  /** Return true if this frame has masonry layout in any axis. */
+  bool IsMasonry() const {
+    return HasAnyStateBits(NS_STATE_GRID_IS_ROW_MASONRY |
+                           NS_STATE_GRID_IS_COL_MASONRY);
+  }
+
   /** Return true if this frame is subgridded in its aAxis. */
-  bool IsSubgrid(mozilla::LogicalAxis aAxis) const {
-    return HasAnyStateBits(aAxis == mozilla::eLogicalAxisBlock
+  bool IsSubgrid(LogicalAxis aAxis) const {
+    return HasAnyStateBits(aAxis == mozilla::LogicalAxis::Block
                                ? NS_STATE_GRID_IS_ROW_SUBGRID
                                : NS_STATE_GRID_IS_COL_SUBGRID);
   }
-  bool IsColSubgrid() const { return IsSubgrid(mozilla::eLogicalAxisInline); }
-  bool IsRowSubgrid() const { return IsSubgrid(mozilla::eLogicalAxisBlock); }
+  bool IsColSubgrid() const { return IsSubgrid(mozilla::LogicalAxis::Inline); }
+  bool IsRowSubgrid() const { return IsSubgrid(mozilla::LogicalAxis::Block); }
   /** Return true if this frame is subgridded in any axis. */
   bool IsSubgrid() const {
     return HasAnyStateBits(NS_STATE_GRID_IS_ROW_SUBGRID |
@@ -232,8 +260,8 @@ class nsGridContainerFrame final : public nsContainerFrame {
   }
 
   /** Return true if this frame has an item that is subgridded in our aAxis. */
-  bool HasSubgridItems(mozilla::LogicalAxis aAxis) const {
-    return HasAnyStateBits(aAxis == mozilla::eLogicalAxisBlock
+  bool HasSubgridItems(LogicalAxis aAxis) const {
+    return HasAnyStateBits(aAxis == mozilla::LogicalAxis::Block
                                ? NS_STATE_GRID_HAS_ROW_SUBGRID_ITEM
                                : NS_STATE_GRID_HAS_COL_SUBGRID_ITEM);
   }
@@ -242,6 +270,28 @@ class nsGridContainerFrame final : public nsContainerFrame {
     return HasAnyStateBits(NS_STATE_GRID_HAS_ROW_SUBGRID_ITEM |
                            NS_STATE_GRID_HAS_COL_SUBGRID_ITEM);
   }
+  /**
+   * Return true if the grid item aChild should stretch in its aAxis (i.e. aAxis
+   * is in the aChild's writing-mode).
+   *
+   * Note: this method does *not* consider the grid item's aspect-ratio and
+   * natural size in the axis when the self-alignment value is 'normal' per
+   * https://drafts.csswg.org/css-grid/#grid-item-sizing
+   */
+  bool GridItemShouldStretch(const nsIFrame* aChild, LogicalAxis aAxis) const;
+
+  /**
+   * Returns true if aFrame forms an independent formatting context and hence
+   * should be inhibited from being a subgrid (i.e. if the used value of
+   * 'grid-template-{rows,columns}:subgrid' should be 'none').
+   * https://drafts.csswg.org/css-grid-2/#subgrid-listing
+   *
+   * (Note this only makes sense to call if aFrame is itself either a grid
+   * container frame or a wrapper frame for a grid container frame, e.g. a
+   * scroll container frame for a scrollable grid.  Having said that, this is
+   * technically safe to call on any non-null frame.)
+   */
+  static bool ShouldInhibitSubgridDueToIFC(const nsIFrame* aFrame);
 
   /**
    * Return a container grid frame for the supplied frame, if available.
@@ -257,6 +307,14 @@ class nsGridContainerFrame final : public nsContainerFrame {
   MOZ_CAN_RUN_SCRIPT_BOUNDARY
   static nsGridContainerFrame* GetGridFrameWithComputedInfo(nsIFrame* aFrame);
 
+  /**
+   * Callback for nsIFrame::MarkIntrinsicISizesDirty() on a grid item.
+   */
+  static void MarkCachedGridMeasurementsDirty(nsIFrame* aItemFrame);
+
+  class CachedBAxisMeasurement;
+  struct Subgrid;
+  struct UsedTrackSizes;
   struct TrackSize;
   struct GridItemInfo;
   struct GridReflowInput;
@@ -267,25 +325,25 @@ class nsGridContainerFrame final : public nsContainerFrame {
     bool mIsInEdgeTrack;
   };
 
+  /** Return our parent grid container; |this| MUST be a subgrid. */
+  nsGridContainerFrame* ParentGridContainerForSubgrid() const;
+
+  // https://drafts.csswg.org/css-sizing/#constraints
+  enum class SizingConstraint {
+    MinContent,   // sizing under min-content constraint
+    MaxContent,   // sizing under max-content constraint
+    NoConstraint  // no constraint, used during Reflow
+  };
+
  protected:
-  static const uint32_t kAutoLine;
-  // The maximum line number, in the zero-based translated grid.
-  static const uint32_t kTranslatedMaxLine;
-  typedef mozilla::LogicalPoint LogicalPoint;
   typedef mozilla::LogicalRect LogicalRect;
-  typedef mozilla::LogicalSize LogicalSize;
-  typedef mozilla::CSSOrderAwareFrameIterator CSSOrderAwareFrameIterator;
-  typedef mozilla::ReverseCSSOrderAwareFrameIterator
-      ReverseCSSOrderAwareFrameIterator;
   typedef mozilla::WritingMode WritingMode;
-  typedef mozilla::css::GridNamedArea GridNamedArea;
-  typedef mozilla::layout::AutoFrameListPtr AutoFrameListPtr;
-  typedef nsLayoutUtils::IntrinsicISizeType IntrinsicISizeType;
   struct Grid;
   struct GridArea;
   class LineNameMap;
   struct LineRange;
   struct SharedGridData;
+  struct SubgridFallbackTrackSizingFunctions;
   struct TrackSizingFunctions;
   struct Tracks;
   struct TranslatedLineRange;
@@ -293,9 +351,7 @@ class nsGridContainerFrame final : public nsContainerFrame {
       mozilla::PresShell* aPresShell, ComputedStyle* aStyle);
   explicit nsGridContainerFrame(ComputedStyle* aStyle,
                                 nsPresContext* aPresContext)
-      : nsContainerFrame(aStyle, aPresContext, kClassID),
-        mCachedMinISize(NS_INTRINSIC_ISIZE_UNKNOWN),
-        mCachedPrefISize(NS_INTRINSIC_ISIZE_UNKNOWN) {
+      : nsContainerFrame(aStyle, aPresContext, kClassID) {
     for (auto& perAxisBaseline : mBaseline) {
       for (auto& baseline : perAxisBaseline) {
         baseline = NS_INTRINSIC_ISIZE_UNKNOWN;
@@ -310,41 +366,35 @@ class nsGridContainerFrame final : public nsContainerFrame {
    * property when needed, as a ImplicitNamedAreas* value.
    */
   void InitImplicitNamedAreas(const nsStylePosition* aStyle);
-  void AddImplicitNamedAreas(
-      const nsTArray<nsTArray<nsString>>& aLineNameLists);
+
+  using LineNameList =
+      const mozilla::StyleOwnedSlice<mozilla::StyleCustomIdent>;
+  void AddImplicitNamedAreas(mozilla::Span<LineNameList>);
+  using StyleLineNameListValue =
+      const mozilla::StyleGenericLineNameListValue<mozilla::StyleInteger>;
+  void AddImplicitNamedAreas(mozilla::Span<StyleLineNameListValue>);
 
   /**
    * Reflow and place our children.
    * @return the consumed size of all of this grid container's continuations
    *         so far including this frame
    */
-  nscoord ReflowChildren(GridReflowInput& aState,
+  nscoord ReflowChildren(GridReflowInput& aGridRI,
                          const LogicalRect& aContentArea,
+                         const nsSize& aContainerSize,
                          ReflowOutput& aDesiredSize, nsReflowStatus& aStatus);
 
   /**
-   * Helper for GetMinISize / GetPrefISize.
+   * Helper to implement IntrinsicISize().
    */
-  nscoord IntrinsicISize(gfxContext* aRenderingContext,
-                         IntrinsicISizeType aConstraint);
+  nscoord ComputeIntrinsicISize(const mozilla::IntrinsicSizeInput& aInput,
+                                mozilla::IntrinsicISizeType aType);
 
-  // Helper for AppendFrames / InsertFrames.
-  void NoteNewChildren(ChildListID aListID, const nsFrameList& aFrameList);
-
-  // Helper to move child frames into the kOverflowList.
-  void MergeSortedOverflow(nsFrameList& aList);
-  // Helper to move child frames into the kExcessOverflowContainersList:.
-  void MergeSortedExcessOverflowContainers(nsFrameList& aList);
-
-  bool GetBBaseline(BaselineSharingGroup aBaselineGroup,
-                    nscoord* aResult) const {
-    *aResult = mBaseline[mozilla::eLogicalAxisBlock][aBaselineGroup];
-    return true;
+  nscoord GetBBaseline(BaselineSharingGroup aBaselineGroup) const {
+    return mBaseline[mozilla::LogicalAxis::Block][aBaselineGroup];
   }
-  bool GetIBaseline(BaselineSharingGroup aBaselineGroup,
-                    nscoord* aResult) const {
-    *aResult = mBaseline[mozilla::eLogicalAxisInline][aBaselineGroup];
-    return true;
+  nscoord GetIBaseline(BaselineSharingGroup aBaselineGroup) const {
+    return mBaseline[mozilla::LogicalAxis::Inline][aBaselineGroup];
   }
 
   /**
@@ -364,7 +414,7 @@ class nsGridContainerFrame final : public nsContainerFrame {
     eBoth = eFirst | eLast,
   };
   void CalculateBaselines(BaselineSet aBaselineSet,
-                          CSSOrderAwareFrameIterator* aIter,
+                          mozilla::CSSOrderAwareFrameIterator* aIter,
                           const nsTArray<GridItemInfo>* aGridItems,
                           const Tracks& aTracks, uint32_t aFragmentStartTrack,
                           uint32_t aFirstExcludedTrack, WritingMode aWM,
@@ -375,9 +425,8 @@ class nsGridContainerFrame final : public nsContainerFrame {
   /**
    * Synthesize a Grid container baseline for aGroup.
    */
-  nscoord SynthesizeBaseline(const FindItemInGridOrderResult& aItem,
-                             mozilla::LogicalAxis aAxis,
-                             BaselineSharingGroup aGroup,
+  nscoord SynthesizeBaseline(const FindItemInGridOrderResult& aGridOrderItem,
+                             LogicalAxis aAxis, BaselineSharingGroup aGroup,
                              const nsSize& aCBPhysicalSize, nscoord aCBSize,
                              WritingMode aCBWM);
   /**
@@ -387,9 +436,9 @@ class nsGridContainerFrame final : public nsContainerFrame {
    * axis as aMajor.  Pass zero if that's not the axis we're fragmenting in.
    */
   static FindItemInGridOrderResult FindFirstItemInGridOrder(
-      CSSOrderAwareFrameIterator& aIter,
-      const nsTArray<GridItemInfo>& aGridItems, LineRange GridArea::*aMajor,
-      LineRange GridArea::*aMinor, uint32_t aFragmentStartTrack);
+      mozilla::CSSOrderAwareFrameIterator& aIter,
+      const nsTArray<GridItemInfo>& aGridItems, LineRange GridArea::* aMajor,
+      LineRange GridArea::* aMinor, uint32_t aFragmentStartTrack);
   /**
    * Find the last item in Grid Order in this fragment.
    * @param aFragmentStartTrack is the first track in this fragment in the same
@@ -399,14 +448,24 @@ class nsGridContainerFrame final : public nsContainerFrame {
    * Pass the number of tracks if that's not the axis we're fragmenting in.
    */
   static FindItemInGridOrderResult FindLastItemInGridOrder(
-      ReverseCSSOrderAwareFrameIterator& aIter,
-      const nsTArray<GridItemInfo>& aGridItems, LineRange GridArea::*aMajor,
-      LineRange GridArea::*aMinor, uint32_t aFragmentStartTrack,
+      mozilla::ReverseCSSOrderAwareFrameIterator& aIter,
+      const nsTArray<GridItemInfo>& aGridItems, LineRange GridArea::* aMajor,
+      LineRange GridArea::* aMinor, uint32_t aFragmentStartTrack,
       uint32_t aFirstExcludedTrack);
 
-#ifdef DEBUG
-  void SanityCheckGridItemsBeforeReflow() const;
-#endif  // DEBUG
+  /**
+   * Update our NS_STATE_GRID_IS_COL/ROW_SUBGRID bits and related subgrid state
+   * on our entire continuation chain based on the current style.
+   * This is needed because grid-template-columns/rows style changes only
+   * trigger a reflow so we need to update this dynamically.
+   */
+  void UpdateSubgridFrameState();
+
+  /**
+   * Return the NS_STATE_GRID_IS_COL/ROW_SUBGRID and
+   * NS_STATE_GRID_IS_ROW/COL_MASONRY bits we ought to have.
+   */
+  nsFrameState ComputeSelfSubgridMasonryBits() const;
 
  private:
   // Helpers for ReflowChildren
@@ -435,10 +494,10 @@ class nsGridContainerFrame final : public nsContainerFrame {
   };
 
   mozilla::Maybe<nsGridContainerFrame::Fragmentainer> GetNearestFragmentainer(
-      const GridReflowInput& aState) const;
+      const GridReflowInput& aGridRI) const;
 
   // @return the consumed size of all continuations so far including this frame
-  nscoord ReflowInFragmentainer(GridReflowInput& aState,
+  nscoord ReflowInFragmentainer(GridReflowInput& aGridRI,
                                 const LogicalRect& aContentArea,
                                 ReflowOutput& aDesiredSize,
                                 nsReflowStatus& aStatus,
@@ -448,7 +507,7 @@ class nsGridContainerFrame final : public nsContainerFrame {
   // Helper for ReflowInFragmentainer
   // @return the consumed size of all continuations so far including this frame
   nscoord ReflowRowsInFragmentainer(
-      GridReflowInput& aState, const LogicalRect& aContentArea,
+      GridReflowInput& aGridRI, const LogicalRect& aContentArea,
       ReflowOutput& aDesiredSize, nsReflowStatus& aStatus,
       Fragmentainer& aFragmentainer, const nsSize& aContainerSize,
       const nsTArray<const GridItemInfo*>& aItems, uint32_t aStartRow,
@@ -459,25 +518,62 @@ class nsGridContainerFrame final : public nsContainerFrame {
                          nsSize aContainerSize,
                          const mozilla::Maybe<nscoord>& aStretchBSize,
                          const Fragmentainer* aFragmentainer,
-                         const GridReflowInput& aState,
+                         const GridReflowInput& aGridRI,
                          const LogicalRect& aContentArea,
                          ReflowOutput& aDesiredSize, nsReflowStatus& aStatus);
 
+  // Helper for Reflow. This is intended to be called *before* the first pass of
+  // CalculateTrackSizesForAxis() for the block-axis.
+  //
+  // @return The block-size that can be used to resolve row sizes in the first
+  // pass.
+  nscoord ComputeBSizeForResolvingRowSizes(
+      GridReflowInput& aGridRI, nscoord aComputedBSize,
+      const Maybe<nscoord>& aContainIntrinsicBSize) const;
+
+  // Helper for Reflow. This is intended to be called *after* the final call to
+  // CalculateTrackSizesForAxis() for the block-axis.
+  //
+  // @param aBSizeForResolvingRowSizes the definite block-size determined by
+  // ComputeBSizeForResolvingRowSizes() or after resolving row sizes in the
+  // first pass.
+  // @return The intrinsic content block-size that can be used with other
+  // logic in Reflow() to determine the content block-size.
+  nscoord ComputeIntrinsicContentBSize(
+      const GridReflowInput& aGridRI, nscoord aComputedBSize,
+      nscoord aBSizeForResolvingRowSizes,
+      const Maybe<nscoord>& aContainIntrinsicBSize) const;
+
   /**
-   * Cached values to optimize GetMinISize/GetPrefISize.
+   * Places and reflows items when we have masonry layout.
+   * It handles unconstrained reflow and also fragmentation when the row axis
+   * is the masonry axis.  ReflowInFragmentainer handles the case when we're
+   * fragmenting and our row axis is a grid axis and it handles masonry layout
+   * in the column axis in that case.
+   * @return the intrinsic size in the masonry axis
    */
-  nscoord mCachedMinISize;
-  nscoord mCachedPrefISize;
+  nscoord MasonryLayout(GridReflowInput& aGridRI,
+                        const LogicalRect& aContentArea,
+                        SizingConstraint aConstraint,
+                        ReflowOutput& aDesiredSize, nsReflowStatus& aStatus,
+                        Fragmentainer* aFragmentainer,
+                        const nsSize& aContainerSize);
+
+  // Return the stored UsedTrackSizes, if any.
+  UsedTrackSizes* GetUsedTrackSizes() const;
+
+  // Store the given TrackSizes in aAxis on a UsedTrackSizes frame property.
+  void StoreUsedTrackSizes(LogicalAxis aAxis,
+                           const nsTArray<TrackSize>& aSizes);
+
+  // The internal implementation for AddImplicitNamedAreas().
+  void AddImplicitNamedAreasInternal(LineNameList& aNameList,
+                                     ImplicitNamedAreas*& aAreas);
+
+  mozilla::IntrinsicISizesCache mCachedIntrinsicSizes;
 
   // Our baselines, one per BaselineSharingGroup per axis.
   PerLogicalAxis<PerBaseline<nscoord>> mBaseline;
-
-#ifdef DEBUG
-  // If true, NS_STATE_GRID_DID_PUSH_ITEMS may be set even though all pushed
-  // frames may have been removed.  This is used to suppress an assertion
-  // in case RemoveFrame removed all associated child frames.
-  bool mDidPushItemsBitMayLie{false};
-#endif
 };
 
 #endif /* nsGridContainerFrame_h___ */
