@@ -10,15 +10,17 @@
 #include "nsRubyTextContainerFrame.h"
 #include "nsRubyBaseFrame.h"
 #include "nsRubyTextFrame.h"
+#include "mozilla/ComputedStyle.h"
 #include "mozilla/DebugOnly.h"
 #include "mozilla/Maybe.h"
+#include "mozilla/PresShell.h"
 #include "mozilla/WritingModes.h"
 #include "nsLayoutUtils.h"
 #include "nsLineLayout.h"
 #include "nsPresContext.h"
-#include "nsStyleContext.h"
 #include "nsStyleStructInlines.h"
 #include "nsTextFrame.h"
+#include "gfxContext.h"
 #include "RubyUtils.h"
 
 using namespace mozilla;
@@ -30,24 +32,25 @@ using namespace mozilla::gfx;
 // =======================
 
 NS_QUERYFRAME_HEAD(nsRubyBaseContainerFrame)
-NS_QUERYFRAME_ENTRY(nsRubyBaseContainerFrame)
+  NS_QUERYFRAME_ENTRY(nsRubyBaseContainerFrame)
 NS_QUERYFRAME_TAIL_INHERITING(nsContainerFrame)
 
 NS_IMPL_FRAMEARENA_HELPERS(nsRubyBaseContainerFrame)
 
-nsContainerFrame* NS_NewRubyBaseContainerFrame(nsIPresShell* aPresShell,
-                                               nsStyleContext* aContext) {
-  return new (aPresShell) nsRubyBaseContainerFrame(aContext);
+nsContainerFrame* NS_NewRubyBaseContainerFrame(PresShell* aPresShell,
+                                               ComputedStyle* aStyle) {
+  return new (aPresShell)
+      nsRubyBaseContainerFrame(aStyle, aPresShell->GetPresContext());
 }
 
-  //----------------------------------------------------------------------
+//----------------------------------------------------------------------
 
-  // nsRubyBaseContainerFrame Method Implementations
-  // ===============================================
+// nsRubyBaseContainerFrame Method Implementations
+// ===============================================
 
 #ifdef DEBUG_FRAME_DUMP
 nsresult nsRubyBaseContainerFrame::GetFrameName(nsAString& aResult) const {
-  return MakeFrameName(NS_LITERAL_STRING("RubyBaseContainer"), aResult);
+  return MakeFrameName(u"RubyBaseContainer"_ns, aResult);
 }
 #endif
 
@@ -97,7 +100,7 @@ static void GetIsLineBreakAllowed(nsIFrame* aFrame, bool aIsLineBreakable,
                                   bool* aAllowInitialLineBreak,
                                   bool* aAllowLineBreak) {
   nsIFrame* parent = aFrame->GetParent();
-  bool lineBreakSuppressed = parent->StyleContext()->ShouldSuppressLineBreak();
+  bool lineBreakSuppressed = parent->Style()->ShouldSuppressLineBreak();
   // Allow line break between ruby bases when white-space allows,
   // we are not inside a nested ruby, and there is no span.
   bool allowLineBreak =
@@ -123,7 +126,7 @@ static void GetIsLineBreakAllowed(nsIFrame* aFrame, bool aIsLineBreakable,
  * happens across the boundary of those frames.
  */
 static nscoord CalculateColumnPrefISize(
-    gfxContext* aRenderingContext, const RubyColumnEnumerator& aEnumerator,
+    const IntrinsicSizeInput& aInput, const RubyColumnEnumerator& aEnumerator,
     nsIFrame::InlineIntrinsicISizeData* aBaseISizeData) {
   nscoord max = 0;
   uint32_t levelCount = aEnumerator.GetLevelCount();
@@ -140,7 +143,7 @@ static nscoord CalculateColumnPrefISize(
         // ruby text container frame.
         data.SetLineContainer(frame->GetParent());
       }
-      frame->AddInlinePrefISize(aRenderingContext, &data);
+      frame->AddInlinePrefISize(aInput, &data);
       MOZ_ASSERT(data.mPrevLines == 0, "Shouldn't have prev lines");
       max = std::max(max, data.mCurrentLine);
       if (i == 0) {
@@ -155,19 +158,20 @@ static nscoord CalculateColumnPrefISize(
 // FIXME Currently we use pref isize of ruby content frames for
 //       computing min isize of ruby frame, which may cause problem.
 //       See bug 1134945.
-/* virtual */ void nsRubyBaseContainerFrame::AddInlineMinISize(
-    gfxContext* aRenderingContext, nsIFrame::InlineMinISizeData* aData) {
+/* virtual */
+void nsRubyBaseContainerFrame::AddInlineMinISize(
+    const IntrinsicSizeInput& aInput, InlineMinISizeData* aData) {
   AutoRubyTextContainerArray textContainers(this);
 
   for (uint32_t i = 0, iend = textContainers.Length(); i < iend; i++) {
     if (textContainers[i]->IsSpanContainer()) {
       // Since spans are not breakable internally, use our pref isize
       // directly if there is any span.
-      nsIFrame::InlinePrefISizeData data;
+      InlinePrefISizeData data;
       data.SetLineContainer(aData->LineContainer());
       data.mSkipWhitespace = aData->mSkipWhitespace;
       data.mTrailingWhitespace = aData->mTrailingWhitespace;
-      AddInlinePrefISize(aRenderingContext, &data);
+      AddInlinePrefISize(aInput, &data);
       aData->mCurrentLine += data.mCurrentLine;
       if (data.mCurrentLine > 0) {
         aData->mAtStartOfLine = false;
@@ -190,15 +194,14 @@ static nscoord CalculateColumnPrefISize(
         nsIFrame* baseFrame = enumerator.GetFrameAtLevel(0);
         if (baseFrame) {
           gfxBreakPriority breakPriority = LineBreakBefore(
-              baseFrame, aRenderingContext->GetDrawTarget(), nullptr, nullptr);
+              baseFrame, aInput.mContext->GetDrawTarget(), nullptr, nullptr);
           if (breakPriority != gfxBreakPriority::eNoBreak) {
             aData->OptionallyBreak();
           }
         }
       }
       firstFrame = false;
-      nscoord isize =
-          CalculateColumnPrefISize(aRenderingContext, enumerator, aData);
+      nscoord isize = CalculateColumnPrefISize(aInput, enumerator, aData);
       aData->mCurrentLine += isize;
       if (isize > 0) {
         aData->mAtStartOfLine = false;
@@ -207,23 +210,25 @@ static nscoord CalculateColumnPrefISize(
   }
 }
 
-/* virtual */ void nsRubyBaseContainerFrame::AddInlinePrefISize(
-    gfxContext* aRenderingContext, nsIFrame::InlinePrefISizeData* aData) {
+/* virtual */
+void nsRubyBaseContainerFrame::AddInlinePrefISize(
+    const IntrinsicSizeInput& aInput, InlinePrefISizeData* aData) {
   AutoRubyTextContainerArray textContainers(this);
+  const IntrinsicSizeInput input(aInput.mContext, Nothing(), Nothing());
 
   nscoord sum = 0;
   for (nsIFrame* frame = this; frame; frame = frame->GetNextInFlow()) {
     RubyColumnEnumerator enumerator(
         static_cast<nsRubyBaseContainerFrame*>(frame), textContainers);
     for (; !enumerator.AtEnd(); enumerator.Next()) {
-      sum += CalculateColumnPrefISize(aRenderingContext, enumerator, aData);
+      sum += CalculateColumnPrefISize(input, enumerator, aData);
     }
   }
   for (uint32_t i = 0, iend = textContainers.Length(); i < iend; i++) {
     if (textContainers[i]->IsSpanContainer()) {
       nsIFrame* frame = textContainers[i]->PrincipalChildList().FirstChild();
-      nsIFrame::InlinePrefISizeData data;
-      frame->AddInlinePrefISize(aRenderingContext, &data);
+      InlinePrefISizeData data;
+      frame->AddInlinePrefISize(input, &data);
       MOZ_ASSERT(data.mPrevLines == 0, "Shouldn't have prev lines");
       sum = std::max(sum, data.mCurrentLine);
     }
@@ -231,32 +236,28 @@ static nscoord CalculateColumnPrefISize(
   aData->mCurrentLine += sum;
 }
 
-/* virtual */ bool nsRubyBaseContainerFrame::IsFrameOfType(
-    uint32_t aFlags) const {
-  if (aFlags & eSupportsCSSTransforms) {
-    return false;
-  }
-  return nsContainerFrame::IsFrameOfType(aFlags &
-                                         ~(nsIFrame::eLineParticipant));
-}
+/* virtual */
+bool nsRubyBaseContainerFrame::CanContinueTextRun() const { return true; }
 
-/* virtual */ bool nsRubyBaseContainerFrame::CanContinueTextRun() const {
-  return true;
-}
-
-/* virtual */ LogicalSize nsRubyBaseContainerFrame::ComputeSize(
+/* virtual */
+nsIFrame::SizeComputationResult nsRubyBaseContainerFrame::ComputeSize(
     gfxContext* aRenderingContext, WritingMode aWM, const LogicalSize& aCBSize,
     nscoord aAvailableISize, const LogicalSize& aMargin,
-    const LogicalSize& aBorder, const LogicalSize& aPadding,
+    const LogicalSize& aBorderPadding, const StyleSizeOverrides& aSizeOverrides,
     ComputeSizeFlags aFlags) {
   // Ruby base container frame is inline,
   // hence don't compute size before reflow.
-  return LogicalSize(aWM, NS_UNCONSTRAINEDSIZE, NS_UNCONSTRAINEDSIZE);
+  return {LogicalSize(aWM, NS_UNCONSTRAINEDSIZE, NS_UNCONSTRAINEDSIZE),
+          AspectRatioUsage::None};
 }
 
-/* virtual */ nscoord nsRubyBaseContainerFrame::GetLogicalBaseline(
-    WritingMode aWritingMode) const {
-  return mBaseline;
+Maybe<nscoord> nsRubyBaseContainerFrame::GetNaturalBaselineBOffset(
+    WritingMode aWM, BaselineSharingGroup aBaselineGroup,
+    BaselineExportContext) const {
+  if (aBaselineGroup == BaselineSharingGroup::Last) {
+    return Nothing{};
+  }
+  return Some(mBaseline);
 }
 
 struct nsRubyBaseContainerFrame::RubyReflowInput {
@@ -267,12 +268,13 @@ struct nsRubyBaseContainerFrame::RubyReflowInput {
   const nsTArray<UniquePtr<ReflowInput>>& mTextReflowInputs;
 };
 
-/* virtual */ void nsRubyBaseContainerFrame::Reflow(
-    nsPresContext* aPresContext, ReflowOutput& aDesiredSize,
-    const ReflowInput& aReflowInput, nsReflowStatus& aStatus) {
+/* virtual */
+void nsRubyBaseContainerFrame::Reflow(nsPresContext* aPresContext,
+                                      ReflowOutput& aDesiredSize,
+                                      const ReflowInput& aReflowInput,
+                                      nsReflowStatus& aStatus) {
   MarkInReflow();
   DO_GLOBAL_REFLOW_COUNT("nsRubyBaseContainerFrame");
-  DISPLAY_REFLOW(aPresContext, this, aReflowInput, aDesiredSize, aStatus);
   MOZ_ASSERT(aStatus.IsEmpty(), "Caller should pass a fresh reflow status!");
 
   if (!aReflowInput.mLineLayout) {
@@ -297,11 +299,11 @@ struct nsRubyBaseContainerFrame::RubyReflowInput {
   LogicalSize availSize(lineWM, aReflowInput.AvailableISize(),
                         aReflowInput.AvailableBSize());
 
-  // We have a reflow state and a line layout for each RTC.
+  // We have a reflow input and a line layout for each RTC.
   // They are conceptually the state of the RTCs, but we don't actually
   // reflow those RTCs in this code. These two arrays are holders of
-  // the reflow states and line layouts.
-  // Since there are pointers refer to reflow states and line layouts,
+  // the reflow inputs and line layouts.
+  // Since there are pointers refer to reflow inputs and line layouts,
   // it is necessary to guarantee that they won't be moved. For this
   // reason, they are wrapped in UniquePtr here.
   AutoTArray<UniquePtr<ReflowInput>, RTC_ARRAY_SIZE> reflowInputs;
@@ -324,7 +326,7 @@ struct nsRubyBaseContainerFrame::RubyReflowInput {
         availSize.ConvertTo(textContainer->GetWritingMode(), lineWM));
     reflowInputs.AppendElement(reflowInput);
     nsLineLayout* lineLayout =
-        new nsLineLayout(aPresContext, reflowInput->mFloatManager, reflowInput,
+        new nsLineLayout(aPresContext, reflowInput->mFloatManager, *reflowInput,
                          nullptr, aReflowInput.mLineLayout);
     lineLayout->SetSuppressLineWrap(true);
     lineLayouts.AppendElement(lineLayout);
@@ -332,7 +334,7 @@ struct nsRubyBaseContainerFrame::RubyReflowInput {
     // Line number is useless for ruby text
     // XXX nullptr here may cause problem, see comments for
     //     nsLineLayout::mBlockRI and nsLineLayout::AddFloat
-    lineLayout->Init(nullptr, reflowInput->CalcLineHeight(), -1);
+    lineLayout->Init(nullptr, reflowInput->GetLineHeight(), -1);
     reflowInput->mLineLayout = lineLayout;
 
     // Border and padding are suppressed on ruby text containers.
@@ -352,12 +354,13 @@ struct nsRubyBaseContainerFrame::RubyReflowInput {
   GetIsLineBreakAllowed(this, aReflowInput.mLineLayout->LineIsBreakable(),
                         &allowInitialLineBreak, &allowLineBreak);
 
-  nscoord isize = 0;
   // Reflow columns excluding any span
   RubyReflowInput reflowInput = {allowInitialLineBreak,
                                  allowLineBreak && !hasSpan, textContainers,
                                  aReflowInput, reflowInputs};
-  isize = ReflowColumns(reflowInput, aStatus);
+  aDesiredSize.BSize(lineWM) = 0;
+  aDesiredSize.SetBlockStartAscent(0);
+  nscoord isize = ReflowColumns(reflowInput, aDesiredSize, aStatus);
   DebugOnly<nscoord> lineSpanSize = aReflowInput.mLineLayout->EndSpan(this);
   aDesiredSize.ISize(lineWM) = isize;
   // When there are no frames inside the ruby base container, EndSpan
@@ -402,12 +405,18 @@ struct nsRubyBaseContainerFrame::RubyReflowInput {
     lineLayout->EndLineReflow();
   }
 
-  // Border and padding are suppressed on ruby base container,
-  // create a fake borderPadding for setting BSize.
-  WritingMode frameWM = aReflowInput.GetWritingMode();
-  LogicalMargin borderPadding(frameWM);
-  nsLayoutUtils::SetBSizeFromFontMetrics(this, aDesiredSize, borderPadding,
-                                         lineWM, frameWM);
+  // If this ruby base container is empty, size it as if there were
+  // an empty inline child inside.
+  if (mFrames.IsEmpty()) {
+    // Border and padding are suppressed on ruby base container, so we
+    // create a dummy zero-sized borderPadding for setting BSize.
+    WritingMode frameWM = aReflowInput.GetWritingMode();
+    LogicalMargin borderPadding(frameWM);
+    nsLayoutUtils::SetBSizeFromFontMetrics(this, aDesiredSize, borderPadding,
+                                           lineWM, frameWM);
+  }
+
+  ReflowAbsoluteFrames(aPresContext, aDesiredSize, aReflowInput, aStatus);
 }
 
 /**
@@ -425,7 +434,8 @@ struct MOZ_STACK_CLASS nsRubyBaseContainerFrame::PullFrameState {
 };
 
 nscoord nsRubyBaseContainerFrame::ReflowColumns(
-    const RubyReflowInput& aReflowInput, nsReflowStatus& aStatus) {
+    const RubyReflowInput& aReflowInput, ReflowOutput& aDesiredSize,
+    nsReflowStatus& aStatus) {
   nsLineLayout* lineLayout = aReflowInput.mBaseReflowInput.mLineLayout;
   const uint32_t rtcCount = aReflowInput.mTextContainers.Length();
   nscoord icoord = lineLayout->GetCurrentICoord();
@@ -439,7 +449,8 @@ nscoord nsRubyBaseContainerFrame::ReflowColumns(
   RubyColumnEnumerator e(this, aReflowInput.mTextContainers);
   for (; !e.AtEnd(); e.Next()) {
     e.GetColumn(column);
-    icoord += ReflowOneColumn(aReflowInput, columnIndex, column, reflowStatus);
+    icoord += ReflowOneColumn(aReflowInput, columnIndex, column, aDesiredSize,
+                              reflowStatus);
     if (!reflowStatus.IsInlineBreakBefore()) {
       columnIndex++;
     }
@@ -463,7 +474,8 @@ nscoord nsRubyBaseContainerFrame::ReflowColumns(
       // No more frames can be pulled.
       break;
     }
-    icoord += ReflowOneColumn(aReflowInput, columnIndex, column, reflowStatus);
+    icoord += ReflowOneColumn(aReflowInput, columnIndex, column, aDesiredSize,
+                              reflowStatus);
     if (!reflowStatus.IsInlineBreakBefore()) {
       columnIndex++;
     }
@@ -533,7 +545,8 @@ nscoord nsRubyBaseContainerFrame::ReflowColumns(
 
 nscoord nsRubyBaseContainerFrame::ReflowOneColumn(
     const RubyReflowInput& aReflowInput, uint32_t aColumnIndex,
-    const RubyColumn& aColumn, nsReflowStatus& aStatus) {
+    const RubyColumn& aColumn, ReflowOutput& aDesiredSize,
+    nsReflowStatus& aStatus) {
   const ReflowInput& baseReflowInput = aReflowInput.mBaseReflowInput;
   const auto& textReflowInputs = aReflowInput.mTextReflowInputs;
   nscoord istart = baseReflowInput.mLineLayout->GetCurrentICoord();
@@ -578,18 +591,23 @@ nscoord nsRubyBaseContainerFrame::ReflowOneColumn(
   for (uint32_t i = 0; i < rtcCount; i++) {
     nsRubyTextFrame* textFrame = aColumn.mTextFrames[i];
     if (textFrame) {
-      nsAutoString annotationText;
-      nsLayoutUtils::GetFrameTextContent(textFrame, annotationText);
-
-      // Per CSS Ruby spec, the content comparison for auto-hiding
-      // takes place prior to white spaces collapsing (white-space)
-      // and text transformation (text-transform), and ignores elements
-      // (considers only the textContent of the boxes). Which means
-      // using the content tree text comparison is correct.
-      if (annotationText.Equals(baseText)) {
-        textFrame->AddStateBits(NS_RUBY_TEXT_FRAME_AUTOHIDE);
+      bool isCollapsed = false;
+      if (textFrame->StyleVisibility()->mVisible == StyleVisibility::Collapse) {
+        isCollapsed = true;
       } else {
-        textFrame->RemoveStateBits(NS_RUBY_TEXT_FRAME_AUTOHIDE);
+        // Per CSS Ruby spec, the content comparison for auto-hiding
+        // takes place prior to white spaces collapsing (white-space)
+        // and text transformation (text-transform), and ignores elements
+        // (considers only the textContent of the boxes). Which means
+        // using the content tree text comparison is correct.
+        nsAutoString annotationText;
+        nsLayoutUtils::GetFrameTextContent(textFrame, annotationText);
+        isCollapsed = annotationText.Equals(baseText);
+      }
+      if (isCollapsed) {
+        textFrame->AddStateBits(NS_RUBY_TEXT_FRAME_COLLAPSED);
+      } else {
+        textFrame->RemoveStateBits(NS_RUBY_TEXT_FRAME_COLLAPSED);
       }
       RubyUtils::ClearReservedISize(textFrame);
 
@@ -617,8 +635,10 @@ nscoord nsRubyBaseContainerFrame::ReflowOneColumn(
     bool pushedFrame;
     nsReflowStatus reflowStatus;
     nsLineLayout* lineLayout = baseReflowInput.mLineLayout;
+    WritingMode lineWM = lineLayout->GetWritingMode();
     nscoord baseIStart = lineLayout->GetCurrentICoord();
-    lineLayout->ReflowFrame(aColumn.mBaseFrame, reflowStatus, nullptr,
+    ReflowOutput metrics(lineWM);
+    lineLayout->ReflowFrame(aColumn.mBaseFrame, reflowStatus, &metrics,
                             pushedFrame);
     if (MOZ_UNLIKELY(reflowStatus.IsInlineBreak() || pushedFrame)) {
       MOZ_ASSERT_UNREACHABLE(
@@ -629,6 +649,17 @@ nscoord nsRubyBaseContainerFrame::ReflowOneColumn(
     }
     nscoord baseISize = lineLayout->GetCurrentICoord() - baseIStart;
     columnISize = std::max(columnISize, baseISize);
+    // Calculate ascent & descent of the base frame and update desired
+    // size of this base container accordingly.
+    nscoord oldAscent = aDesiredSize.BlockStartAscent();
+    nscoord oldDescent = aDesiredSize.BSize(lineWM) - oldAscent;
+    nscoord baseAscent = metrics.BlockStartAscent();
+    nscoord baseDesent = metrics.BSize(lineWM) - baseAscent;
+    LogicalMargin margin = aColumn.mBaseFrame->GetLogicalUsedMargin(lineWM);
+    nscoord newAscent = std::max(baseAscent + margin.BStart(lineWM), oldAscent);
+    nscoord newDescent = std::max(baseDesent + margin.BEnd(lineWM), oldDescent);
+    aDesiredSize.SetBlockStartAscent(newAscent);
+    aDesiredSize.BSize(lineWM) = newAscent + newDescent;
   }
 
   // Align all the line layout to the new coordinate.
@@ -649,7 +680,7 @@ nscoord nsRubyBaseContainerFrame::ReflowOneColumn(
     nscoord deltaISize = icoord - lineLayout->GetCurrentICoord();
     if (deltaISize > 0) {
       lineLayout->AdvanceICoord(deltaISize);
-      if (textFrame && !textFrame->IsAutoHidden()) {
+      if (textFrame && !textFrame->IsCollapsed()) {
         RubyUtils::SetReservedISize(textFrame, deltaISize);
       }
     }
@@ -735,8 +766,7 @@ void nsRubyBaseContainerFrame::PullOneColumn(nsLineLayout* aLineLayout,
                  "the same old float containing block");
     }
 #endif
-    nsBlockFrame* newFloatCB =
-        nsLayoutUtils::GetAsBlock(aLineLayout->LineContainerFrame());
+    nsBlockFrame* newFloatCB = do_QueryFrame(aLineLayout->LineContainerFrame());
     MOZ_ASSERT(newFloatCB, "Must have a float containing block");
     if (oldFloatCB != newFloatCB) {
       for (nsIFrame* frame : aColumn) {
