@@ -11,34 +11,33 @@
 
 #include "nsSplittableFrame.h"
 #include "nsContainerFrame.h"
+#include "nsFieldSetFrame.h"
 #include "nsIFrameInlines.h"
 
 using namespace mozilla;
 
+NS_QUERYFRAME_HEAD(nsSplittableFrame)
+  NS_QUERYFRAME_ENTRY(nsSplittableFrame)
+NS_QUERYFRAME_TAIL_INHERITING(nsIFrame)
+
 void nsSplittableFrame::Init(nsIContent* aContent, nsContainerFrame* aParent,
                              nsIFrame* aPrevInFlow) {
-  nsFrame::Init(aContent, aParent, aPrevInFlow);
-
   if (aPrevInFlow) {
     // Hook the frame into the flow
     SetPrevInFlow(aPrevInFlow);
     aPrevInFlow->SetNextInFlow(this);
   }
+  nsIFrame::Init(aContent, aParent, aPrevInFlow);
 }
 
-void nsSplittableFrame::DestroyFrom(nsIFrame* aDestructRoot,
-                                    PostDestroyData& aPostDestroyData) {
+void nsSplittableFrame::Destroy(DestroyContext& aContext) {
   // Disconnect from the flow list
   if (mPrevContinuation || mNextContinuation) {
     RemoveFromFlow(this);
   }
 
   // Let the base class destroy the frame
-  nsFrame::DestroyFrom(aDestructRoot, aPostDestroyData);
-}
-
-nsSplittableType nsSplittableFrame::GetSplittableType() const {
-  return NS_FRAME_SPLITTABLE;
+  nsIFrame::Destroy(aContext);
 }
 
 nsIFrame* nsSplittableFrame::GetPrevContinuation() const {
@@ -52,6 +51,7 @@ void nsSplittableFrame::SetPrevContinuation(nsIFrame* aFrame) {
                "creating a loop in continuation chain!");
   mPrevContinuation = aFrame;
   RemoveStateBits(NS_FRAME_IS_FLUID_CONTINUATION);
+  UpdateFirstContinuationAndFirstInFlowCache();
 }
 
 nsIFrame* nsSplittableFrame::GetNextContinuation() const {
@@ -64,16 +64,23 @@ void nsSplittableFrame::SetNextContinuation(nsIFrame* aFrame) {
   NS_ASSERTION(!IsInNextContinuationChain(aFrame, this),
                "creating a loop in continuation chain!");
   mNextContinuation = aFrame;
-  if (aFrame) aFrame->RemoveStateBits(NS_FRAME_IS_FLUID_CONTINUATION);
+  if (mNextContinuation) {
+    mNextContinuation->RemoveStateBits(NS_FRAME_IS_FLUID_CONTINUATION);
+  }
 }
 
 nsIFrame* nsSplittableFrame::FirstContinuation() const {
-  nsSplittableFrame* firstContinuation = const_cast<nsSplittableFrame*>(this);
-  while (firstContinuation->mPrevContinuation) {
-    firstContinuation =
-        static_cast<nsSplittableFrame*>(firstContinuation->mPrevContinuation);
+  if (mFirstContinuation) {
+    return mFirstContinuation;
   }
-  MOZ_ASSERT(firstContinuation, "post-condition failed");
+
+  // We fall back to the slow path during the frame destruction where our
+  // first-continuation cache was purged.
+  auto* firstContinuation = const_cast<nsSplittableFrame*>(this);
+  while (nsIFrame* prev = firstContinuation->GetPrevContinuation()) {
+    firstContinuation = static_cast<nsSplittableFrame*>(prev);
+  }
+  MOZ_ASSERT(firstContinuation);
   return firstContinuation;
 }
 
@@ -114,8 +121,8 @@ bool nsSplittableFrame::IsInNextContinuationChain(nsIFrame* aFrame1,
 #endif
 
 nsIFrame* nsSplittableFrame::GetPrevInFlow() const {
-  return (GetStateBits() & NS_FRAME_IS_FLUID_CONTINUATION) ? mPrevContinuation
-                                                           : nullptr;
+  return HasAnyStateBits(NS_FRAME_IS_FLUID_CONTINUATION) ? mPrevContinuation
+                                                         : nullptr;
 }
 
 void nsSplittableFrame::SetPrevInFlow(nsIFrame* aFrame) {
@@ -125,11 +132,12 @@ void nsSplittableFrame::SetPrevInFlow(nsIFrame* aFrame) {
                "creating a loop in continuation chain!");
   mPrevContinuation = aFrame;
   AddStateBits(NS_FRAME_IS_FLUID_CONTINUATION);
+  UpdateFirstContinuationAndFirstInFlowCache();
 }
 
 nsIFrame* nsSplittableFrame::GetNextInFlow() const {
-  return mNextContinuation && (mNextContinuation->GetStateBits() &
-                               NS_FRAME_IS_FLUID_CONTINUATION)
+  return mNextContinuation && mNextContinuation->HasAnyStateBits(
+                                  NS_FRAME_IS_FLUID_CONTINUATION)
              ? mNextContinuation
              : nullptr;
 }
@@ -140,15 +148,23 @@ void nsSplittableFrame::SetNextInFlow(nsIFrame* aFrame) {
   NS_ASSERTION(!IsInNextContinuationChain(aFrame, this),
                "creating a loop in continuation chain!");
   mNextContinuation = aFrame;
-  if (aFrame) aFrame->AddStateBits(NS_FRAME_IS_FLUID_CONTINUATION);
+  if (mNextContinuation) {
+    mNextContinuation->AddStateBits(NS_FRAME_IS_FLUID_CONTINUATION);
+  }
 }
 
 nsIFrame* nsSplittableFrame::FirstInFlow() const {
-  nsSplittableFrame* firstInFlow = const_cast<nsSplittableFrame*>(this);
+  if (mFirstInFlow) {
+    return mFirstInFlow;
+  }
+
+  // We fall back to the slow path during the frame destruction where our
+  // first-in-flow cache was purged.
+  auto* firstInFlow = const_cast<nsSplittableFrame*>(this);
   while (nsIFrame* prev = firstInFlow->GetPrevInFlow()) {
     firstInFlow = static_cast<nsSplittableFrame*>(prev);
   }
-  MOZ_ASSERT(firstInFlow, "post-condition failed");
+  MOZ_ASSERT(firstInFlow);
   return firstInFlow;
 }
 
@@ -161,7 +177,6 @@ nsIFrame* nsSplittableFrame::LastInFlow() const {
   return lastInFlow;
 }
 
-// Remove this frame from the flow. Connects prev in flow and next in flow
 void nsSplittableFrame::RemoveFromFlow(nsIFrame* aFrame) {
   nsIFrame* prevContinuation = aFrame->GetPrevContinuation();
   nsIFrame* nextContinuation = aFrame->GetNextContinuation();
@@ -184,84 +199,150 @@ void nsSplittableFrame::RemoveFromFlow(nsIFrame* aFrame) {
     }
   }
 
-  aFrame->SetPrevInFlow(nullptr);
+  // **Note: it is important here that we clear the Next link from aFrame
+  // BEFORE clearing its Prev link, because in nsContinuingTextFrame,
+  // SetPrevInFlow() would follow the Next pointers, wiping out the cached
+  // mFirstContinuation field from each following frame in the list.
   aFrame->SetNextInFlow(nullptr);
+  aFrame->SetPrevInFlow(nullptr);
 }
 
-nscoord nsSplittableFrame::ConsumedBSize(WritingMode aWM) const {
-  nscoord bSize = 0;
-  for (nsIFrame* prev = GetPrevInFlow(); prev; prev = prev->GetPrevInFlow()) {
-    bSize += prev->ContentBSize(aWM);
+void nsSplittableFrame::UpdateFirstContinuationAndFirstInFlowCache() {
+  nsIFrame* oldCachedFirstContinuation = mFirstContinuation;
+  if (nsIFrame* prevContinuation = GetPrevContinuation()) {
+    nsIFrame* newFirstContinuation = prevContinuation->FirstContinuation();
+    if (oldCachedFirstContinuation != newFirstContinuation) {
+      // Update the first-continuation cache for us and our next-continuations.
+      for (nsSplittableFrame* f = this; f;
+           f = reinterpret_cast<nsSplittableFrame*>(f->GetNextContinuation())) {
+        f->mFirstContinuation = newFirstContinuation;
+      }
+    }
+  } else {
+    // We become the new first-continuation due to our prev-continuation being
+    // removed.
+    if (oldCachedFirstContinuation) {
+      // It's tempting to update the first-continuation cache for our
+      // next-continuations here, but that would result in overall O(n^2)
+      // behavior when a frame list is destroyed from the front. To avoid that
+      // pathological behavior, we simply purge the cached values.
+      for (nsSplittableFrame* f = this; f;
+           f = reinterpret_cast<nsSplittableFrame*>(f->GetNextContinuation())) {
+        f->mFirstContinuation = nullptr;
+      }
+    }
   }
+
+  nsIFrame* oldCachedFirstInFlow = mFirstInFlow;
+  if (nsIFrame* prevInFlow = GetPrevInFlow()) {
+    nsIFrame* newFirstInFlow = prevInFlow->FirstInFlow();
+    if (oldCachedFirstInFlow != newFirstInFlow) {
+      // Update the first-in-flow cache for us and our next-in-flows.
+      for (nsSplittableFrame* f = this; f;
+           f = reinterpret_cast<nsSplittableFrame*>(f->GetNextInFlow())) {
+        f->mFirstInFlow = newFirstInFlow;
+      }
+    }
+  } else {
+    // We become the new first-in-flow due to our prev-in-flow being removed.
+    if (oldCachedFirstInFlow) {
+      // It's tempting to update the first-in-flow cache for our
+      // next-in-flows here, but that would result in overall O(n^2)
+      // behavior when a frame list is destroyed from the front. To avoid that
+      // pathological behavior, we simply purge the cached values.
+      for (nsSplittableFrame* f = this; f;
+           f = reinterpret_cast<nsSplittableFrame*>(f->GetNextInFlow())) {
+        f->mFirstInFlow = nullptr;
+      }
+    }
+  }
+}
+
+NS_DECLARE_FRAME_PROPERTY_SMALL_VALUE(ConsumedBSizeProperty, nscoord);
+
+nscoord nsSplittableFrame::CalcAndCacheConsumedBSize() {
+  nsIFrame* prev = GetPrevContinuation();
+  if (!prev) {
+    return 0;
+  }
+  const auto wm = GetWritingMode();
+  nscoord bSize = 0;
+  for (; prev; prev = prev->GetPrevContinuation()) {
+    if (prev->IsTrueOverflowContainer()) {
+      // Overflow containers might not get reflowed, and they have no bSize
+      // anyways.
+      continue;
+    }
+
+    bSize += prev->ContentBSize(wm);
+    bool found = false;
+    nscoord consumed = prev->GetProperty(ConsumedBSizeProperty(), &found);
+    if (found) {
+      bSize += consumed;
+      break;
+    }
+    MOZ_ASSERT(!prev->GetPrevContinuation(),
+               "Property should always be set on prev continuation if not "
+               "the first continuation");
+  }
+  SetProperty(ConsumedBSizeProperty(), bSize);
   return bSize;
 }
 
 nscoord nsSplittableFrame::GetEffectiveComputedBSize(
     const ReflowInput& aReflowInput, nscoord aConsumedBSize) const {
   nscoord bSize = aReflowInput.ComputedBSize();
-  if (bSize == NS_INTRINSICSIZE) {
-    return NS_INTRINSICSIZE;
-  }
-
-  if (aConsumedBSize == NS_INTRINSICSIZE) {
-    aConsumedBSize = ConsumedBSize(aReflowInput.GetWritingMode());
+  if (bSize == NS_UNCONSTRAINEDSIZE) {
+    return NS_UNCONSTRAINEDSIZE;
   }
 
   bSize -= aConsumedBSize;
 
+  // nsFieldSetFrame's inner frames are special since some of their content-box
+  // BSize may be consumed by positioning it below the legend.  So we always
+  // report zero for true overflow containers here.
+  // XXXmats: hmm, can we fix this so that the sizes actually adds up instead?
+  if (IsTrueOverflowContainer() &&
+      Style()->GetPseudoType() == PseudoStyleType::fieldsetContent) {
+    for (nsFieldSetFrame* fieldset = do_QueryFrame(GetParent()); fieldset;
+         fieldset = static_cast<nsFieldSetFrame*>(fieldset->GetPrevInFlow())) {
+      bSize -= fieldset->LegendSpace();
+    }
+  }
+
   // We may have stretched the frame beyond its computed height. Oh well.
   return std::max(0, bSize);
 }
 
-nsIFrame::LogicalSides nsSplittableFrame::GetLogicalSkipSides(
-    const ReflowInput* aReflowInput) const {
-  if (IS_TRUE_OVERFLOW_CONTAINER(this)) {
-    return LogicalSides(eLogicalSideBitsBBoth);
+LogicalSides nsSplittableFrame::GetBlockLevelLogicalSkipSides(
+    bool aAfterReflow) const {
+  LogicalSides skip(mWritingMode);
+  if (MOZ_UNLIKELY(IsTrueOverflowContainer())) {
+    skip += LogicalSides(mWritingMode, LogicalSides::BBoth);
+    return skip;
   }
 
   if (MOZ_UNLIKELY(StyleBorder()->mBoxDecorationBreak ==
                    StyleBoxDecorationBreak::Clone)) {
-    return LogicalSides();
+    return skip;
   }
 
-  LogicalSides skip;
-  if (GetPrevInFlow()) {
-    skip |= eLogicalSideBitsBStart;
+  if (GetPrevContinuation()) {
+    skip += LogicalSide::BStart;
   }
 
-  if (aReflowInput) {
-    // We're in the midst of reflow right now, so it's possible that we haven't
-    // created a nif yet. If our content height is going to exceed our available
-    // height, though, then we're going to need a next-in-flow, it just hasn't
-    // been created yet.
+  // Always skip block-end side if we have a *later* sibling across column-span
+  // split.
+  if (HasColumnSpanSiblings()) {
+    skip += LogicalSide::BEnd;
+  }
 
-    if (NS_UNCONSTRAINEDSIZE != aReflowInput->AvailableBSize()) {
-      nscoord effectiveCH = this->GetEffectiveComputedBSize(*aReflowInput);
-      if (effectiveCH != NS_INTRINSICSIZE &&
-          effectiveCH > aReflowInput->AvailableBSize()) {
-        // Our content height is going to exceed our available height, so we're
-        // going to need a next-in-flow.
-        skip |= eLogicalSideBitsBEnd;
-      }
-    }
-  } else {
-    nsIFrame* nif = GetNextInFlow();
-    if (nif && !IS_TRUE_OVERFLOW_CONTAINER(nif)) {
-      skip |= eLogicalSideBitsBEnd;
+  if (aAfterReflow) {
+    nsIFrame* nif = GetNextContinuation();
+    if (nif && !nif->IsTrueOverflowContainer()) {
+      skip += LogicalSide::BEnd;
     }
   }
 
   return skip;
 }
-
-LogicalSides nsSplittableFrame::PreReflowBlockLevelLogicalSkipSides() const {
-  if (MOZ_UNLIKELY(IS_TRUE_OVERFLOW_CONTAINER(this))) {
-    return LogicalSides(mozilla::eLogicalSideBitsBBoth);
-  }
-  if (MOZ_LIKELY(StyleBorder()->mBoxDecorationBreak !=
-                 StyleBoxDecorationBreak::Clone) &&
-      GetPrevInFlow()) {
-    return LogicalSides(mozilla::eLogicalSideBitsBStart);
-  }
-  return LogicalSides();
-}
