@@ -6,13 +6,14 @@
 
 #include "nsMathMLOperators.h"
 #include "nsCOMPtr.h"
-#include "nsDataHashtable.h"
+#include "nsTHashMap.h"
 #include "nsHashKeys.h"
+#include "nsNetUtil.h"
 #include "nsTArray.h"
 
+#include "mozilla/intl/UnicodeProperties.h"
 #include "nsIPersistentProperties2.h"
 #include "nsISimpleEnumerator.h"
-#include "nsContentUtils.h"
 #include "nsCRT.h"
 
 // operator dictionary entry
@@ -29,70 +30,81 @@ struct OperatorData {
 static int32_t gTableRefCount = 0;
 static uint32_t gOperatorCount = 0;
 static OperatorData* gOperatorArray = nullptr;
-static nsDataHashtable<nsStringHashKey, OperatorData*>* gOperatorTable =
-    nullptr;
+static nsTHashMap<nsStringHashKey, OperatorData*>* gOperatorTable = nullptr;
 static bool gGlobalsInitialized = false;
 
 static const char16_t kDashCh = char16_t('#');
 static const char16_t kColonCh = char16_t(':');
 
-static void SetBooleanProperty(OperatorData* aOperatorData, nsString aName) {
-  if (aName.IsEmpty()) return;
+static uint32_t ToUnicodeCodePoint(const nsString& aOperator) {
+  if (aOperator.Length() == 1) {
+    return aOperator[0];
+  }
+  if (aOperator.Length() == 2 &&
+      NS_IS_SURROGATE_PAIR(aOperator[0], aOperator[1])) {
+    return SURROGATE_TO_UCS4(aOperator[0], aOperator[1]);
+  }
+  return 0;
+}
 
-  if (aName.EqualsLiteral("stretchy") && (1 == aOperatorData->mStr.Length()))
+static void SetBooleanProperty(OperatorData* aOperatorData, nsString aName) {
+  if (aName.IsEmpty()) {
+    return;
+  }
+
+  if (aName.EqualsLiteral("stretchy") && (1 == aOperatorData->mStr.Length())) {
     aOperatorData->mFlags |= NS_MATHML_OPERATOR_STRETCHY;
-  else if (aName.EqualsLiteral("fence"))
+  } else if (aName.EqualsLiteral("fence")) {
     aOperatorData->mFlags |= NS_MATHML_OPERATOR_FENCE;
-  else if (aName.EqualsLiteral("accent"))
+  } else if (aName.EqualsLiteral("accent")) {
     aOperatorData->mFlags |= NS_MATHML_OPERATOR_ACCENT;
-  else if (aName.EqualsLiteral("largeop"))
+  } else if (aName.EqualsLiteral("largeop")) {
     aOperatorData->mFlags |= NS_MATHML_OPERATOR_LARGEOP;
-  else if (aName.EqualsLiteral("separator"))
+  } else if (aName.EqualsLiteral("separator")) {
     aOperatorData->mFlags |= NS_MATHML_OPERATOR_SEPARATOR;
-  else if (aName.EqualsLiteral("movablelimits"))
+  } else if (aName.EqualsLiteral("movablelimits")) {
     aOperatorData->mFlags |= NS_MATHML_OPERATOR_MOVABLELIMITS;
-  else if (aName.EqualsLiteral("symmetric"))
+  } else if (aName.EqualsLiteral("symmetric")) {
     aOperatorData->mFlags |= NS_MATHML_OPERATOR_SYMMETRIC;
-  else if (aName.EqualsLiteral("integral"))
-    aOperatorData->mFlags |= NS_MATHML_OPERATOR_INTEGRAL;
-  else if (aName.EqualsLiteral("mirrorable"))
-    aOperatorData->mFlags |= NS_MATHML_OPERATOR_MIRRORABLE;
+  }
 }
 
 static void SetProperty(OperatorData* aOperatorData, nsString aName,
                         nsString aValue) {
-  if (aName.IsEmpty() || aValue.IsEmpty()) return;
-
-  // XXX These ones are not kept in the dictionary
-  // Support for these requires nsString member variables
-  // maxsize (default: infinity)
-  // minsize (default: 1)
+  if (aName.IsEmpty() || aValue.IsEmpty()) {
+    return;
+  }
 
   if (aName.EqualsLiteral("direction")) {
-    if (aValue.EqualsLiteral("vertical"))
+    if (aValue.EqualsLiteral("vertical")) {
       aOperatorData->mFlags |= NS_MATHML_OPERATOR_DIRECTION_VERTICAL;
-    else if (aValue.EqualsLiteral("horizontal"))
+    } else if (aValue.EqualsLiteral("horizontal")) {
       aOperatorData->mFlags |= NS_MATHML_OPERATOR_DIRECTION_HORIZONTAL;
-    else
+    } else {
       return;  // invalid value
+    }
   } else {
     bool isLeadingSpace;
-    if (aName.EqualsLiteral("lspace"))
+    if (aName.EqualsLiteral("lspace")) {
       isLeadingSpace = true;
-    else if (aName.EqualsLiteral("rspace"))
+    } else if (aName.EqualsLiteral("rspace")) {
       isLeadingSpace = false;
-    else
+    } else {
       return;  // input is not applicable
+    }
 
     // aValue is assumed to be a digit from 0 to 7
     nsresult error = NS_OK;
     float space = aValue.ToFloat(&error) / 18.0;
-    if (NS_FAILED(error)) return;
+    if (NS_FAILED(error)) {
+      return;
+    }
 
-    if (isLeadingSpace)
+    if (isLeadingSpace) {
       aOperatorData->mLeadingSpace = space;
-    else
+    } else {
       aOperatorData->mTrailingSpace = space;
+    }
   }
 }
 
@@ -112,23 +124,34 @@ static bool SetOperator(OperatorData* aOperatorData, nsOperatorFlags aForm,
   char16_t uchar = 0;
   while (i <= len) {
     if (0 == state) {
-      if (c != '\\') return false;
-      if (i < len) c = aOperator[i];
+      if (c != '\\') {
+        return false;
+      }
+      if (i < len) {
+        c = aOperator[i];
+      }
       i++;
-      if (('u' != c) && ('U' != c)) return false;
-      if (i < len) c = aOperator[i];
+      if (('u' != c) && ('U' != c)) {
+        return false;
+      }
+      if (i < len) {
+        c = aOperator[i];
+      }
       i++;
       state++;
     } else {
-      if (('0' <= c) && (c <= '9'))
+      if (('0' <= c) && (c <= '9')) {
         uchar = (uchar << 4) | (c - '0');
-      else if (('a' <= c) && (c <= 'f'))
+      } else if (('a' <= c) && (c <= 'f')) {
         uchar = (uchar << 4) | (c - 'a' + 0x0a);
-      else if (('A' <= c) && (c <= 'F'))
+      } else if (('A' <= c) && (c <= 'F')) {
         uchar = (uchar << 4) | (c - 'A' + 0x0a);
-      else
+      } else {
         return false;
-      if (i < len) c = aOperator[i];
+      }
+      if (i < len) {
+        c = aOperator[i];
+      }
       i++;
       state++;
       if (5 == state) {
@@ -138,18 +161,22 @@ static bool SetOperator(OperatorData* aOperatorData, nsOperatorFlags aForm,
       }
     }
   }
-  if (0 != state) return false;
+  if (0 != state) {
+    return false;
+  }
 
   // Quick return when the caller doesn't care about the attributes and just
   // wants to know if this is a valid operator (this is the case at the first
   // pass of the parsing of the dictionary in InitOperators())
-  if (!aForm) return true;
+  if (!aForm) {
+    return true;
+  }
 
   // Add operator to hash table
   aOperatorData->mFlags |= aForm;
   aOperatorData->mStr.Assign(value);
   value.AppendInt(aForm, 10);
-  gOperatorTable->Put(value, aOperatorData);
+  gOperatorTable->InsertOrUpdate(value, aOperatorData);
 
 #ifdef DEBUG
   NS_LossyConvertUTF16toASCII str(aAttributes);
@@ -207,9 +234,11 @@ static nsresult InitOperators(void) {
   nsCOMPtr<nsIPersistentProperties> mathfontProp;
   rv = NS_LoadPersistentPropertiesFromURISpec(
       getter_AddRefs(mathfontProp),
-      NS_LITERAL_CSTRING("resource://gre/res/fonts/mathfont.properties"));
+      "resource://gre/res/fonts/mathfont.properties"_ns);
 
-  if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
 
   // Parse the Operator Dictionary in two passes.
   // The first pass is to count the number of operators; the second pass is to
@@ -244,14 +273,16 @@ static nsresult InitOperators(void) {
               } else if (kNotFound != name.RFind(".prefix")) {
                 form = NS_MATHML_OPERATOR_FORM_PREFIX;
                 len -= 7;  // 7 is the length of ".prefix";
-              } else
+              } else {
                 continue;  // input is not applicable
+              }
               name.SetLength(len);
               if (2 == pass) {  // allocate space and start the storage
                 if (!gOperatorArray) {
-                  if (0 == gOperatorCount) return NS_ERROR_UNEXPECTED;
+                  if (0 == gOperatorCount) {
+                    return NS_ERROR_UNEXPECTED;
+                  }
                   gOperatorArray = new OperatorData[gOperatorCount];
-                  if (!gOperatorArray) return NS_ERROR_OUT_OF_MEMORY;
                 }
                 operatorData = &gOperatorArray[index];
               } else {
@@ -260,7 +291,9 @@ static nsresult InitOperators(void) {
               // See if the operator should be retained
               if (SetOperator(operatorData, form, name, attributes)) {
                 index++;
-                if (1 == pass) gOperatorCount = index;
+                if (1 == pass) {
+                  gOperatorCount = index;
+                }
               }
             }
           }
@@ -274,11 +307,13 @@ static nsresult InitOperators(void) {
 static nsresult InitOperatorGlobals() {
   gGlobalsInitialized = true;
   nsresult rv = NS_ERROR_OUT_OF_MEMORY;
-  gOperatorTable = new nsDataHashtable<nsStringHashKey, OperatorData*>();
+  gOperatorTable = new nsTHashMap<nsStringHashKey, OperatorData*>();
   if (gOperatorTable) {
     rv = InitOperators();
   }
-  if (NS_FAILED(rv)) nsMathMLOperators::CleanUp();
+  if (NS_FAILED(rv)) {
+    nsMathMLOperators::CleanUp();
+  }
   return rv;
 }
 
@@ -302,126 +337,129 @@ void nsMathMLOperators::ReleaseTable(void) {
 }
 
 static OperatorData* GetOperatorData(const nsString& aOperator,
-                                     nsOperatorFlags aForm) {
+                                     const uint8_t aForm) {
   nsAutoString key(aOperator);
   key.AppendInt(aForm);
   return gOperatorTable->Get(key);
 }
 
 bool nsMathMLOperators::LookupOperator(const nsString& aOperator,
-                                       const nsOperatorFlags aForm,
+                                       const uint8_t aForm,
                                        nsOperatorFlags* aFlags,
                                        float* aLeadingSpace,
                                        float* aTrailingSpace) {
+  NS_ASSERTION(aFlags && aLeadingSpace && aTrailingSpace, "bad usage");
+  NS_ASSERTION(aForm > 0 && aForm < 4, "*** invalid call ***");
+
+  // Operator strings must be of length 1 or 2 in UTF-16.
+  // https://w3c.github.io/mathml-core/#dfn-algorithm-to-determine-the-category-of-an-operator
+  if (aOperator.IsEmpty() || aOperator.Length() > 2) {
+    return false;
+  }
+
+  if (aOperator.Length() == 2) {
+    // Try and handle Arabic operators.
+    // https://w3c.github.io/mathml-core/#dfn-algorithm-to-determine-the-category-of-an-operator
+    if (auto codePoint = ToUnicodeCodePoint(aOperator)) {
+      if (aForm == NS_MATHML_OPERATOR_FORM_POSTFIX &&
+          (codePoint == 0x1EEF0 || codePoint == 0x1EEF1)) {
+        // Use category I.
+        // https://w3c.github.io/mathml-core/#operator-dictionary-categories-values
+        *aFlags = NS_MATHML_OPERATOR_FORM_POSTFIX |
+                  NS_MATHML_OPERATOR_STRETCHY |
+                  NS_MATHML_OPERATOR_DIRECTION_HORIZONTAL;
+        *aLeadingSpace = 0;
+        *aTrailingSpace = 0;
+        return true;
+      }
+      return false;
+    }
+
+    // Ignore the combining "negation" suffix for 2-character strings.
+    // https://w3c.github.io/mathml-core/#dfn-algorithm-to-determine-the-category-of-an-operator
+    if (aOperator[1] == 0x0338 || aOperator[1] == 0x20D2) {
+      nsAutoString newOperator;
+      newOperator.Append(aOperator[0]);
+      return LookupOperator(newOperator, aForm, aFlags, aLeadingSpace,
+                            aTrailingSpace);
+    }
+  }
+
   if (!gGlobalsInitialized) {
     InitOperatorGlobals();
   }
   if (gOperatorTable) {
-    NS_ASSERTION(aFlags && aLeadingSpace && aTrailingSpace, "bad usage");
-    NS_ASSERTION(aForm > 0 && aForm < 4, "*** invalid call ***");
-
-    // The MathML REC says:
-    // If the operator does not occur in the dictionary with the specified form,
-    // the renderer should use one of the forms which is available there, in the
-    // order of preference: infix, postfix, prefix.
-
-    OperatorData* found;
-    int32_t form = NS_MATHML_OPERATOR_GET_FORM(aForm);
-    if (!(found = GetOperatorData(aOperator, form))) {
-      if (form == NS_MATHML_OPERATOR_FORM_INFIX ||
-          !(found =
-                GetOperatorData(aOperator, NS_MATHML_OPERATOR_FORM_INFIX))) {
-        if (form == NS_MATHML_OPERATOR_FORM_POSTFIX ||
-            !(found = GetOperatorData(aOperator,
-                                      NS_MATHML_OPERATOR_FORM_POSTFIX))) {
-          if (form != NS_MATHML_OPERATOR_FORM_PREFIX) {
-            found = GetOperatorData(aOperator, NS_MATHML_OPERATOR_FORM_PREFIX);
-          }
-        }
-      }
+    if (OperatorData* data = GetOperatorData(aOperator, aForm)) {
+      NS_ASSERTION(data->mStr.Equals(aOperator), "bad setup");
+      *aFlags = data->mFlags;
+      *aLeadingSpace = data->mLeadingSpace;
+      *aTrailingSpace = data->mTrailingSpace;
+      return true;
     }
-    if (found) {
-      NS_ASSERTION(found->mStr.Equals(aOperator), "bad setup");
-      *aLeadingSpace = found->mLeadingSpace;
-      *aTrailingSpace = found->mTrailingSpace;
-      *aFlags &= ~NS_MATHML_OPERATOR_FORM;  // clear the form bits
-      *aFlags |= found->mFlags;             // just add bits without overwriting
+  }
+
+  return false;
+}
+
+bool nsMathMLOperators::LookupOperatorWithFallback(const nsString& aOperator,
+                                                   const uint8_t aForm,
+                                                   nsOperatorFlags* aFlags,
+                                                   float* aLeadingSpace,
+                                                   float* aTrailingSpace) {
+  if (LookupOperator(aOperator, aForm, aFlags, aLeadingSpace, aTrailingSpace)) {
+    return true;
+  }
+  for (const auto& form :
+       {NS_MATHML_OPERATOR_FORM_INFIX, NS_MATHML_OPERATOR_FORM_POSTFIX,
+        NS_MATHML_OPERATOR_FORM_PREFIX}) {
+    if (form == aForm) {
+      // This form was tried above, skip it.
+      continue;
+    }
+    if (LookupOperator(aOperator, form, aFlags, aLeadingSpace,
+                       aTrailingSpace)) {
       return true;
     }
   }
   return false;
 }
 
-void nsMathMLOperators::LookupOperators(const nsString& aOperator,
-                                        nsOperatorFlags* aFlags,
-                                        float* aLeadingSpace,
-                                        float* aTrailingSpace) {
-  if (!gGlobalsInitialized) {
-    InitOperatorGlobals();
-  }
-
-  aFlags[NS_MATHML_OPERATOR_FORM_INFIX] = 0;
-  aLeadingSpace[NS_MATHML_OPERATOR_FORM_INFIX] = 0.0f;
-  aTrailingSpace[NS_MATHML_OPERATOR_FORM_INFIX] = 0.0f;
-
-  aFlags[NS_MATHML_OPERATOR_FORM_POSTFIX] = 0;
-  aLeadingSpace[NS_MATHML_OPERATOR_FORM_POSTFIX] = 0.0f;
-  aTrailingSpace[NS_MATHML_OPERATOR_FORM_POSTFIX] = 0.0f;
-
-  aFlags[NS_MATHML_OPERATOR_FORM_PREFIX] = 0;
-  aLeadingSpace[NS_MATHML_OPERATOR_FORM_PREFIX] = 0.0f;
-  aTrailingSpace[NS_MATHML_OPERATOR_FORM_PREFIX] = 0.0f;
-
-  if (gOperatorTable) {
-    OperatorData* found;
-    found = GetOperatorData(aOperator, NS_MATHML_OPERATOR_FORM_INFIX);
-    if (found) {
-      aFlags[NS_MATHML_OPERATOR_FORM_INFIX] = found->mFlags;
-      aLeadingSpace[NS_MATHML_OPERATOR_FORM_INFIX] = found->mLeadingSpace;
-      aTrailingSpace[NS_MATHML_OPERATOR_FORM_INFIX] = found->mTrailingSpace;
-    }
-    found = GetOperatorData(aOperator, NS_MATHML_OPERATOR_FORM_POSTFIX);
-    if (found) {
-      aFlags[NS_MATHML_OPERATOR_FORM_POSTFIX] = found->mFlags;
-      aLeadingSpace[NS_MATHML_OPERATOR_FORM_POSTFIX] = found->mLeadingSpace;
-      aTrailingSpace[NS_MATHML_OPERATOR_FORM_POSTFIX] = found->mTrailingSpace;
-    }
-    found = GetOperatorData(aOperator, NS_MATHML_OPERATOR_FORM_PREFIX);
-    if (found) {
-      aFlags[NS_MATHML_OPERATOR_FORM_PREFIX] = found->mFlags;
-      aLeadingSpace[NS_MATHML_OPERATOR_FORM_PREFIX] = found->mLeadingSpace;
-      aTrailingSpace[NS_MATHML_OPERATOR_FORM_PREFIX] = found->mTrailingSpace;
-    }
+/* static */
+bool nsMathMLOperators::IsMirrorableOperator(const nsString& aOperator) {
+  if (auto codePoint = ToUnicodeCodePoint(aOperator)) {
+    return mozilla::intl::UnicodeProperties::IsMirrored(codePoint);
   }
+  return false;
 }
 
-/* static */ bool nsMathMLOperators::IsMirrorableOperator(
+/* static */
+bool nsMathMLOperators::IsIntegralOperator(const nsString& aOperator) {
+  if (auto codePoint = ToUnicodeCodePoint(aOperator)) {
+    return (0x222B <= codePoint && codePoint <= 0x2233) ||
+           (0x2A0B <= codePoint && codePoint <= 0x2A1C);
+  }
+  return false;
+}
+
+/* static */
+nsStretchDirection nsMathMLOperators::GetStretchyDirection(
     const nsString& aOperator) {
-  // LookupOperator will search infix, postfix and prefix forms of aOperator and
-  // return the first form found. It is assumed that all these forms have same
-  // mirrorability.
-  nsOperatorFlags flags = 0;
-  float dummy;
-  nsMathMLOperators::LookupOperator(aOperator, NS_MATHML_OPERATOR_FORM_INFIX,
-                                    &flags, &dummy, &dummy);
-  return NS_MATHML_OPERATOR_IS_MIRRORABLE(flags);
-}
-
-/* static */ nsStretchDirection nsMathMLOperators::GetStretchyDirection(
-    const nsString& aOperator) {
-  // LookupOperator will search infix, postfix and prefix forms of aOperator and
-  // return the first form found. It is assumed that all these forms have same
-  // direction.
-  nsOperatorFlags flags = 0;
-  float dummy;
-  nsMathMLOperators::LookupOperator(aOperator, NS_MATHML_OPERATOR_FORM_INFIX,
-                                    &flags, &dummy, &dummy);
-
-  if (NS_MATHML_OPERATOR_IS_DIRECTION_VERTICAL(flags)) {
-    return NS_STRETCH_DIRECTION_VERTICAL;
-  } else if (NS_MATHML_OPERATOR_IS_DIRECTION_HORIZONTAL(flags)) {
-    return NS_STRETCH_DIRECTION_HORIZONTAL;
-  } else {
-    return NS_STRETCH_DIRECTION_UNSUPPORTED;
+  // Search any entry for that operator and return the corresponding direction.
+  // It is assumed that all the forms have same direction.
+  for (const auto& form :
+       {NS_MATHML_OPERATOR_FORM_INFIX, NS_MATHML_OPERATOR_FORM_POSTFIX,
+        NS_MATHML_OPERATOR_FORM_PREFIX}) {
+    nsOperatorFlags flags;
+    float dummy;
+    if (nsMathMLOperators::LookupOperator(aOperator, form, &flags, &dummy,
+                                          &dummy)) {
+      if (NS_MATHML_OPERATOR_IS_DIRECTION_VERTICAL(flags)) {
+        return NS_STRETCH_DIRECTION_VERTICAL;
+      }
+      if (NS_MATHML_OPERATOR_IS_DIRECTION_HORIZONTAL(flags)) {
+        return NS_STRETCH_DIRECTION_HORIZONTAL;
+      }
+    }
   }
+  return NS_STRETCH_DIRECTION_UNSUPPORTED;
 }
