@@ -28,6 +28,7 @@
 #include "mozilla/dom/ViewTransition.h"
 #include "nsILoadContext.h"
 #include "nsIFrame.h"
+#include "nsIFrameInlines.h"
 #include "nsINode.h"
 #include "nsIURI.h"
 #include "nsFontMetrics.h"
@@ -63,7 +64,6 @@
 #include "mozilla/StyleAnimationValue.h"
 #include "mozilla/ServoBindings.h"
 #include "mozilla/ServoTraversalStatistics.h"
-#include "mozilla/Telemetry.h"
 #include "mozilla/TimelineManager.h"
 #include "mozilla/RWLock.h"
 #include "mozilla/dom/Element.h"
@@ -310,7 +310,6 @@ bool Gecko_AnimationNameMayBeReferencedFromStyle(
 
 float Gecko_GetScrollbarInlineSize(const nsPresContext* aPc) {
   MOZ_ASSERT(aPc);
-  AutoWriteLock guard(*sServoFFILock);  // We read some look&feel values.
   auto overlay = aPc->UseOverlayScrollbars() ? nsITheme::Overlay::Yes
                                              : nsITheme::Overlay::No;
   LayoutDeviceIntCoord size =
@@ -749,20 +748,16 @@ nscolor Gecko_ComputeSystemColor(StyleSystemColor aColor, const Document* aDoc,
   }
 
   auto useStandins = LookAndFeel::ShouldUseStandins(*aDoc, aColor);
-
-  AutoWriteLock guard(*sServoFFILock);
   return LookAndFeel::Color(aColor, colorScheme, useStandins);
 }
 
 int32_t Gecko_GetLookAndFeelInt(int32_t aId) {
   auto intId = static_cast<LookAndFeel::IntID>(aId);
-  AutoWriteLock guard(*sServoFFILock);
   return LookAndFeel::GetInt(intId);
 }
 
 float Gecko_GetLookAndFeelFloat(int32_t aId) {
   auto id = static_cast<LookAndFeel::FloatID>(aId);
-  AutoWriteLock guard(*sServoFFILock);
   return LookAndFeel::GetFloat(id);
 }
 
@@ -803,6 +798,22 @@ bool Gecko_MatchLang(const Element* aElement, nsAtom* aOverrideLang,
   return false;
 }
 
+bool Gecko_MatchViewTransitionClass(
+    const mozilla::dom::Element* aElement,
+    const nsTArray<StyleAtom>* aPtNameAndClassSelector) {
+  MOZ_ASSERT(aElement && aPtNameAndClassSelector);
+
+  const Document* doc = aElement->OwnerDoc();
+  MOZ_ASSERT(doc);
+  const ViewTransition* vt = doc->GetActiveViewTransition();
+  MOZ_ASSERT(
+      vt, "We should have an active view transition for this pseudo-element");
+
+  nsAtom* name = Gecko_GetImplementedPseudoIdentifier(aElement);
+  MOZ_ASSERT(name);
+  return vt->MatchClassList(name, *aPtNameAndClassSelector);
+}
+
 nsAtom* Gecko_GetXMLLangValue(const Element* aElement) {
   const nsAttrValue* attr =
       aElement->GetParsedAttr(nsGkAtoms::lang, kNameSpaceID_XML);
@@ -940,7 +951,6 @@ void Gecko_nsFont_InitSystem(nsFont* aDest, StyleSystemFont aFontId,
   // itself, so this will do.
   new (aDest) nsFont(defaultVariableFont);
 
-  AutoWriteLock guard(*sServoFFILock);
   nsLayoutUtils::ComputeSystemFont(aDest, aFontId, defaultVariableFont,
                                    aDocument);
 }
@@ -1292,8 +1302,8 @@ void AssertIsMainThreadOrServoFontMetricsLocked() {
 GeckoFontMetrics Gecko_GetFontMetrics(const nsPresContext* aPresContext,
                                       bool aIsVertical,
                                       const nsStyleFont* aFont,
-                                      Length aFontSize, bool aUseUserFontSet,
-                                      bool aRetrieveMathScales) {
+                                      Length aFontSize,
+                                      StyleQueryFontMetricsFlags flags) {
   AutoWriteLock guard(*sServoFFILock);
 
   // Getting font metrics can require some main thread only work to be
@@ -1309,13 +1319,14 @@ GeckoFontMetrics Gecko_GetFontMetrics(const nsPresContext* aPresContext,
 
   nsPresContext* presContext = const_cast<nsPresContext*>(aPresContext);
   RefPtr<nsFontMetrics> fm = nsLayoutUtils::GetMetricsFor(
-      presContext, aIsVertical, aFont, aFontSize, aUseUserFontSet);
+      presContext, aIsVertical, aFont, aFontSize,
+      bool(flags & StyleQueryFontMetricsFlags::USE_USER_FONT_SET));
   auto* fontGroup = fm->GetThebesFontGroup();
-  auto metrics = fontGroup->GetMetricsForCSSUnits(fm->Orientation());
+  auto metrics = fontGroup->GetMetricsForCSSUnits(fm->Orientation(), flags);
 
   float scriptPercentScaleDown = 0;
   float scriptScriptPercentScaleDown = 0;
-  if (aRetrieveMathScales) {
+  if (flags & StyleQueryFontMetricsFlags::NEEDS_MATH_SCALES) {
     RefPtr<gfxFont> font = fontGroup->GetFirstValidFont();
     if (font->TryGetMathTable()) {
       scriptPercentScaleDown = static_cast<float>(
@@ -1824,3 +1835,196 @@ StyleFontFamilyList StyleFontFamilyList::WithOneUnquotedFamily(
       {StyleAtom(NS_Atomize(aName)), StyleFontFamilyNameSyntax::Identifiers}));
   return WithNames(std::move(names));
 }
+
+// Find the aContainer's child that is the ancestor of aDescendant.
+static const nsIFrame* TraverseUpToContainerChild(const nsIFrame* aContainer,
+                                                  const nsIFrame* aDescendant) {
+  const auto* current = aDescendant;
+  while (true) {
+    const auto* parent = current->GetParent();
+    if (!parent) {
+      return nullptr;
+    }
+    if (parent == aContainer) {
+      return current;
+    }
+    current = parent;
+  }
+}
+
+static bool AnchorSideUsesCBWM(
+    const StyleAnchorSideKeyword& aAnchorSideKeyword) {
+  switch (aAnchorSideKeyword) {
+    case StyleAnchorSideKeyword::SelfStart:
+    case StyleAnchorSideKeyword::SelfEnd:
+      return false;
+    case StyleAnchorSideKeyword::Inside:
+    case StyleAnchorSideKeyword::Outside:
+    case StyleAnchorSideKeyword::Start:
+    case StyleAnchorSideKeyword::End:
+    case StyleAnchorSideKeyword::Center:
+      return true;
+    // Return value shouldn't matter for these physical keywords.
+    case StyleAnchorSideKeyword::Left:
+    case StyleAnchorSideKeyword::Right:
+    case StyleAnchorSideKeyword::Top:
+    case StyleAnchorSideKeyword::Bottom:
+      return true;
+  }
+  return false;
+}
+
+struct AnchorPosInfo {
+  // Border-box of the anchor frame, offset against `mContainingBlock`'s padding
+  // box.
+  nsRect mRect;
+  const nsIFrame* mContainingBlock;
+};
+
+static Maybe<AnchorPosInfo> GetAnchorPosRect(const nsIFrame* aPositioned,
+                                             const nsAtom* aAnchorName,
+                                             bool aCBRectIsvalid) {
+  if (!aPositioned) {
+    return Nothing{};
+  }
+  const auto* presShell = aPositioned->PresShell();
+  MOZ_ASSERT(presShell, "No PresShell for frame?");
+
+  const auto* anchorName = aAnchorName;
+  if (!anchorName || anchorName->IsEmpty()) {
+    const auto* stylePos = aPositioned->StylePosition();
+    if (!stylePos->mPositionAnchor.IsIdent()) {
+      // No valid anchor specified, bail.
+      // TODO(dshin): Implicit anchor should be looked at here.
+      return Nothing{};
+    }
+    anchorName = stylePos->mPositionAnchor.AsIdent().AsAtom();
+  }
+  const auto* anchor = presShell->GetAnchorPosAnchor(anchorName, aPositioned);
+  if (!anchor) {
+    return Nothing{};
+  }
+
+  MOZ_ASSERT(aPositioned->HasAnyStateBits(NS_FRAME_OUT_OF_FLOW),
+             "Calling GetAnchorPoseRect on non-abspos frame?");
+  // We're assuming that the caller already check for abspos.
+  const auto* containingBlock = aPositioned->GetParent();
+  auto rect = [&]() -> Maybe<nsRect> {
+    if (aCBRectIsvalid) {
+      nsRect result = anchor->GetRectRelativeToSelf();
+      nsLayoutUtils::TransformRect(anchor, containingBlock, result);
+      // Easy, just use the existing function.
+      return Some(result);
+    }
+
+    // Ok, containing block doesn't have its rect fully resolved. Figure out
+    // rect relative to the child of containing block that is also the ancestor
+    // of the anchor, and manually compute the offset.
+    // TODO(dshin): This wouldn't handle anchor in a previous top layer.
+    const auto* containerChild =
+        TraverseUpToContainerChild(containingBlock, anchor);
+    if (!containerChild) {
+      return Nothing{};
+    }
+
+    if (anchor == containerChild) {
+      // Anchor is the direct child of anchor's CBWM.
+      return Some(anchor->GetRect());
+    }
+
+    // TODO(dshin): Already traversed up to find `containerChild`, and we're
+    // going to do it again here, which feels a little wasteful.
+    nsRect rectToContainerChild = anchor->GetRectRelativeToSelf();
+    nsLayoutUtils::TransformRect(anchor, containerChild, rectToContainerChild);
+
+    return Some(rectToContainerChild + containerChild->GetPosition());
+  }();
+  return rect.map([&](const nsRect& aRect) {
+    // We need to position the border box of the anchor within the abspos
+    // containing block's size - So the rectangle's size (i.e. Anchor size)
+    // stays the same, while "the outer rectangle" (i.e. The abspos cb size)
+    // "shrinks" by shifting the position.
+    const auto border = containingBlock->GetUsedBorder();
+    const nsPoint borderTopLeft{border.left, border.top};
+    return AnchorPosInfo{
+        .mRect = aRect - borderTopLeft,
+        .mContainingBlock = containingBlock,
+    };
+  });
+}
+
+bool Gecko_GetAnchorPosOffset(
+    const AnchorPosOffsetResolutionParams* aParams, const nsAtom* aAnchorName,
+    StylePhysicalSide aPropSide,
+    mozilla::StyleAnchorSideKeyword aAnchorSideKeyword, float aPercentage,
+    mozilla::Length* aOut) {
+  if (!aParams || !aParams->mBaseParams.mFrame) {
+    return false;
+  }
+  const auto info = GetAnchorPosRect(aParams->mBaseParams.mFrame, aAnchorName,
+                                     !aParams->mCBSize);
+  if (info.isNothing()) {
+    return false;
+  }
+  // Compute the offset here in C++, where translating between physical/logical
+  // coordinates is easier.
+  const auto& rect = info.ref().mRect;
+  const auto* containingBlock = info.ref().mContainingBlock;
+  const auto usesCBWM = AnchorSideUsesCBWM(aAnchorSideKeyword);
+  const auto cbwm = containingBlock->GetWritingMode();
+  const auto wm =
+      usesCBWM ? aParams->mBaseParams.mFrame->GetWritingMode() : cbwm;
+  const auto logicalCBSize = aParams->mCBSize
+                                 ? aParams->mCBSize->ConvertTo(wm, cbwm)
+                                 : containingBlock->PaddingSize(wm);
+  const LogicalRect logicalAnchorRect{wm, rect,
+                                      logicalCBSize.GetPhysicalSize(wm)};
+  const auto logicalPropSide = wm.LogicalSideForPhysicalSide(ToSide(aPropSide));
+  const auto propAxis = GetAxis(logicalPropSide);
+  const auto propEdge = GetEdge(logicalPropSide);
+
+  const auto anchorEdge = [&]() {
+    switch (aAnchorSideKeyword) {
+      case StyleAnchorSideKeyword::Left:
+        return GetEdge(wm.LogicalSideForPhysicalSide(eSideLeft));
+      case StyleAnchorSideKeyword::Right:
+        return GetEdge(wm.LogicalSideForPhysicalSide(eSideRight));
+      case StyleAnchorSideKeyword::Top:
+        return GetEdge(wm.LogicalSideForPhysicalSide(eSideTop));
+      case StyleAnchorSideKeyword::Bottom:
+        return GetEdge(wm.LogicalSideForPhysicalSide(eSideBottom));
+      case StyleAnchorSideKeyword::Inside:
+      case StyleAnchorSideKeyword::Outside:
+        return propEdge;
+      case StyleAnchorSideKeyword::Start:
+      case StyleAnchorSideKeyword::SelfStart:
+      case StyleAnchorSideKeyword::Center:
+        return LogicalEdge::Start;
+      case StyleAnchorSideKeyword::End:
+      case StyleAnchorSideKeyword::SelfEnd:
+        return LogicalEdge::End;
+    }
+    return LogicalEdge::Start;
+  }();
+
+  // Do we need to flip the computed offset by containing block's size?
+  const auto opposite =
+      propEdge != anchorEdge && propEdge != LogicalEdge::Start;
+  const auto size = logicalCBSize.Size(propAxis, wm);
+  const auto offset = anchorEdge == LogicalEdge::Start
+                          ? logicalAnchorRect.Start(propAxis, wm)
+                          : logicalAnchorRect.End(propAxis, wm);
+  const auto side = opposite ? size - offset : offset;
+  nscoord result = side;
+  if (aPercentage != 1.0f) {
+    // Apply the percentage value, with the percentage basis as the anchor
+    // element's size in the relevant axis.
+    const LogicalSize anchorSize{wm, rect.Size()};
+    result = side + (opposite ? -1 : 1) *
+                        NSToCoordRoundWithClamp(
+                            aPercentage *
+                            static_cast<float>(anchorSize.Size(propAxis, wm)));
+  }
+  *aOut = Length::FromPixels(CSSPixel::FromAppUnits(result));
+  return true;
+}
