@@ -18,6 +18,7 @@
 #include "mozilla/Maybe.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/WindowButtonType.h"
+#include "UniqueOrNonOwningPtr.h"
 #include "nsColor.h"
 #include "nsCoord.h"
 #include "nsMargin.h"
@@ -37,9 +38,11 @@ class nsIURI;
 class nsTextFrame;
 struct nsStyleDisplay;
 struct nsStyleVisibility;
+class nsComputedDOMStyle;
 namespace mozilla {
 class ComputedStyle;
 struct IntrinsicSize;
+struct ReflowInput;
 
 }  // namespace mozilla
 
@@ -372,10 +375,43 @@ struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStyleBackground {
   mozilla::StyleColor mBackgroundColor;
 };
 
+using AnchorResolvedMargin =
+    mozilla::UniqueOrNonOwningPtr<const mozilla::StyleMargin>;
+
+struct AnchorResolvedMarginHelper {
+  static const mozilla::StyleMargin& ZeroValue() {
+    static const auto value = mozilla::StyleMargin::LengthPercentage(
+        mozilla::StyleLengthPercentage::Zero());
+    return value;
+  }
+
+  // XXXjwatt: We need to store/pass the nsIFrame so that we can call
+  // nsIFrame::FindAnchorPosAnchor().
+  static AnchorResolvedMargin FromUnresolved(
+      const mozilla::StyleMargin& aValue,
+      mozilla::StylePositionProperty aPosition) {
+    if (!aValue.HasAnchorPositioningFunction()) {
+      return AnchorResolvedMargin::NonOwning(&aValue);
+    }
+    return ResolveAnchor(aValue, aPosition);
+  }
+
+ private:
+  static AnchorResolvedMargin Zero() {
+    return AnchorResolvedMargin::NonOwning(&ZeroValue());
+  }
+
+  static AnchorResolvedMargin ResolveAnchor(
+      const mozilla::StyleMargin& aValue,
+      mozilla::StylePositionProperty aPosition);
+};
+
 struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStyleMargin {
   STYLE_STRUCT(nsStyleMargin)
   nsStyleMargin();
 
+  // Returns false if any margin is layout-dependent in any way.
+  // Percentage values and/or `anchor-size()` will do this.
   bool GetMargin(nsMargin& aMargin) const {
     bool convertsToLength = mMargin.All(
         [](const auto& aLength) { return aLength.ConvertsToLength(); });
@@ -385,7 +421,7 @@ struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStyleMargin {
     }
 
     for (const auto side : mozilla::AllPhysicalSides()) {
-      aMargin.Side(side) = GetMargin(side).AsLengthPercentage().ToLength();
+      aMargin.Side(side) = mMargin.Get(side).AsLengthPercentage().ToLength();
     }
     return true;
   }
@@ -399,25 +435,26 @@ struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStyleMargin {
 
   // Return true if either the start or end side in the axis is 'auto'.
   // (defined in WritingModes.h since we need the full WritingMode type)
-  inline bool HasBlockAxisAuto(mozilla::WritingMode aWM) const;
-  inline bool HasInlineAxisAuto(mozilla::WritingMode aWM) const;
-  inline bool HasAuto(mozilla::LogicalAxis, mozilla::WritingMode) const;
+  inline bool HasBlockAxisAuto(mozilla::WritingMode aWM,
+                               mozilla::StylePositionProperty aPosition) const;
+  inline bool HasInlineAxisAuto(mozilla::WritingMode aWM,
+                                mozilla::StylePositionProperty aPosition) const;
+  inline bool HasAuto(mozilla::LogicalAxis, mozilla::WritingMode,
+                      mozilla::StylePositionProperty) const;
 
-  // TODO(dshin): The following functions are used as shims to deal
-  // anchor size functions as if it's zero, before the computation
-  // is implemented.
-  static const mozilla::StyleMargin kZeroMargin;
-  const mozilla::StyleMargin& GetMargin(mozilla::Side aSide) const {
-    const auto& result = mMargin.Get(aSide);
-    if (MOZ_UNLIKELY(result.IsAnchorSizeFunction())) {
-      return kZeroMargin;
-    }
-    return result;
+  // Attempt to return the resolved margin, resolving anchor functions, and
+  // using a dummy percentage basis. If the resulting value returns true for
+  // `HasPercent`, percentage value needs to be resolved with a proper basis at
+  // a later point.
+  AnchorResolvedMargin GetMargin(
+      mozilla::Side aSide, mozilla::StylePositionProperty aPosition) const {
+    return AnchorResolvedMarginHelper::FromUnresolved(mMargin.Get(aSide),
+                                                      aPosition);
   }
 
   bool MarginEquals(const nsStyleMargin& aOther) const {
     for (const auto side : mozilla::AllPhysicalSides()) {
-      if (GetMargin(side) != aOther.GetMargin(side)) {
+      if (mMargin.Get(side) != aOther.mMargin.Get(side)) {
         return false;
       }
     }
@@ -426,8 +463,9 @@ struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStyleMargin {
 
   // As with other logical-coordinate accessors, definitions for these
   // are found in WritingModes.h.
-  inline const mozilla::StyleMargin& GetMargin(mozilla::LogicalSide aSide,
-                                               mozilla::WritingMode aWM) const;
+  inline AnchorResolvedMargin GetMargin(
+      mozilla::LogicalSide aSide, mozilla::WritingMode aWM,
+      mozilla::StylePositionProperty aPosition) const;
 
   mozilla::StyleRect<mozilla::StyleMargin> mMargin;
   mozilla::StyleRect<mozilla::StyleLength> mScrollMargin;
@@ -703,59 +741,170 @@ struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStylePage {
   StylePageOrientation mPageOrientation = StylePageOrientation::Upright;
 };
 
-template <typename T>
-class AnchorResolved {
- public:
-  const T* operator->() const { return Ptr(); }
+using AnchorResolvedInset =
+    mozilla::UniqueOrNonOwningPtr<const mozilla::StyleInset>;
 
-  const T& operator*() const { return *Ptr(); }
+// Base set of parameters required to resolve a reference to an anchor.
+struct AnchorPosResolutionParams {
+  // Frame of the anchor positioned element.
+  // If nullptr, skips anchor lookup and returns invalid, resolving fallbacks.
+  const nsIFrame* mFrame;
+  // Position property of the element in question.
+  mozilla::StylePositionProperty mPosition;
 
- protected:
-  static AnchorResolved Evaluated(T&& aValue) {
-    return AnchorResolved{V{aValue}};
-  }
-
-  static AnchorResolved Unchanged(const T& aValue) {
-    return AnchorResolved{V{std::cref(aValue)}};
-  }
-
- private:
-  // Anchor resolution was not required, or resolves to a fallback.
-  // Note the storage of reference - Computed style values won't update in the
-  // middle of reflow, but take care not to keep this for too long.
-  using U = std::reference_wrapper<const T>;
-
-  // Resolved value & Invalid-At-Computed-Value-Time (IACVT) is stored as T.
-  using V = mozilla::Variant<U, T>;
-
-  explicit AnchorResolved(V&& aValue) : mValue{aValue} {}
-
-  const T* Ptr() const {
-    return mValue.match([](const U& aValue) { return &aValue.get(); },
-                        [](const T& aValue) { return &aValue; });
-  }
-
-  V mValue;
+  // Helper functions for creating anchor resolution parameters.
+  // Defined in corresponding header files.
+  static inline AnchorPosResolutionParams From(const nsIFrame* aFrame);
+  static inline AnchorPosResolutionParams From(const mozilla::ReflowInput* aRI);
+  static inline AnchorPosResolutionParams From(
+      const nsComputedDOMStyle* aComputedDOMStyle);
 };
 
-class AnchorResolvedInset final : public AnchorResolved<mozilla::StyleInset> {
- public:
-  AnchorResolvedInset(const mozilla::StyleInset& aValue,
-                      mozilla::StylePhysicalAxis aAxis,
-                      mozilla::StylePositionProperty aPosition);
-  AnchorResolvedInset(const mozilla::StyleInset& aValue,
-                      mozilla::LogicalAxis aAxis, mozilla::WritingMode aWM,
-                      mozilla::StylePositionProperty aPosition);
+// Set of parameters required to resolve the anchor's position offset in a
+// containing block.
+struct AnchorPosOffsetResolutionParams {
+  // Size of the containing block for the anchor positioned element.
+  // This needs to be set only if all of the following conditions are true:
+  //
+  //   * The resolution is happening during reflow (i.e. Containing block
+  //   doesn't have its rect set)
+  //   * The caller needs the correct size, not just its type (e.g. Just
+  //     checking `HasPercent()` of the inset resolved value)
+  const mozilla::LogicalSize* mCBSize;
+  AnchorPosResolutionParams mBaseParams;
+
+  static AnchorPosOffsetResolutionParams UseCBFrameSize(
+      const AnchorPosResolutionParams& aBaseParams) {
+    return {aBaseParams, nullptr};
+  }
+
+  static AnchorPosOffsetResolutionParams ExplicitCBFrameSize(
+      const AnchorPosResolutionParams& aBaseParams,
+      const mozilla::LogicalSize* aCBSize) {
+    return {aBaseParams, aCBSize};
+  }
 
  private:
-  static AnchorResolved<mozilla::StyleInset> FromUnresolved(
-      const mozilla::StyleInset& aValue, mozilla::StylePhysicalAxis aAxis,
+  AnchorPosOffsetResolutionParams(const AnchorPosResolutionParams& aBaseParams,
+                                  const mozilla::LogicalSize* aCBSize)
+      : mCBSize{aCBSize}, mBaseParams{aBaseParams} {}
+};
+
+struct AnchorResolvedInsetHelper {
+  static const mozilla::StyleInset& AutoValue() {
+    static const auto value = mozilla::StyleInset::Auto();
+    return value;
+  }
+
+  static AnchorResolvedInset FromUnresolved(
+      const mozilla::StyleInset& aValue, mozilla::Side aSide,
+      const AnchorPosOffsetResolutionParams& aParams) {
+    if (!aValue.HasAnchorPositioningFunction()) {
+      return AnchorResolvedInset::NonOwning(&aValue);
+    }
+    return ResolveAnchor(aValue, mozilla::ToStylePhysicalSide(aSide), aParams);
+  }
+
+ private:
+  static AnchorResolvedInset Auto() {
+    return AnchorResolvedInset::NonOwning(&AutoValue());
+  }
+
+  static AnchorResolvedInset ResolveAnchor(
+      const mozilla::StyleInset& aValue, mozilla::StylePhysicalSide aSide,
+      const AnchorPosOffsetResolutionParams& aParams);
+};
+
+using AnchorResolvedSize =
+    mozilla::UniqueOrNonOwningPtr<const mozilla::StyleSize>;
+
+struct AnchorResolvedSizeHelper {
+  static const mozilla::StyleSize& ZeroValue() {
+    static const auto value = mozilla::StyleSize::LengthPercentage(
+        mozilla::StyleLengthPercentage::Zero());
+    return value;
+  }
+
+  static const mozilla::StyleSize& MinContentValue() {
+    static const auto value = mozilla::StyleSize::MinContent();
+    return value;
+  }
+
+  static const mozilla::StyleSize& AutoValue() {
+    static const auto value = mozilla::StyleSize::Auto();
+    return value;
+  }
+
+  static AnchorResolvedSize FromUnresolved(
+      const mozilla::StyleSize& aValue,
+      mozilla::StylePositionProperty aPosition) {
+    if (!aValue.HasAnchorPositioningFunction()) {
+      return AnchorResolvedSize::NonOwning(&aValue);
+    }
+    return ResolveAnchor(aValue, aPosition);
+  }
+
+  static AnchorResolvedSize Overridden(const mozilla::StyleSize& aSize) {
+    return AnchorResolvedSize::NonOwning(&aSize);
+  }
+
+  static AnchorResolvedSize Zero() {
+    return AnchorResolvedSize::NonOwning(&ZeroValue());
+  }
+
+  static AnchorResolvedSize MinContent() {
+    return AnchorResolvedSize::NonOwning(&MinContentValue());
+  }
+
+  static AnchorResolvedSize Auto() {
+    return AnchorResolvedSize::NonOwning(&AutoValue());
+  }
+
+  static AnchorResolvedSize LengthPercentage(
+      const mozilla::StyleLengthPercentage& aLP) {
+    return mozilla::MakeUniqueOfUniqueOrNonOwning<const mozilla::StyleSize>(
+        aLP);
+  }
+
+ private:
+  static AnchorResolvedSize ResolveAnchor(
+      const mozilla::StyleSize& aValue,
+      mozilla::StylePositionProperty aPosition);
+};
+
+using AnchorResolvedMaxSize =
+    mozilla::UniqueOrNonOwningPtr<const mozilla::StyleMaxSize>;
+
+struct AnchorResolvedMaxSizeHelper {
+  static const mozilla::StyleMaxSize& MaxContentValue() {
+    static const auto value = mozilla::StyleMaxSize::MaxContent();
+    return value;
+  }
+  static const mozilla::StyleMaxSize& NoneValue() {
+    static const auto value = mozilla::StyleMaxSize::None();
+    return value;
+  }
+
+  static AnchorResolvedMaxSize FromUnresolved(
+      const mozilla::StyleMaxSize& aValue,
+      mozilla::StylePositionProperty aPosition) {
+    if (!aValue.HasAnchorPositioningFunction()) {
+      return AnchorResolvedMaxSize::NonOwning(&aValue);
+    }
+    return ResolveAnchor(aValue, aPosition);
+  }
+  static AnchorResolvedMaxSize MaxContent() {
+    return AnchorResolvedMaxSize::NonOwning(&MaxContentValue());
+  }
+
+  static AnchorResolvedMaxSize None() {
+    return AnchorResolvedMaxSize::NonOwning(&NoneValue());
+  }
+
+ private:
+  static AnchorResolvedMaxSize ResolveAnchor(
+      const mozilla::StyleMaxSize& aValue,
       mozilla::StylePositionProperty aPosition);
-  static AnchorResolved<mozilla::StyleInset> Invalid();
-  static AnchorResolved<mozilla::StyleInset> Evaluated(
-      mozilla::StyleLengthPercentage&& aLP);
-  static AnchorResolved<mozilla::StyleInset> Evaluated(
-      const mozilla::StyleLengthPercentage& aLP);
 };
 
 struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStylePosition {
@@ -781,17 +930,24 @@ struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStylePosition {
   // Returns whether we need to compute an hypothetical position if we were
   // absolutely positioned.
   bool NeedsHypotheticalPositionIfAbsPos() const {
-    return (GetAnchorResolvedInset(mozilla::eSideRight,
-                                   mozilla::StylePositionProperty::Absolute)
+    // NOTE(dshin): By passing in nullptr for frames, we're guaranteeing that
+    // no anchor is found, instead considering fallbacks or anchor-invalid
+    // values. Since anchor resolved value can never be auto, this is
+    // guaranteed to be correct, if not pessimistic.
+    // TODO(dshin): Probably nicer if we try to resolve. Two tricky parts:
+    //   * `CalcDifference`: Need to pass in the changing frame somehow.
+    //   * Reflow cases where containing block is not yet set (Mainly when
+    //     `nsBlockFrame` is about to run another reflow for clearance)
+    const auto anchorResolutionParams =
+        AnchorPosOffsetResolutionParams::UseCBFrameSize(
+            {nullptr, mozilla::StylePositionProperty::Absolute});
+    return (GetAnchorResolvedInset(mozilla::eSideRight, anchorResolutionParams)
                 ->IsAuto() &&
-            GetAnchorResolvedInset(mozilla::eSideLeft,
-                                   mozilla::StylePositionProperty::Absolute)
+            GetAnchorResolvedInset(mozilla::eSideLeft, anchorResolutionParams)
                 ->IsAuto()) ||
-           (GetAnchorResolvedInset(mozilla::eSideTop,
-                                   mozilla::StylePositionProperty::Absolute)
+           (GetAnchorResolvedInset(mozilla::eSideTop, anchorResolutionParams)
                 ->IsAuto() &&
-            GetAnchorResolvedInset(mozilla::eSideBottom,
-                                   mozilla::StylePositionProperty::Absolute)
+            GetAnchorResolvedInset(mozilla::eSideBottom, anchorResolutionParams)
                 ->IsAuto());
   }
 
@@ -884,75 +1040,68 @@ struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStylePosition {
   // given a WritingMode value. The definitions of these methods are
   // found in WritingModes.h (after the WritingMode class is fully
   // declared).
-  inline const StyleSize& ISize(WritingMode) const;
-  inline const StyleSize& MinISize(WritingMode) const;
-  inline const StyleMaxSize& MaxISize(WritingMode) const;
-  inline const StyleSize& BSize(WritingMode) const;
-  inline const StyleSize& MinBSize(WritingMode) const;
-  inline const StyleMaxSize& MaxBSize(WritingMode) const;
-  inline const StyleSize& Size(LogicalAxis, WritingMode) const;
-  inline const StyleSize& MinSize(LogicalAxis, WritingMode) const;
-  inline const StyleMaxSize& MaxSize(LogicalAxis, WritingMode) const;
-  inline bool ISizeDependsOnContainer(WritingMode) const;
-  inline bool MinISizeDependsOnContainer(WritingMode) const;
-  inline bool MaxISizeDependsOnContainer(WritingMode) const;
-  inline bool BSizeDependsOnContainer(WritingMode) const;
-  inline bool MinBSizeDependsOnContainer(WritingMode) const;
-  inline bool MaxBSizeDependsOnContainer(WritingMode) const;
+  inline AnchorResolvedSize ISize(WritingMode,
+                                  mozilla::StylePositionProperty) const;
+  inline AnchorResolvedSize MinISize(WritingMode,
+                                     mozilla::StylePositionProperty) const;
+  inline AnchorResolvedMaxSize MaxISize(WritingMode,
+                                        mozilla::StylePositionProperty) const;
+  inline AnchorResolvedSize BSize(WritingMode,
+                                  mozilla::StylePositionProperty) const;
+  inline AnchorResolvedSize MinBSize(WritingMode,
+                                     mozilla::StylePositionProperty) const;
+  inline AnchorResolvedMaxSize MaxBSize(WritingMode,
+                                        mozilla::StylePositionProperty) const;
+  inline AnchorResolvedSize Size(LogicalAxis, WritingMode,
+                                 mozilla::StylePositionProperty) const;
+  inline AnchorResolvedSize MinSize(LogicalAxis, WritingMode,
+                                    mozilla::StylePositionProperty) const;
+  inline AnchorResolvedMaxSize MaxSize(LogicalAxis, WritingMode,
+                                       mozilla::StylePositionProperty) const;
+  static inline bool ISizeDependsOnContainer(const AnchorResolvedSize&);
+  static inline bool MinISizeDependsOnContainer(const AnchorResolvedSize&);
+  static inline bool MaxISizeDependsOnContainer(const AnchorResolvedMaxSize&);
+  static inline bool BSizeDependsOnContainer(const AnchorResolvedSize&);
+  static inline bool MinBSizeDependsOnContainer(const AnchorResolvedSize&);
+  static inline bool MaxBSizeDependsOnContainer(const AnchorResolvedMaxSize&);
 
   // TODO(dshin): These inset getters are to be removed when
   // interleaving computation is implemented.
   AnchorResolvedInset GetAnchorResolvedInset(
-      mozilla::Side aSide, mozilla::StylePositionProperty aPosition) const;
-  AnchorResolvedInset GetAnchorResolvedInset(
+      mozilla::Side aSide,
+      const AnchorPosOffsetResolutionParams& aParams) const {
+    return AnchorResolvedInsetHelper::FromUnresolved(mOffset.Get(aSide), aSide,
+                                                     aParams);
+  }
+
+  inline AnchorResolvedInset GetAnchorResolvedInset(
       mozilla::LogicalSide aSide, WritingMode aWM,
-      mozilla::StylePositionProperty aPosition) const;
+      const AnchorPosOffsetResolutionParams& aParams) const;
 
-  // TODO(dshin): These size getters can be removed when anchor
-  // size is actually calculated.
-  static const StyleSize kAutoSize;
-  static const StyleMaxSize kNoneMaxSize;
-
-  const StyleSize& GetWidth() const {
-    if (MOZ_UNLIKELY(mWidth.IsAnchorSizeFunction())) {
-      return kAutoSize;
-    }
-    return mWidth;
+  AnchorResolvedSize GetWidth(mozilla::StylePositionProperty aProp) const {
+    return AnchorResolvedSizeHelper::FromUnresolved(mWidth, aProp);
   }
 
-  const StyleSize& GetHeight() const {
-    if (MOZ_UNLIKELY(mHeight.IsAnchorSizeFunction())) {
-      return kAutoSize;
-    }
-    return mHeight;
+  AnchorResolvedSize GetHeight(mozilla::StylePositionProperty aProp) const {
+    return AnchorResolvedSizeHelper::FromUnresolved(mHeight, aProp);
   }
 
-  const StyleSize& GetMinWidth() const {
-    if (MOZ_UNLIKELY(mMinWidth.IsAnchorSizeFunction())) {
-      return kAutoSize;
-    }
-    return mMinWidth;
+  AnchorResolvedSize GetMinWidth(mozilla::StylePositionProperty aProp) const {
+    return AnchorResolvedSizeHelper::FromUnresolved(mMinWidth, aProp);
   }
 
-  const StyleSize& GetMinHeight() const {
-    if (MOZ_UNLIKELY(mMinHeight.IsAnchorSizeFunction())) {
-      return kAutoSize;
-    }
-    return mMinHeight;
+  AnchorResolvedSize GetMinHeight(mozilla::StylePositionProperty aProp) const {
+    return AnchorResolvedSizeHelper::FromUnresolved(mMinHeight, aProp);
   }
 
-  const StyleMaxSize& GetMaxWidth() const {
-    if (MOZ_UNLIKELY(mMaxWidth.IsAnchorSizeFunction())) {
-      return kNoneMaxSize;
-    }
-    return mMaxWidth;
+  AnchorResolvedMaxSize GetMaxWidth(
+      mozilla::StylePositionProperty aProp) const {
+    return AnchorResolvedMaxSizeHelper::FromUnresolved(mMaxWidth, aProp);
   }
 
-  const StyleMaxSize& GetMaxHeight() const {
-    if (MOZ_UNLIKELY(mMaxHeight.IsAnchorSizeFunction())) {
-      return kNoneMaxSize;
-    }
-    return mMaxHeight;
+  AnchorResolvedMaxSize GetMaxHeight(
+      mozilla::StylePositionProperty aProp) const {
+    return AnchorResolvedMaxSizeHelper::FromUnresolved(mMaxHeight, aProp);
   }
 
  private:
@@ -1522,17 +1671,17 @@ struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStyleDisplay {
         // `appearance: textfield` should behave like `auto` on all elements
         // except <input type=search/number/password> elements, which we
         // identify using the internal -moz-default-appearance property.
-        if (mDefaultAppearance == mozilla::StyleAppearance::Searchfield ||
-            mDefaultAppearance == mozilla::StyleAppearance::NumberInput ||
+        if (mDefaultAppearance == mozilla::StyleAppearance::NumberInput ||
             mDefaultAppearance == mozilla::StyleAppearance::PasswordInput) {
           return mAppearance;
         }
         return mDefaultAppearance;
       case mozilla::StyleAppearance::MenulistButton:
         // `appearance: menulist-button` should behave like `auto` on all
-        // elements except for drop down selects, but since we have very little
-        // difference between menulist and menulist-button handling, we don't
-        // bother.
+        // elements except for drop down selects.
+        if (mDefaultAppearance == mozilla::StyleAppearance::Menulist) {
+          return mAppearance;
+        }
         return mDefaultAppearance;
       default:
         return mAppearance;
@@ -1593,6 +1742,8 @@ struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStyleDisplay {
            mozilla::StylePositionProperty::Fixed == mPosition;
   }
 
+  bool HasAnchorName() const { return !mAnchorName.IsEmpty(); }
+
   bool IsRubyDisplayType() const { return mDisplay.IsRuby(); }
 
   bool IsInternalRubyDisplayType() const { return mDisplay.IsInternalRuby(); }
@@ -1914,6 +2065,7 @@ struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStyleUIReset {
   }
 
   mozilla::StyleViewTransitionName mViewTransitionName;
+  mozilla::StyleViewTransitionClass mViewTransitionClass;
 };
 
 struct MOZ_NEEDS_MEMMOVABLE_MEMBERS nsStyleUI {
