@@ -44,7 +44,7 @@ nsresult SVGMarkerFrame::AttributeChanged(int32_t aNameSpaceID,
        aAttribute == nsGkAtoms::orient ||
        aAttribute == nsGkAtoms::preserveAspectRatio ||
        aAttribute == nsGkAtoms::viewBox)) {
-    SVGObserverUtils::InvalidateDirectRenderingObservers(this);
+    SVGObserverUtils::InvalidateRenderingObservers(this);
   }
 
   return SVGContainerFrame::AttributeChanged(aNameSpaceID, aAttribute,
@@ -134,8 +134,9 @@ void SVGMarkerFrame::PaintMark(gfxContext& aContext,
   ISVGDisplayableFrame* SVGFrame = do_QueryFrame(kid);
   // The CTM of each frame referencing us may be different.
   SVGFrame->NotifySVGChanged(ISVGDisplayableFrame::TRANSFORM_CHANGED);
-  RefPtr<SVGContextPaintImpl> contextPaint = new SVGContextPaintImpl();
-  contextPaint->Init(aContext.GetDrawTarget(), aContext.CurrentMatrixDouble(),
+  auto contextPaint = MakeRefPtr<SVGContextPaintImpl>();
+  contextPaint->Init(aContext.GetDrawTarget(),
+                     aToMarkedFrameUserSpace * aContext.CurrentMatrixDouble(),
                      aMarkedFrame, SVGContextPaint::GetContextPaint(marker),
                      aImgParams);
   AutoSetRestoreSVGContextPaint autoSetRestore(contextPaint,
