@@ -42,124 +42,133 @@ using namespace mozilla::dom;
 
 namespace mozilla {
 
-bool URLAndReferrerInfo::operator==(const URLAndReferrerInfo& aRHS) const {
-  bool uriEqual = false, referrerEqual = false;
-  this->mURI->Equals(aRHS.mURI, &uriEqual);
-  this->mReferrerInfo->Equals(aRHS.mReferrerInfo, &referrerEqual);
+/*
+ * This class contains URL and referrer information (referrer and referrer
+ * policy).
+ * We use it to pass to svg system instead of nsIURI. The object brings referrer
+ * and referrer policy so we can send correct Referer headers.
+ */
+class SVGReference {
+ public:
+  SVGReference(const nsAString& aLocalRef, nsIURI* aBase,
+               nsIReferrerInfo* aReferrerInfo)
+      : mLocalRef(aLocalRef), mURI(aBase), mReferrerInfo(aReferrerInfo) {
+    MOZ_ASSERT(nsContentUtils::IsLocalRefURL(mLocalRef));
+  }
 
+  SVGReference(const nsACString& aLocalRef, const URLExtraData& aExtraData)
+      : mURI(aExtraData.BaseURI()), mReferrerInfo(aExtraData.ReferrerInfo()) {
+    CopyUTF8toUTF16(aLocalRef, mLocalRef);
+    MOZ_ASSERT(nsContentUtils::IsLocalRefURL(mLocalRef));
+  }
+
+  SVGReference(nsIURI* aURI, nsIReferrerInfo* aReferrerInfo)
+      : mURI(aURI), mReferrerInfo(aReferrerInfo) {
+    MOZ_ASSERT(aURI);
+  }
+
+  SVGReference(nsIURI* aURI, const URLExtraData& aExtraData)
+      : mURI(aURI), mReferrerInfo(aExtraData.ReferrerInfo()) {
+    MOZ_ASSERT(aURI);
+  }
+
+  NS_INLINE_DECL_REFCOUNTING(SVGReference)
+
+  bool IsLocalRef() const { return !mLocalRef.IsEmpty(); }
+  const nsAString& GetLocalRef() const { return mLocalRef; }
+
+  nsIURI* GetURI() const { return mURI; }
+  nsIReferrerInfo* GetReferrerInfo() const { return mReferrerInfo; }
+
+  bool operator==(const SVGReference& aRHS) const;
+
+ private:
+  ~SVGReference() = default;
+
+  nsString mLocalRef;
+  // For local refs, this is the base URI that specified the URL. For non-local
+  // refs, this is the whole URI. This is needed so that we can distinguish URIs
+  // from an inner shadow tree and inherited ones.
+  const nsCOMPtr<nsIURI> mURI;
+  const nsCOMPtr<nsIReferrerInfo> mReferrerInfo;
+};
+
+bool SVGReference::operator==(const SVGReference& aRHS) const {
+  if (mLocalRef != aRHS.mLocalRef) {
+    return false;
+  }
+  bool uriEqual = false, referrerEqual = false;
+  mURI->Equals(aRHS.mURI, &uriEqual);
+  mReferrerInfo->Equals(aRHS.mReferrerInfo, &referrerEqual);
   return uriEqual && referrerEqual;
 }
 
-class URLAndReferrerInfoHashKey : public PLDHashEntryHdr {
+class SVGReferenceHashKey : public PLDHashEntryHdr {
  public:
-  using KeyType = const URLAndReferrerInfo*;
-  using KeyTypePointer = const URLAndReferrerInfo*;
+  using KeyType = const SVGReference*;
+  using KeyTypePointer = const SVGReference*;
 
-  explicit URLAndReferrerInfoHashKey(const URLAndReferrerInfo* aKey) noexcept
-      : mKey(aKey) {
-    MOZ_COUNT_CTOR(URLAndReferrerInfoHashKey);
+  explicit SVGReferenceHashKey(const SVGReference* aKey) noexcept : mKey(aKey) {
+    MOZ_COUNT_CTOR(SVGReferenceHashKey);
   }
-  URLAndReferrerInfoHashKey(URLAndReferrerInfoHashKey&& aToMove) noexcept
+  SVGReferenceHashKey(SVGReferenceHashKey&& aToMove) noexcept
       : PLDHashEntryHdr(std::move(aToMove)), mKey(std::move(aToMove.mKey)) {
-    MOZ_COUNT_CTOR(URLAndReferrerInfoHashKey);
+    MOZ_COUNT_CTOR(SVGReferenceHashKey);
   }
-  MOZ_COUNTED_DTOR(URLAndReferrerInfoHashKey)
+  MOZ_COUNTED_DTOR(SVGReferenceHashKey)
 
-  const URLAndReferrerInfo* GetKey() const { return mKey; }
+  const SVGReference* GetKey() const { return mKey; }
 
-  bool KeyEquals(const URLAndReferrerInfo* aKey) const {
+  bool KeyEquals(const SVGReference* aKey) const {
     if (!mKey) {
       return !aKey;
     }
     return *mKey == *aKey;
   }
 
-  static const URLAndReferrerInfo* KeyToPointer(
-      const URLAndReferrerInfo* aKey) {
+  static const SVGReference* KeyToPointer(const SVGReference* aKey) {
     return aKey;
   }
 
-  static PLDHashNumber HashKey(const URLAndReferrerInfo* aKey) {
-    if (!aKey) {
-      // If the key is null, return hash for empty string.
-      return HashString(""_ns);
-    }
+  static PLDHashNumber HashKey(const SVGReference* aKey) {
+    MOZ_ASSERT(aKey);
+
     nsAutoCString urlSpec, referrerSpec;
     // nsURIHashKey ignores GetSpec() failures, so we do too:
     Unused << aKey->GetURI()->GetSpec(urlSpec);
     return AddToHash(
-        HashString(urlSpec),
+        HashString(aKey->GetLocalRef()), HashString(urlSpec),
         static_cast<ReferrerInfo*>(aKey->GetReferrerInfo())->Hash());
   }
 
   enum { ALLOW_MEMMOVE = true };
 
  protected:
-  RefPtr<const URLAndReferrerInfo> mKey;
+  RefPtr<const SVGReference> mKey;
 };
 
-/**
- * Return a baseURL for resolving a local-ref URL.
- *
- * @param aContent an element which uses a local-ref property. Here are some
- *                 examples:
- *                   <rect fill=url(#foo)>
- *                   <circle clip-path=url(#foo)>
- *                   <use xlink:href="#foo">
- */
-static already_AddRefed<nsIURI> GetBaseURLForLocalRef(nsIContent* content,
-                                                      nsIURI* aURI) {
-  MOZ_ASSERT(content);
-
-  // Content is in a shadow tree.  If this URL was specified in the subtree
-  // referenced by the <use>, element, and that subtree came from a separate
-  // resource document, then we want the fragment-only URL to resolve to an
-  // element from the resource document.  Otherwise, the URL was specified
-  // somewhere in the document with the <use> element, and we want the
-  // fragment-only URL to resolve to an element in that document.
-  if (SVGUseElement* use = content->GetContainingSVGUseShadowHost()) {
-    if (nsIURI* originalURI = use->GetSourceDocURI()) {
-      bool isEqualsExceptRef = false;
-      aURI->EqualsExceptRef(originalURI, &isEqualsExceptRef);
-      if (isEqualsExceptRef) {
-        return do_AddRef(originalURI);
-      }
-    }
+static already_AddRefed<SVGReference> ResolveURLUsingLocalRef(
+    const StyleComputedUrl& aURL) {
+  if (aURL.IsLocalRef()) {
+    return MakeAndAddRef<SVGReference>(aURL.SpecifiedSerialization(),
+                                       aURL.ExtraData());
   }
 
-  // For a local-reference URL, resolve that fragment against the current
-  // document that relative URLs are resolved against.
-  return do_AddRef(content->OwnerDoc()->GetDocumentURI());
-}
-
-static already_AddRefed<URLAndReferrerInfo> ResolveURLUsingLocalRef(
-    nsIFrame* aFrame, const StyleComputedUrl& aURL) {
-  MOZ_ASSERT(aFrame);
-
   nsCOMPtr<nsIURI> uri = aURL.GetURI();
-
-  if (aURL.IsLocalRef()) {
-    uri = GetBaseURLForLocalRef(aFrame->GetContent(), uri);
-    uri = aURL.ResolveLocalRef(uri);
-  }
-
   if (!uri) {
     return nullptr;
   }
 
-  return do_AddRef(new URLAndReferrerInfo(uri, aURL.ExtraData()));
+  return do_AddRef(new SVGReference(uri, aURL.ExtraData()));
 }
 
-static already_AddRefed<URLAndReferrerInfo> ResolveURLUsingLocalRef(
+static already_AddRefed<SVGReference> ResolveURLUsingLocalRef(
     nsIContent* aContent, const nsAString& aURL) {
-  // Like GetBaseURLForLocalRef, we want to resolve the
-  // URL against any <use> element shadow tree's source document.
-  //
-  // Unlike GetBaseURLForLocalRef, we are assuming that the URL was specified
-  // directly on mFrame's content (because this ResolveURLUsingLocalRef
-  // overload is used for href="" attributes and not CSS URL values), so there
-  // is no need to check whether the URL was specified / inherited from
-  // outside the shadow tree.
+  // We want to resolve the URL against any <use> element shadow tree's source
+  // document. We are assuming that the URL was specified directly on mFrame's
+  // content (because this ResolveURLUsingLocalRef overload is used for href=""
+  // attributes and not CSS URL values), so there is no need to check whether
+  // the URL was specified / inherited from outside the shadow tree.
   nsIURI* base = nullptr;
   const Encoding* encoding = nullptr;
   if (SVGUseElement* use = aContent->GetContainingSVGUseShadowHost()) {
@@ -167,6 +176,15 @@ static already_AddRefed<URLAndReferrerInfo> ResolveURLUsingLocalRef(
     encoding = use->GetSourceDocCharacterSet();
   }
 
+  // There's no clear refererer policy spec about non-CSS SVG resource
+  // references Bug 1415044 to investigate which referrer we should use
+  nsIReferrerInfo* referrerInfo =
+      aContent->OwnerDoc()->ReferrerInfoForInternalCSSAndSVGResources();
+
+  if (nsContentUtils::IsLocalRefURL(aURL)) {
+    return MakeAndAddRef<SVGReference>(aURL, base, referrerInfo);
+  }
+
   if (!base) {
     base = aContent->OwnerDoc()->GetDocumentURI();
     encoding = aContent->OwnerDoc()->GetDocumentCharacterSet();
@@ -174,17 +192,10 @@ static already_AddRefed<URLAndReferrerInfo> ResolveURLUsingLocalRef(
 
   nsCOMPtr<nsIURI> uri;
   Unused << NS_NewURI(getter_AddRefs(uri), aURL, WrapNotNull(encoding), base);
-
   if (!uri) {
     return nullptr;
   }
-
-  // There's no clear refererer policy spec about non-CSS SVG resource
-  // references Bug 1415044 to investigate which referrer we should use
-  nsIReferrerInfo* referrerInfo =
-      aContent->OwnerDoc()->ReferrerInfoForInternalCSSAndSVGResources();
-
-  return do_AddRef(new URLAndReferrerInfo(uri, referrerInfo));
+  return do_AddRef(new SVGReference(uri, referrerInfo));
 }
 
 class SVGFilterObserverList;
@@ -346,7 +357,7 @@ class SVGIDRenderingObserver : public SVGRenderingObserver {
   // deriving class is fully constructed.
   using TargetIsValidCallback = bool (*)(const Element&);
   SVGIDRenderingObserver(
-      URLAndReferrerInfo* aURI, nsIContent* aObservingContent,
+      SVGReference* aReference, Element* aObservingElement,
       bool aReferenceImage,
       uint32_t aCallbacks = kAttributeChanged | kContentAppended |
                             kContentInserted | kContentWillBeRemoved,
@@ -372,8 +383,8 @@ class SVGIDRenderingObserver : public SVGRenderingObserver {
       //
       // TODO(emilio): Should we allow content observing its own descendants?
       // That seems potentially-bad as well.
-      if (observed->OwnerDoc() == mObservingContent->OwnerDoc() &&
-          nsContentUtils::ContentIsHostIncludingDescendantOf(mObservingContent,
+      if (observed->OwnerDoc() == mObservingElement->OwnerDoc() &&
+          nsContentUtils::ContentIsHostIncludingDescendantOf(mObservingElement,
                                                              observed)) {
         return false;
       }
@@ -423,7 +434,7 @@ class SVGIDRenderingObserver : public SVGRenderingObserver {
   };
 
   ElementTracker mObservedElementTracker;
-  RefPtr<Element> mObservingContent;
+  RefPtr<Element> mObservingElement;
   bool mTargetIsValid = false;
   TargetIsValidCallback mTargetIsValidCallback;
 };
@@ -445,30 +456,34 @@ class SVGIDRenderingObserver : public SVGRenderingObserver {
  * benefits/necessity of maintaining a second observer list.
  */
 SVGIDRenderingObserver::SVGIDRenderingObserver(
-    URLAndReferrerInfo* aURI, nsIContent* aObservingContent,
-    bool aReferenceImage, uint32_t aCallbacks,
-    TargetIsValidCallback aTargetIsValidCallback)
+    SVGReference* aReference, Element* aObservingElement, bool aReferenceImage,
+    uint32_t aCallbacks, TargetIsValidCallback aTargetIsValidCallback)
     : SVGRenderingObserver(aCallbacks),
       mObservedElementTracker(this),
-      mObservingContent(aObservingContent->AsElement()),
+      mObservingElement(aObservingElement),
       mTargetIsValidCallback(aTargetIsValidCallback) {
   // Start watching the target element
-  nsIURI* uri = nullptr;
-  nsIReferrerInfo* referrerInfo = nullptr;
-  if (aURI) {
-    uri = aURI->GetURI();
-    referrerInfo = aURI->GetReferrerInfo();
+  if (aReference) {
+    if (aReference->IsLocalRef()) {
+      mObservedElementTracker.ResetToLocalFragmentID(
+          *aObservingElement, aReference->GetLocalRef(), aReference->GetURI(),
+          aReference->GetReferrerInfo(), aReferenceImage);
+    } else {
+      mObservedElementTracker.ResetToURIWithFragmentID(
+          *aObservingElement, aReference->GetURI(),
+          aReference->GetReferrerInfo(), aReferenceImage);
+    }
+  } else {
+    mObservedElementTracker.Unlink();
   }
 
-  mObservedElementTracker.ResetToURIWithFragmentID(
-      aObservingContent, uri, referrerInfo, aReferenceImage);
   TargetChanged();
   StartObserving();
 }
 
 void SVGIDRenderingObserver::Traverse(nsCycleCollectionTraversalCallback* aCB) {
-  NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(*aCB, "mObservingContent");
-  aCB->NoteXPCOMChild(mObservingContent);
+  NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(*aCB, "mObservingElement");
+  aCB->NoteXPCOMChild(mObservingElement);
   mObservedElementTracker.Traverse(aCB);
 }
 
@@ -480,17 +495,30 @@ void SVGIDRenderingObserver::OnRenderingChange() {
   }
 }
 
+// Convenience function to return aFrame->GetContent() as an Element* if the
+// content pointer is non-null (or just return nullptr otherwise).
+// (AsElement itself isn't callable on null pointers.)
+static Element* GetFrameContentAsElement(nsIFrame* aFrame) {
+  MOZ_ASSERT(aFrame, "Expecting a non-null frame");
+  auto* content = aFrame->GetContent();
+  if (content) {
+    return content->AsElement();
+  }
+  return nullptr;
+}
+
 class SVGRenderingObserverProperty : public SVGIDRenderingObserver {
  public:
   NS_DECL_ISUPPORTS
 
   SVGRenderingObserverProperty(
-      URLAndReferrerInfo* aURI, nsIFrame* aFrame, bool aReferenceImage,
+      SVGReference* aReference, nsIFrame* aFrame, bool aReferenceImage,
       uint32_t aCallbacks = kAttributeChanged | kContentAppended |
                             kContentInserted | kContentWillBeRemoved,
       TargetIsValidCallback aTargetIsValidCallback = nullptr)
-      : SVGIDRenderingObserver(aURI, aFrame->GetContent(), aReferenceImage,
-                               aCallbacks, aTargetIsValidCallback),
+      : SVGIDRenderingObserver(aReference, GetFrameContentAsElement(aFrame),
+                               aReferenceImage, aCallbacks,
+                               aTargetIsValidCallback),
         mFrameReference(aFrame) {}
 
  protected:
@@ -530,9 +558,9 @@ static bool IsSVGGeometryElement(const Element& aObserved) {
 
 class SVGTextPathObserver final : public SVGRenderingObserverProperty {
  public:
-  SVGTextPathObserver(URLAndReferrerInfo* aURI, nsIFrame* aFrame,
+  SVGTextPathObserver(SVGReference* aReference, nsIFrame* aFrame,
                       bool aReferenceImage)
-      : SVGRenderingObserverProperty(aURI, aFrame, aReferenceImage,
+      : SVGRenderingObserverProperty(aReference, aFrame, aReferenceImage,
                                      kAttributeChanged, IsSVGGeometryElement) {}
 
  protected:
@@ -580,9 +608,10 @@ class SVGMPathObserver final : public SVGIDRenderingObserver {
  public:
   NS_DECL_ISUPPORTS
 
-  SVGMPathObserver(URLAndReferrerInfo* aURI, SVGMPathElement* aElement)
-      : SVGIDRenderingObserver(aURI, aElement, /* aReferenceImage = */ false,
-                               kAttributeChanged, IsSVGGeometryElement) {}
+  SVGMPathObserver(SVGReference* aReference, SVGMPathElement* aElement)
+      : SVGIDRenderingObserver(aReference, aElement,
+                               /* aReferenceImage = */ false, kAttributeChanged,
+                               IsSVGGeometryElement) {}
 
  protected:
   virtual ~SVGMPathObserver() = default;  // non-public
@@ -599,15 +628,15 @@ void SVGMPathObserver::OnRenderingChange() {
     return;
   }
 
-  auto* element = static_cast<SVGMPathElement*>(mObservingContent.get());
+  auto* element = static_cast<SVGMPathElement*>(mObservingElement.get());
   element->NotifyParentOfMpathChange();
 }
 
 class SVGMarkerObserver final : public SVGRenderingObserverProperty {
  public:
-  SVGMarkerObserver(URLAndReferrerInfo* aURI, nsIFrame* aFrame,
+  SVGMarkerObserver(SVGReference* aReference, nsIFrame* aFrame,
                     bool aReferenceImage)
-      : SVGRenderingObserverProperty(aURI, aFrame, aReferenceImage,
+      : SVGRenderingObserverProperty(aReference, aFrame, aReferenceImage,
                                      kAttributeChanged | kContentAppended |
                                          kContentInserted |
                                          kContentWillBeRemoved) {}
@@ -643,9 +672,9 @@ void SVGMarkerObserver::OnRenderingChange() {
 
 class SVGPaintingProperty : public SVGRenderingObserverProperty {
  public:
-  SVGPaintingProperty(URLAndReferrerInfo* aURI, nsIFrame* aFrame,
+  SVGPaintingProperty(SVGReference* aReference, nsIFrame* aFrame,
                       bool aReferenceImage)
-      : SVGRenderingObserverProperty(aURI, aFrame, aReferenceImage) {}
+      : SVGRenderingObserverProperty(aReference, aFrame, aReferenceImage) {}
 
  protected:
   void OnRenderingChange() override;
@@ -673,8 +702,8 @@ void SVGPaintingProperty::OnRenderingChange() {
 // have to be an SVG element.
 class SVGMozElementObserver final : public SVGPaintingProperty {
  public:
-  SVGMozElementObserver(URLAndReferrerInfo* aURI, nsIFrame* aFrame)
-      : SVGPaintingProperty(aURI, aFrame, /* aReferenceImage = */ true) {}
+  SVGMozElementObserver(SVGReference* aReference, nsIFrame* aFrame)
+      : SVGPaintingProperty(aReference, aFrame, /* aReferenceImage = */ true) {}
 
   // We only return true here because GetAndObserveBackgroundImage uses us
   // to implement observing of arbitrary elements (including HTML elements)
@@ -753,9 +782,9 @@ static bool IsSVGFilterElement(const Element& aObserved) {
  */
 class SVGFilterObserver final : public SVGIDRenderingObserver {
  public:
-  SVGFilterObserver(URLAndReferrerInfo* aURI, nsIContent* aObservingContent,
+  SVGFilterObserver(SVGReference* aReference, Element* aObservingElement,
                     SVGFilterObserverList* aFilterChainObserver)
-      : SVGIDRenderingObserver(aURI, aObservingContent, false,
+      : SVGIDRenderingObserver(aReference, aObservingElement, false,
                                kAttributeChanged | kContentAppended |
                                    kContentInserted | kContentWillBeRemoved,
                                IsSVGFilterElement),
@@ -793,13 +822,13 @@ NS_IMPL_CYCLE_COLLECTION_CLASS(SVGFilterObserver)
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(SVGFilterObserver)
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mObservedElementTracker)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mObservingContent)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mObservingElement)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(SVGFilterObserver)
   tmp->StopObserving();
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mObservedElementTracker);
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mObservingContent)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mObservingElement)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 SVGFilterFrame* SVGFilterObserver::GetAndObserveFilterFrame() {
@@ -833,7 +862,7 @@ NS_INTERFACE_MAP_END
 class SVGFilterObserverList : public ISVGFilterObserverList {
  public:
   SVGFilterObserverList(Span<const StyleFilter> aFilters,
-                        nsIContent* aFilteredElement,
+                        Element* aFilteredElement,
                         nsIFrame* aFilteredFrame = nullptr);
 
   const nsTArray<RefPtr<SVGFilterObserver>>& GetObservers() const override {
@@ -845,7 +874,7 @@ class SVGFilterObserverList : public ISVGFilterObserverList {
   NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(SVGFilterObserverList,
                                            ISVGFilterObserverList)
 
-  virtual void OnRenderingChange(Element* aObservingContent) = 0;
+  virtual void OnRenderingChange(Element* aObservingElement) = 0;
 
  protected:
   virtual ~SVGFilterObserverList();
@@ -867,7 +896,7 @@ void SVGFilterObserver::OnRenderingChange() {
   }
 
   if (mFilterObserverList) {
-    mFilterObserverList->OnRenderingChange(mObservingContent);
+    mFilterObserverList->OnRenderingChange(mObservingElement);
   }
 }
 
@@ -893,7 +922,7 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(SVGFilterObserverList)
 NS_INTERFACE_MAP_END
 
 SVGFilterObserverList::SVGFilterObserverList(Span<const StyleFilter> aFilters,
-                                             nsIContent* aFilteredElement,
+                                             Element* aFilteredElement,
                                              nsIFrame* aFilteredFrame) {
   for (const auto& filter : aFilters) {
     if (!filter.IsUrl()) {
@@ -904,16 +933,7 @@ SVGFilterObserverList::SVGFilterObserverList(Span<const StyleFilter> aFilters,
 
     // aFilteredFrame can be null if this filter belongs to a
     // CanvasRenderingContext2D.
-    RefPtr<URLAndReferrerInfo> filterURL;
-    if (aFilteredFrame) {
-      filterURL = ResolveURLUsingLocalRef(aFilteredFrame, url);
-    } else {
-      nsCOMPtr<nsIURI> resolvedURI = url.ResolveLocalRef(aFilteredElement);
-      if (resolvedURI) {
-        filterURL = new URLAndReferrerInfo(resolvedURI, url.ExtraData());
-      }
-    }
-
+    RefPtr<SVGReference> filterURL = ResolveURLUsingLocalRef(url);
     auto observer =
         MakeRefPtr<SVGFilterObserver>(filterURL, aFilteredElement, this);
     mObservers.AppendElement(std::move(observer));
@@ -926,16 +946,17 @@ class SVGFilterObserverListForCSSProp final : public SVGFilterObserverList {
  public:
   SVGFilterObserverListForCSSProp(Span<const StyleFilter> aFilters,
                                   nsIFrame* aFilteredFrame)
-      : SVGFilterObserverList(aFilters, aFilteredFrame->GetContent(),
+      : SVGFilterObserverList(aFilters,
+                              GetFrameContentAsElement(aFilteredFrame),
                               aFilteredFrame) {}
 
  protected:
-  void OnRenderingChange(Element* aObservingContent) override;
+  void OnRenderingChange(Element* aObservingElement) override;
 };
 
 void SVGFilterObserverListForCSSProp::OnRenderingChange(
-    Element* aObservingContent) {
-  nsIFrame* frame = aObservingContent->GetPrimaryFrame();
+    Element* aObservingElement) {
+  nsIFrame* frame = aObservingElement->GetPrimaryFrame();
   if (!frame) {
     return;
   }
@@ -955,7 +976,7 @@ void SVGFilterObserverListForCSSProp::OnRenderingChange(
     changeHint |= nsChangeHint_UpdateOverflow;
   }
   frame->PresContext()->RestyleManager()->PostRestyleEvent(
-      aObservingContent, RestyleHint{0}, changeHint);
+      aObservingElement, RestyleHint{0}, changeHint);
 }
 
 class SVGFilterObserverListForCanvasContext final
@@ -966,7 +987,7 @@ class SVGFilterObserverListForCanvasContext final
                                         Span<const StyleFilter> aFilters)
       : SVGFilterObserverList(aFilters, aCanvasElement), mContext(aContext) {}
 
-  void OnRenderingChange(Element* aObservingContent) override;
+  void OnRenderingChange(Element* aObservingElement) override;
   void Detach() override { mContext = nullptr; }
 
  private:
@@ -974,7 +995,7 @@ class SVGFilterObserverListForCanvasContext final
 };
 
 void SVGFilterObserverListForCanvasContext::OnRenderingChange(
-    Element* aObservingContent) {
+    Element* aObservingElement) {
   if (!mContext) {
     NS_WARNING(
         "GFX: This should never be called without a context, except during "
@@ -1018,22 +1039,26 @@ SVGMaskObserverList::SVGMaskObserverList(nsIFrame* aFrame) : mFrame(aFrame) {
   for (uint32_t i = 0; i < svgReset->mMask.mImageCount; i++) {
     const StyleComputedUrl* data =
         svgReset->mMask.mLayers[i].mImage.GetImageRequestURLValue();
-    RefPtr<URLAndReferrerInfo> maskUri;
+    RefPtr<SVGReference> maskUri;
     if (data) {
-      maskUri = ResolveURLUsingLocalRef(aFrame, *data);
+      maskUri = ResolveURLUsingLocalRef(*data);
     }
 
     bool hasRef = false;
     if (maskUri) {
-      maskUri->GetURI()->GetHasRef(&hasRef);
+      if (maskUri->IsLocalRef()) {
+        hasRef = true;
+      } else {
+        maskUri->GetURI()->GetHasRef(&hasRef);
+      }
     }
 
-    // Accrording to maskUri, SVGPaintingProperty's ctor may trigger an
-    // external SVG resource download, so we should pass maskUri in only if
-    // maskUri has a chance pointing to an SVG mask resource.
+    // According to maskUri, SVGPaintingProperty's ctor may trigger an external
+    // SVG resource download, so we should pass maskUri in only if maskUri has a
+    // chance pointing to an SVG mask resource.
     //
-    // And, an URL may refer to an SVG mask resource if it consists of
-    // a fragment.
+    // And, an URL may refer to an SVG mask resource if it consists of a
+    // fragment.
     auto prop = MakeRefPtr<SVGPaintingProperty>(
         hasRef ? maskUri.get() : nullptr, aFrame, false);
     mProperties.AppendElement(std::move(prop));
@@ -1065,9 +1090,10 @@ class SVGTemplateElementObserver : public SVGIDRenderingObserver {
  public:
   NS_DECL_ISUPPORTS
 
-  SVGTemplateElementObserver(URLAndReferrerInfo* aURI, nsIFrame* aFrame,
+  SVGTemplateElementObserver(SVGReference* aReference, nsIFrame* aFrame,
                              bool aReferenceImage)
-      : SVGIDRenderingObserver(aURI, aFrame->GetContent(), aReferenceImage,
+      : SVGIDRenderingObserver(aReference, GetFrameContentAsElement(aFrame),
+                               aReferenceImage,
                                kAttributeChanged | kContentAppended |
                                    kContentInserted | kContentWillBeRemoved),
         mFrameReference(aFrame) {}
@@ -1228,7 +1254,7 @@ void SVGRenderingObserver::DebugObserverSet() {
 #endif
 
 using URIObserverHashtable =
-    nsInterfaceHashtable<URLAndReferrerInfoHashKey, nsIMutationObserver>;
+    nsInterfaceHashtable<SVGReferenceHashKey, nsIMutationObserver>;
 
 using PaintingPropertyDescriptor =
     const FramePropertyDescriptor<SVGPaintingProperty>*;
@@ -1262,9 +1288,9 @@ NS_DECLARE_FRAME_PROPERTY_RELEASABLE(OffsetPathProperty,
                                      SVGRenderingObserverProperty)
 
 template <class T>
-static T* GetEffectProperty(URLAndReferrerInfo* aURI, nsIFrame* aFrame,
+static T* GetEffectProperty(SVGReference* aReference, nsIFrame* aFrame,
                             const FramePropertyDescriptor<T>* aProperty) {
-  if (!aURI) {
+  if (!aReference) {
     return nullptr;
   }
 
@@ -1274,25 +1300,25 @@ static T* GetEffectProperty(URLAndReferrerInfo* aURI, nsIFrame* aFrame,
     MOZ_ASSERT(prop, "this property should only store non-null values");
     return prop;
   }
-  prop = new T(aURI, aFrame, false);
+  prop = new T(aReference, aFrame, false);
   NS_ADDREF(prop);
   aFrame->AddProperty(aProperty, prop);
   return prop;
 }
 
 static SVGPaintingProperty* GetPaintingProperty(
-    URLAndReferrerInfo* aURI, nsIFrame* aFrame,
+    SVGReference* aReference, nsIFrame* aFrame,
     const FramePropertyDescriptor<SVGPaintingProperty>* aProperty) {
-  return GetEffectProperty(aURI, aFrame, aProperty);
+  return GetEffectProperty(aReference, aFrame, aProperty);
 }
 
-static already_AddRefed<URLAndReferrerInfo> GetMarkerURI(
+static already_AddRefed<SVGReference> GetMarkerURI(
     nsIFrame* aFrame, const StyleUrlOrNone nsStyleSVG::* aMarker) {
   const StyleUrlOrNone& url = aFrame->StyleSVG()->*aMarker;
   if (url.IsNone()) {
     return nullptr;
   }
-  return ResolveURLUsingLocalRef(aFrame, url.AsUrl());
+  return ResolveURLUsingLocalRef(url.AsUrl());
 }
 
 bool SVGObserverUtils::GetAndObserveMarkers(nsIFrame* aMarkedFrame,
@@ -1304,7 +1330,7 @@ bool SVGObserverUtils::GetAndObserveMarkers(nsIFrame* aMarkedFrame,
              "Bad frame");
 
   bool foundMarker = false;
-  RefPtr<URLAndReferrerInfo> markerURL;
+  RefPtr<SVGReference> markerURL;
   SVGMarkerObserver* observer;
   nsIFrame* marker;
 
@@ -1434,8 +1460,7 @@ static SVGPaintingProperty* GetOrCreateClipPathObserver(
     return nullptr;
   }
   const auto& url = svgStyleReset->mClipPath.AsUrl();
-  RefPtr<URLAndReferrerInfo> pathURI =
-      ResolveURLUsingLocalRef(aClippedFrame, url);
+  RefPtr<SVGReference> pathURI = ResolveURLUsingLocalRef(url);
   return GetPaintingProperty(pathURI, aClippedFrame, ClipPathProperty());
 }
 
@@ -1471,7 +1496,7 @@ static SVGRenderingObserverProperty* GetOrCreateGeometryObserver(
     return nullptr;
   }
   const auto& url = disp->mOffsetPath.AsUrl();
-  RefPtr<URLAndReferrerInfo> pathURI = ResolveURLUsingLocalRef(aFrame, url);
+  RefPtr<SVGReference> pathURI = ResolveURLUsingLocalRef(url);
   return GetEffectProperty(pathURI, aFrame, OffsetPathProperty());
 }
 
@@ -1573,7 +1598,7 @@ SVGGeometryElement* SVGObserverUtils::GetAndObserveTextPathsPath(
       return nullptr;  // no URL
     }
 
-    RefPtr<URLAndReferrerInfo> target = ResolveURLUsingLocalRef(content, href);
+    RefPtr<SVGReference> target = ResolveURLUsingLocalRef(content, href);
 
     property =
         GetEffectProperty(target, aTextPathFrame, HrefAsTextPathProperty());
@@ -1595,7 +1620,7 @@ SVGGeometryElement* SVGObserverUtils::GetAndObserveMPathsPath(
       return nullptr;  // no URL
     }
 
-    RefPtr<URLAndReferrerInfo> target =
+    RefPtr<SVGReference> target =
         ResolveURLUsingLocalRef(aSVGMPathElement, href);
 
     aSVGMPathElement->mMPathObserver =
@@ -1644,7 +1669,7 @@ nsIFrame* SVGObserverUtils::GetAndObserveTemplate(
       return nullptr;  // no URL
     }
 
-    RefPtr<URLAndReferrerInfo> info =
+    RefPtr<SVGReference> info =
         ResolveURLUsingLocalRef(aFrame->GetContent(), href);
 
     observer = GetEffectProperty(info, aFrame, HrefToTemplateProperty());
@@ -1668,18 +1693,12 @@ Element* SVGObserverUtils::GetAndObserveBackgroundImage(nsIFrame* aFrame,
   } else {
     MOZ_ASSERT(hashtable, "this property should only store non-null values");
   }
-
-  nsAutoString elementId = u"#"_ns + nsDependentAtomString(aHref);
-  nsCOMPtr<nsIURI> targetURI;
-  nsContentUtils::NewURIWithDocumentCharset(
-      getter_AddRefs(targetURI), elementId,
-      aFrame->GetContent()->GetUncomposedDoc(),
-      aFrame->GetContent()->GetBaseURI());
+  nsAutoString localRef = u"#"_ns + nsDependentAtomString(aHref);
+  auto* doc = aFrame->GetContent()->OwnerDoc();
+  nsIURI* baseURI = aFrame->GetContent()->GetBaseURI();
   nsIReferrerInfo* referrerInfo =
-      aFrame->GetContent()
-          ->OwnerDoc()
-          ->ReferrerInfoForInternalCSSAndSVGResources();
-  auto url = MakeRefPtr<URLAndReferrerInfo>(targetURI, referrerInfo);
+      doc->ReferrerInfoForInternalCSSAndSVGResources();
+  auto url = MakeRefPtr<SVGReference>(localRef, baseURI, referrerInfo);
 
   return static_cast<SVGMozElementObserver*>(
              hashtable
@@ -1725,8 +1744,8 @@ SVGPaintServerFrame* SVGObserverUtils::GetAndObservePaintServer(
     return nullptr;
   }
 
-  RefPtr<URLAndReferrerInfo> paintServerURL = ResolveURLUsingLocalRef(
-      paintedFrame, (svgStyle->*aPaint).kind.AsPaintServer());
+  RefPtr<SVGReference> paintServerURL =
+      ResolveURLUsingLocalRef((svgStyle->*aPaint).kind.AsPaintServer());
 
   MOZ_ASSERT(aPaint == &nsStyleSVG::mFill || aPaint == &nsStyleSVG::mStroke);
   PaintingPropertyDescriptor propDesc =
@@ -1739,8 +1758,8 @@ SVGPaintServerFrame* SVGObserverUtils::GetAndObservePaintServer(
 }
 
 void SVGObserverUtils::UpdateEffects(nsIFrame* aFrame) {
-  NS_ASSERTION(aFrame->GetContent()->IsElement(),
-               "aFrame's content should be an element");
+  NS_ASSERTION(!aFrame->GetContent() || aFrame->GetContent()->IsElement(),
+               "aFrame's content (if non-null) should be an element");
 
   aFrame->RemoveProperty(BackdropFilterProperty());
   aFrame->RemoveProperty(FilterProperty());
@@ -1762,7 +1781,7 @@ void SVGObserverUtils::UpdateEffects(nsIFrame* aFrame) {
   if (aFrame->IsSVGGeometryFrame() &&
       static_cast<SVGGeometryElement*>(aFrame->GetContent())->IsMarkable()) {
     // Set marker properties here to avoid reference loops
-    RefPtr<URLAndReferrerInfo> markerURL =
+    RefPtr<SVGReference> markerURL =
         GetMarkerURI(aFrame, &nsStyleSVG::mMarkerStart);
     GetEffectProperty(markerURL, aFrame, MarkerStartProperty());
     markerURL = GetMarkerURI(aFrame, &nsStyleSVG::mMarkerMid);
