@@ -25,6 +25,7 @@ namespace mozilla {
 
 class CharIterator;
 class DisplaySVGText;
+class SVGContextPaint;
 class SVGTextFrame;
 class TextFrameIterator;
 class TextNodeCorrespondenceRecorder;
@@ -179,15 +180,13 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
   using DrawTarget = gfx::DrawTarget;
   using Path = gfx::Path;
   using Point = gfx::Point;
+  using Rect = gfx::Rect;
 
  protected:
   explicit SVGTextFrame(ComputedStyle* aStyle, nsPresContext* aPresContext)
-      : SVGDisplayContainerFrame(aStyle, aPresContext, kClassID),
-        mTrailingUndisplayedCharacters(0),
-        mFontSizeScaleFactor(1.0f),
-        mLastContextScale(1.0f),
-        mLengthAdjustScaleFactor(1.0f) {
-    AddStateBits(NS_STATE_SVG_TEXT_CORRESPONDENCE_DIRTY |
+      : SVGDisplayContainerFrame(aStyle, aPresContext, kClassID) {
+    AddStateBits(NS_FRAME_SVG_LAYOUT | NS_FRAME_IS_SVG_TEXT |
+                 NS_STATE_SVG_TEXT_CORRESPONDENCE_DIRTY |
                  NS_STATE_SVG_POSITIONING_DIRTY);
   }
 
@@ -198,21 +197,23 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
   NS_DECL_FRAMEARENA_HELPERS(SVGTextFrame)
 
   // nsIFrame:
-  virtual void Init(nsIContent* aContent, nsContainerFrame* aParent,
-                    nsIFrame* aPrevInFlow) override;
+  void Init(nsIContent* aContent, nsContainerFrame* aParent,
+            nsIFrame* aPrevInFlow) override;
 
-  virtual nsresult AttributeChanged(int32_t aNamespaceID, nsAtom* aAttribute,
-                                    int32_t aModType) override;
+  void DidSetComputedStyle(ComputedStyle* aOldComputedStyle) override;
 
-  virtual nsContainerFrame* GetContentInsertionFrame() override {
+  nsresult AttributeChanged(int32_t aNamespaceID, nsAtom* aAttribute,
+                            int32_t aModType) override;
+
+  nsContainerFrame* GetContentInsertionFrame() override {
     return PrincipalChildList().FirstChild()->GetContentInsertionFrame();
   }
 
-  virtual void BuildDisplayList(nsDisplayListBuilder* aBuilder,
-                                const nsDisplayListSet& aLists) override;
+  void BuildDisplayList(nsDisplayListBuilder* aBuilder,
+                        const nsDisplayListSet& aLists) override;
 
 #ifdef DEBUG_FRAME_DUMP
-  virtual nsresult GetFrameName(nsAString& aResult) const override {
+  nsresult GetFrameName(nsAString& aResult) const override {
     return MakeFrameName(u"SVGText"_ns, aResult);
   }
 #endif
@@ -220,18 +221,17 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
   /**
    * Finds the nsTextFrame for the closest rendered run to the specified point.
    */
-  virtual void FindCloserFrameForSelection(
+  void FindCloserFrameForSelection(
       const nsPoint& aPoint, FrameWithDistance* aCurrentBestFrame) override;
 
   // ISVGDisplayableFrame interface:
-  virtual void NotifySVGChanged(uint32_t aFlags) override;
-  virtual void PaintSVG(gfxContext& aContext, const gfxMatrix& aTransform,
-                        imgDrawingParams& aImgParams,
-                        const nsIntRect* aDirtyRect = nullptr) override;
-  virtual nsIFrame* GetFrameForPoint(const gfxPoint& aPoint) override;
-  virtual void ReflowSVG() override;
-  virtual SVGBBox GetBBoxContribution(const Matrix& aToBBoxUserspace,
-                                      uint32_t aFlags) override;
+  void NotifySVGChanged(uint32_t aFlags) override;
+  void PaintSVG(gfxContext& aContext, const gfxMatrix& aTransform,
+                imgDrawingParams& aImgParams) override;
+  nsIFrame* GetFrameForPoint(const gfxPoint& aPoint) override;
+  void ReflowSVG() override;
+  SVGBBox GetBBoxContribution(const Matrix& aToBBoxUserspace,
+                              uint32_t aFlags) override;
 
   // SVG DOM text methods:
   uint32_t GetNumberOfChars(nsIContent* aContent);
@@ -240,9 +240,20 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
                                                    uint32_t charnum,
                                                    uint32_t nchars,
                                                    ErrorResult& aRv);
-  MOZ_CAN_RUN_SCRIPT
-  float GetSubStringLength(nsIContent* aContent, uint32_t charnum,
-                           uint32_t nchars, ErrorResult& aRv);
+  bool RequiresSlowFallbackForSubStringLength();
+  float GetSubStringLengthFastPath(nsIContent* aContent, uint32_t charnum,
+                                   uint32_t nchars, ErrorResult& aRv);
+  /**
+   * This fallback version of GetSubStringLength takes
+   * into account glyph positioning and requires us to have flushed layout
+   * before calling it. As per the SVG 2 spec, typically glyph
+   * positioning does not affect the results of getSubStringLength, but one
+   * exception is text in a textPath where we need to ignore characters that
+   * fall off the end of the textPath path.
+   */
+  float GetSubStringLengthSlowFallback(nsIContent* aContent, uint32_t charnum,
+                                       uint32_t nchars, ErrorResult& aRv);
+
   int32_t GetCharNumAtPosition(nsIContent* aContent,
                                const dom::DOMPointInit& aPoint);
 
@@ -296,10 +307,11 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
    * it.
    *
    * We have to do this in two cases: in response to a style change on a
-   * non-display <text>, where aReason will be eStyleChange (the common case),
-   * and also in response to glyphs changes on non-display <text> (i.e.,
-   * animated SVG-in-OpenType glyphs), in which case aReason will be eResize,
-   * since layout doesn't need to be recomputed.
+   * non-display <text>, where aReason will be
+   * IntrinsicDirty::FrameAncestorsAndDescendants (the common case), and also in
+   * response to glyphs changes on non-display <text> (i.e., animated
+   * SVG-in-OpenType glyphs), in which case aReason will be None, since layout
+   * doesn't need to be recomputed.
    */
   void ScheduleReflowSVGNonDisplayText(IntrinsicDirty aReason);
 
@@ -330,6 +342,14 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
   gfxRect TransformFrameRectFromTextChild(const nsRect& aRect,
                                           const nsIFrame* aChildFrame);
 
+  /** As above, but taking and returning a device px rect. */
+  Rect TransformFrameRectFromTextChild(const Rect& aRect,
+                                       const nsIFrame* aChildFrame);
+
+  /** As above, but with a single point */
+  Point TransformFramePointFromTextChild(const Point& aPoint,
+                                         const nsIFrame* aChildFrame);
+
   // Return our ::-moz-svg-text anonymous box.
   void AppendDirectlyOwnedAnonBoxes(nsTArray<OwnedAnonBox>& aResult) override;
 
@@ -343,6 +363,9 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
     explicit MutationObserver(SVGTextFrame* aFrame) : mFrame(aFrame) {
       MOZ_ASSERT(mFrame, "MutationObserver needs a non-null frame");
       mFrame->GetContent()->AddMutationObserver(this);
+      SetEnabledCallbacks(kCharacterDataChanged | kAttributeChanged |
+                          kContentAppended | kContentInserted |
+                          kContentWillBeRemoved);
     }
 
     // nsISupports
@@ -381,7 +404,7 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
    * Schedules mPositions to be recomputed and the covered region to be
    * updated.
    */
-  void NotifyGlyphMetricsChange();
+  void NotifyGlyphMetricsChange(bool aUpdateTextCorrespondence);
 
   /**
    * Recomputes mPositions by calling DoGlyphPositioning if this information
@@ -395,17 +418,6 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
    */
   void DoGlyphPositioning();
 
-  /**
-   * This fallback version of GetSubStringLength that flushes layout and takes
-   * into account glyph positioning.  As per the SVG 2 spec, typically glyph
-   * positioning does not affect the results of getSubStringLength, but one
-   * exception is text in a textPath where we need to ignore characters that
-   * fall off the end of the textPath path.
-   */
-  MOZ_CAN_RUN_SCRIPT
-  float GetSubStringLengthSlowFallback(nsIContent* aContent, uint32_t charnum,
-                                       uint32_t nchars, ErrorResult& aRv);
-
   /**
    * Converts the specified index into mPositions to an addressable
    * character index (as can be used with the SVG DOM text methods)
@@ -506,10 +518,12 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
    * nsTextFrame::DrawPathCallbacks rather than directly painting
    * the text frames.
    *
+   * @param aContextPaint Used by context-fill and context-stroke.
    * @param aShouldPaintSVGGlyphs (out) Whether SVG glyphs in the text
    *   should be painted.
    */
-  bool ShouldRenderAsPath(nsTextFrame* aFrame, bool& aShouldPaintSVGGlyphs);
+  bool ShouldRenderAsPath(nsTextFrame* aFrame, SVGContextPaint* aContextPaint,
+                          bool& aShouldPaintSVGGlyphs);
 
   // Methods to get information for a <textPath> frame.
   already_AddRefed<Path> GetTextPath(nsIFrame* aTextPathFrame);
@@ -529,7 +543,7 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
    *
    * mTrailingUndisplayedCharacters would be 2.
    */
-  uint32_t mTrailingUndisplayedCharacters;
+  uint32_t mTrailingUndisplayedCharacters = 0;
 
   /**
    * Computed position information for each DOM character within the <text>.
@@ -558,20 +572,20 @@ class SVGTextFrame final : public SVGDisplayContainerFrame {
    * screen.  (The "reasonable" range as determined by some #defines in
    * SVGTextFrame.cpp is 8..200.)
    */
-  float mFontSizeScaleFactor;
+  float mFontSizeScaleFactor = 1.0f;
 
   /**
    * The scale of the context that we last used to compute mFontSizeScaleFactor.
    * We record this so that we can tell when our scale transform has changed
    * enough to warrant reflowing the text.
    */
-  float mLastContextScale;
+  float mLastContextScale = 1.0f;
 
   /**
    * The amount that we need to scale each rendered run to account for
    * lengthAdjust="spacingAndGlyphs".
    */
-  float mLengthAdjustScaleFactor;
+  float mLengthAdjustScaleFactor = 1.0f;
 };
 
 }  // namespace mozilla
