@@ -85,80 +85,32 @@ a11y::AccType nsTableWrapperFrame::AccessibleType() {
 
 void nsTableWrapperFrame::Destroy(DestroyContext& aContext) {
   DestroyAbsoluteFrames(aContext);
-  mCaptionFrames.DestroyFrames(aContext);
   nsContainerFrame::Destroy(aContext);
 }
 
-const nsFrameList& nsTableWrapperFrame::GetChildList(
-    ChildListID aListID) const {
-  if (aListID == FrameChildListID::Caption) {
-    return mCaptionFrames;
-  }
-
-  return nsContainerFrame::GetChildList(aListID);
-}
-
-void nsTableWrapperFrame::GetChildLists(nsTArray<ChildList>* aLists) const {
-  nsContainerFrame::GetChildLists(aLists);
-  mCaptionFrames.AppendIfNonempty(aLists, FrameChildListID::Caption);
-}
-
-void nsTableWrapperFrame::SetInitialChildList(ChildListID aListID,
-                                              nsFrameList&& aChildList) {
-  if (FrameChildListID::Caption == aListID) {
-#ifdef DEBUG
-    nsIFrame::VerifyDirtyBitSet(aChildList);
-    for (nsIFrame* f : aChildList) {
-      MOZ_ASSERT(f->GetParent() == this, "Unexpected parent");
-    }
-#endif
-    // the frame constructor already checked for table-caption display type
-    MOZ_ASSERT(mCaptionFrames.IsEmpty(),
-               "already have child frames in CaptionList");
-    mCaptionFrames = std::move(aChildList);
-  } else {
-    MOZ_ASSERT(FrameChildListID::Principal != aListID ||
-                   (aChildList.FirstChild() &&
-                    aChildList.FirstChild() == aChildList.LastChild() &&
-                    aChildList.FirstChild()->IsTableFrame()),
-               "expected a single table frame in principal child list");
-    nsContainerFrame::SetInitialChildList(aListID, std::move(aChildList));
-  }
-}
-
 void nsTableWrapperFrame::AppendFrames(ChildListID aListID,
                                        nsFrameList&& aFrameList) {
   // We only have two child frames: the inner table and a caption frame.
   // The inner frame is provided when we're initialized, and it cannot change
-  MOZ_ASSERT(FrameChildListID::Caption == aListID, "unexpected child list");
+  MOZ_ASSERT(FrameChildListID::Principal == aListID, "unexpected child list");
   MOZ_ASSERT(aFrameList.IsEmpty() || aFrameList.FirstChild()->IsTableCaption(),
-             "appending non-caption frame to captionList");
-  mCaptionFrames.AppendFrames(nullptr, std::move(aFrameList));
-
-  // Reflow the new caption frame. It's already marked dirty, so
-  // just tell the pres shell.
-  PresShell()->FrameNeedsReflow(this, IntrinsicDirty::FrameAndAncestors,
-                                NS_FRAME_HAS_DIRTY_CHILDREN);
-  // The presence of caption frames makes us sort our display
-  // list differently, so mark us as changed for the new
-  // ordering.
+             "Why are we appending non-caption frames?");
+  nsContainerFrame::AppendFrames(aListID, std::move(aFrameList));
+  // The presence of caption frames makes us sort our display list differently,
+  // so mark us as changed for the new ordering.
   MarkNeedsDisplayItemRebuild();
 }
 
 void nsTableWrapperFrame::InsertFrames(
     ChildListID aListID, nsIFrame* aPrevFrame,
     const nsLineList::iterator* aPrevFrameLine, nsFrameList&& aFrameList) {
-  MOZ_ASSERT(FrameChildListID::Caption == aListID, "unexpected child list");
+  MOZ_ASSERT(FrameChildListID::Principal == aListID, "unexpected child list");
   MOZ_ASSERT(aFrameList.IsEmpty() || aFrameList.FirstChild()->IsTableCaption(),
-             "inserting non-caption frame into captionList");
+             "Why are we inserting non-caption frames?");
   MOZ_ASSERT(!aPrevFrame || aPrevFrame->GetParent() == this,
              "inserting after sibling frame with different parent");
-  mCaptionFrames.InsertFrames(nullptr, aPrevFrame, std::move(aFrameList));
-
-  // Reflow the new caption frame. It's already marked dirty, so
-  // just tell the pres shell.
-  PresShell()->FrameNeedsReflow(this, IntrinsicDirty::FrameAndAncestors,
-                                NS_FRAME_HAS_DIRTY_CHILDREN);
+  nsContainerFrame::InsertFrames(aListID, aPrevFrame, aPrevFrameLine,
+                                 std::move(aFrameList));
   MarkNeedsDisplayItemRebuild();
 }
 
@@ -167,13 +119,8 @@ void nsTableWrapperFrame::RemoveFrame(DestroyContext& aContext,
                                       nsIFrame* aOldFrame) {
   // We only have two child frames: the inner table and one caption frame.
   // The inner frame can't be removed so this should be the caption
-  MOZ_ASSERT(FrameChildListID::Caption == aListID, "can't remove inner frame");
-
-  // Remove the frame and destroy it
-  mCaptionFrames.DestroyFrame(aContext, aOldFrame);
-
-  PresShell()->FrameNeedsReflow(this, IntrinsicDirty::FrameAndAncestors,
-                                NS_FRAME_HAS_DIRTY_CHILDREN);
+  MOZ_ASSERT(aOldFrame->IsTableCaption(), "can't remove inner frame");
+  nsContainerFrame::RemoveFrame(aContext, aListID, aOldFrame);
   MarkNeedsDisplayItemRebuild();
 }
 
@@ -184,17 +131,27 @@ void nsTableWrapperFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
 
   // If there's no caption, take a short cut to avoid having to create
   // the special display list set and then sort it.
-  if (mCaptionFrames.IsEmpty()) {
-    BuildDisplayListForInnerTable(aBuilder, aLists);
+  if (nsIFrame* inner = mFrames.OnlyChild()) {
+    BuildDisplayListForChild(aBuilder, inner, aLists);
     DisplayOutline(aBuilder, aLists);
     return;
   }
 
-  nsDisplayListCollection set(aBuilder);
-  BuildDisplayListForInnerTable(aBuilder, set);
+  MOZ_ASSERT(mFrames.FirstChild());
+  MOZ_ASSERT(mFrames.FirstChild()->IsTableFrame());
 
+  nsDisplayListCollection set(aBuilder);
   nsDisplayListSet captionSet(set, set.BlockBorderBackgrounds());
-  BuildDisplayListForChild(aBuilder, mCaptionFrames.FirstChild(), captionSet);
+  for (auto* frame : mFrames) {
+    const bool isTable = frame->IsTableFrame();
+    auto& setForFrame = isTable ? set : captionSet;
+    BuildDisplayListForChild(aBuilder, frame, setForFrame);
+    if (!isTable) {
+      // FIXME(emilio, bug 144517): Historically we haven't displayed / laid
+      // out multiple captions. This preserves that behavior.
+      break;
+    }
+  }
 
   // Now we have to sort everything by content order, since the caption
   // may be somewhere inside the table.
@@ -211,18 +168,6 @@ void nsTableWrapperFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
   DisplayOutline(aBuilder, aLists);
 }
 
-void nsTableWrapperFrame::BuildDisplayListForInnerTable(
-    nsDisplayListBuilder* aBuilder, const nsDisplayListSet& aLists) {
-  // Just paint the regular children, but the children's background is our
-  // true background (there should only be one, the real table)
-  nsIFrame* kid = mFrames.FirstChild();
-  // The children should be in content order
-  while (kid) {
-    BuildDisplayListForChild(aBuilder, kid, aLists);
-    kid = kid->GetNextSibling();
-  }
-}
-
 ComputedStyle* nsTableWrapperFrame::GetParentComputedStyle(
     nsIFrame** aProviderFrame) const {
   // The table wrapper frame and the (inner) table frame split the style
@@ -261,12 +206,11 @@ nscoord nsTableWrapperFrame::IntrinsicISize(const IntrinsicSizeInput& aInput,
     iSize = std::max(iSize, innerTableMinISize);
   }
 
-  if (mCaptionFrames.NotEmpty()) {
+  if (nsIFrame* caption = GetCaption()) {
     // The table wrapper's intrinsic inline size should be as least as large as
     // caption's min inline size.
     const nscoord capMinISize = nsLayoutUtils::IntrinsicForContainer(
-        aInput.mContext, mCaptionFrames.FirstChild(),
-        IntrinsicISizeType::MinISize);
+        aInput.mContext, caption, IntrinsicISizeType::MinISize);
     iSize = std::max(iSize, capMinISize);
   }
   return iSize;
@@ -321,7 +265,7 @@ LogicalSize nsTableWrapperFrame::CaptionShrinkWrapSize(
     gfxContext* aRenderingContext, nsIFrame* aCaptionFrame, WritingMode aWM,
     const LogicalSize& aCBSize, nscoord aAvailableISize,
     ComputeSizeFlags aFlags) const {
-  MOZ_ASSERT(aCaptionFrame == mCaptionFrames.FirstChild());
+  MOZ_ASSERT(aCaptionFrame != mFrames.FirstChild());
 
   AutoMaybeDisableFontInflation an(aCaptionFrame);
 
@@ -444,8 +388,8 @@ LogicalSize nsTableWrapperFrame::ComputeAutoSize(
     return innerTableSize;
   }
   const LogicalSize captionSize =
-      CaptionShrinkWrapSize(aRenderingContext, mCaptionFrames.FirstChild(), aWM,
-                            aCBSize, innerTableSize.ISize(aWM), flags);
+      CaptionShrinkWrapSize(aRenderingContext, GetCaption(), aWM, aCBSize,
+                            innerTableSize.ISize(aWM), flags);
   const nscoord iSize =
       std::max(innerTableSize.ISize(aWM), captionSize.ISize(aWM));
   nscoord bSize = NS_UNCONSTRAINEDSIZE;
@@ -457,14 +401,14 @@ LogicalSize nsTableWrapperFrame::ComputeAutoSize(
 }
 
 Maybe<StyleCaptionSide> nsTableWrapperFrame::GetCaptionSide() const {
-  if (mCaptionFrames.IsEmpty()) {
+  if (!HasCaption()) {
     return Nothing();
   }
-  return Some(mCaptionFrames.FirstChild()->StyleTableBorder()->mCaptionSide);
+  return Some(GetCaption()->StyleTableBorder()->mCaptionSide);
 }
 
 StyleVerticalAlignKeyword nsTableWrapperFrame::GetCaptionVerticalAlign() const {
-  const auto& va = mCaptionFrames.FirstChild()->StyleDisplay()->mVerticalAlign;
+  const auto& va = GetCaption()->StyleDisplay()->mVerticalAlign;
   return va.IsKeyword() ? va.AsKeyword() : StyleVerticalAlignKeyword::Top;
 }
 
@@ -490,7 +434,7 @@ void nsTableWrapperFrame::GetCaptionOrigin(StyleCaptionSide aCaptionSide,
       (NS_UNCONSTRAINEDSIZE == aCaptionSize.BSize(aWM))) {
     return;
   }
-  if (mCaptionFrames.IsEmpty()) {
+  if (!HasCaption()) {
     return;
   }
 
@@ -630,7 +574,7 @@ void nsTableWrapperFrame::CreateReflowInputForCaption(
     nsPresContext* aPresContext, nsIFrame* aCaptionFrame,
     const ReflowInput& aOuterRI, Maybe<ReflowInput>& aChildRI,
     const nscoord aAvailISize) const {
-  MOZ_ASSERT(aCaptionFrame == mCaptionFrames.FirstChild());
+  MOZ_ASSERT(aCaptionFrame == GetCaption());
 
   const WritingMode wm = aCaptionFrame->GetWritingMode();
 
@@ -680,9 +624,8 @@ void nsTableWrapperFrame::ReflowChild(nsPresContext* aPresContext,
 
 void nsTableWrapperFrame::UpdateOverflowAreas(ReflowOutput& aMet) {
   aMet.SetOverflowAreasToDesiredBounds();
-  ConsiderChildOverflow(aMet.mOverflowAreas, InnerTableFrame());
-  if (mCaptionFrames.NotEmpty()) {
-    ConsiderChildOverflow(aMet.mOverflowAreas, mCaptionFrames.FirstChild());
+  for (auto* frame : mFrames) {
+    ConsiderChildOverflow(aMet.mOverflowAreas, frame);
   }
 }
 
@@ -709,11 +652,10 @@ void nsTableWrapperFrame::Reflow(nsPresContext* aPresContext,
   nsRect origCaptionRect;
   nsRect origCaptionInkOverflow;
   bool captionFirstReflow = false;
-  if (mCaptionFrames.NotEmpty()) {
-    origCaptionRect = mCaptionFrames.FirstChild()->GetRect();
-    origCaptionInkOverflow = mCaptionFrames.FirstChild()->InkOverflowRect();
-    captionFirstReflow =
-        mCaptionFrames.FirstChild()->HasAnyStateBits(NS_FRAME_FIRST_REFLOW);
+  if (nsIFrame* caption = GetCaption()) {
+    origCaptionRect = caption->GetRect();
+    origCaptionInkOverflow = caption->InkOverflowRect();
+    captionFirstReflow = caption->HasAnyStateBits(NS_FRAME_FIRST_REFLOW);
   }
 
   // ComputeAutoSize has to match this logic.
@@ -721,7 +663,7 @@ void nsTableWrapperFrame::Reflow(nsPresContext* aPresContext,
   Maybe<StyleCaptionSide> captionSide = GetCaptionSide();
   const nscoord contentBoxISize = aOuterRI.ComputedSize(wm).ISize(wm);
 
-  MOZ_ASSERT(mCaptionFrames.NotEmpty() == captionSide.isSome());
+  MOZ_ASSERT(HasCaption() == captionSide.isSome());
 
   // Compute the table's size first, and then prevent the caption from
   // being larger in the inline dir unless it has to be.
@@ -745,14 +687,13 @@ void nsTableWrapperFrame::Reflow(nsPresContext* aPresContext,
     // advantage of that later when we call GetCaptionOrigin, though.)
     nscoord innerBorderISize =
         innerRI->ComputedSizeWithBorderPadding(wm).ISize(wm);
-    CreateReflowInputForCaption(aPresContext, mCaptionFrames.FirstChild(),
-                                aOuterRI, captionRI, innerBorderISize);
+    CreateReflowInputForCaption(aPresContext, GetCaption(), aOuterRI, captionRI,
+                                innerBorderISize);
 
     // We intentionally don't merge capStatus into aStatus, since we currently
     // can't handle caption continuations, but we probably should.
     nsReflowStatus capStatus;
-    ReflowChild(aPresContext, mCaptionFrames.FirstChild(), *captionRI,
-                captionMet, capStatus);
+    ReflowChild(aPresContext, GetCaption(), *captionRI, captionMet, capStatus);
     captionSize = captionMet.Size(wm);
     captionMargin = captionRI->ComputedLogicalMargin(wm);
     nscoord bSizeOccupiedByCaption =
@@ -787,13 +728,13 @@ void nsTableWrapperFrame::Reflow(nsPresContext* aPresContext,
   aDesiredSize.SetSize(wm, desiredSize);
   nsSize containerSize = aDesiredSize.PhysicalSize();
 
-  MOZ_ASSERT(mCaptionFrames.NotEmpty() == captionSide.isSome());
-  if (mCaptionFrames.NotEmpty()) {
+  MOZ_ASSERT(HasCaption() == captionSide.isSome());
+  if (nsIFrame* caption = GetCaption()) {
     LogicalPoint captionOrigin(wm);
     GetCaptionOrigin(*captionSide, innerSize, captionSize, captionMargin,
                      captionOrigin, wm);
-    FinishReflowChild(mCaptionFrames.FirstChild(), aPresContext, captionMet,
-                      captionRI.ptr(), wm, captionOrigin, containerSize,
+    FinishReflowChild(caption, aPresContext, captionMet, captionRI.ptr(), wm,
+                      captionOrigin, containerSize,
                       ReflowChildFlags::ApplyRelativePositioning);
     captionRI.reset();
   }
@@ -808,9 +749,9 @@ void nsTableWrapperFrame::Reflow(nsPresContext* aPresContext,
                     wm, innerOrigin, containerSize, ReflowChildFlags::Default);
   innerRI.reset();
 
-  if (mCaptionFrames.NotEmpty()) {
-    nsTableFrame::InvalidateTableFrame(mCaptionFrames.FirstChild(),
-                                       origCaptionRect, origCaptionInkOverflow,
+  if (HasCaption()) {
+    nsTableFrame::InvalidateTableFrame(GetCaption(), origCaptionRect,
+                                       origCaptionInkOverflow,
                                        captionFirstReflow);
   }
 
